/*
 * Class    br.org.scadabr.luciol.Application
 * File     Application.java
 * Author   Alexis BIETTI
 *
 * This class defines the requirements for making application sessions
 * configurable, with a common COMMIT / ROLL BACK mechanism.
 */
package br.org.scadabr.luciol;


/**
 * <p>
 * All applications in Luciol MUST implement this.
 * </p>
 *
 * <p>
 * Applications have two configurations :
 *
 * <ul>
 * <li>
 * An ACTIVE configuration that is used by the runtime for execution.
 * </li>
 * <li>
 * A PASSIVE configuration that is used for configuration only.
 * </li>
 * </ul>
 * </p>
 *
 * <p>
 * To do !!!!!!!!!! <br>
 * Applications should provide an interface for their current status and a
 * log of error messages.
 * </p>
 *
 * @author <a href="mailto:alexis.bietti@sysaware.com"> Alexis BIETTI </a>
 */
public interface AppSession {
  /*
 * Status codes
 */
  public static final byte STATUS_NOT_RUNNING        = 0x01;
  public static final byte STATUS_BAD_CONFIGURATION  = 0x02;
  public static final byte STATUS_PHYSICAL_LINK_DEAD = 0x04;
  public static final byte STATUS_INTERNAL_ERROR     = 0x08;

  /**
   * public Vector errorMessages(); // Messages in human language public byte
   * getStatus(); // 0 means 'running, nothing to say'
   */
  /**
   * <p>
   * Return the current active configuration of this application.
   * </p>
   */
  public Configurable activeConfiguration();

  /**
   * <p>
   * Return the current passive configuration of this application.
   * </p>
   */
  public Configurable passiveConfiguration();

  /**
   * <p>
   * Commit the configuration.
   * </p>
   *
   * <p>
   * The general contract of COMMIT is to copy the passive configuration into
   * the active configuration.
   * </p>
   */
  public void commit();

  /**
   * <p>
   * Roll back the configuration.
   * </p>
   *
   * <p>
   * The general contract of ROLL BACK is to copy the active configuration
   * into the passive configuration.
   * </p>
   */
  public void rollBack();

  /**
   * For a later version...
   */
  /**
   * <p>
   * Return some flags indicating the state of this session.
   * </p>
   *
   * @see #STATUS_NOT_RUNNING
   * @see #STATUS_BAD_CONFIGURATION
   * @see #STATUS_PHYSICAL_LINK_DEAD
   * @see #STATUS_INTERNAL_ERROR
   */
  /**
   * public void byte getFlags();
   */
}


// End AppSession.java
