/*
 * Digital input data element class.
 *
 * Class     br.org.scadabr.luciol.database.DigitalInput
 * File      DigitalInput.java
 * Author    Alexis BIETTI <alexis.bietti@sysaware.com>
 * (c)       SysAware <http://www.sysaware.com>
 */

//-----------------------------------------------------------------------------
// Imports
//-----------------------------------------------------------------------------
package br.org.scadabr.luciol.database;

import br.org.scadabr.luciol.ihm.Labels;
import br.org.scadabr.protocol.lib.xml.KXMLElement;

//-----------------------------------------------------------------------------
// Class
//-----------------------------------------------------------------------------

/**
 * Digital Input class.
 *
 * @author <a href="mailto:alexis.bietti@sysaware.com">Alexis BIETTI</a>
 */
public class DigitalInput extends DataElement {
  //---------------------------------------------------------------------------
  // Attributes
  //---------------------------------------------------------------------------

  /** Logic of this Digital Input. */
  public boolean logic = true;

  //---------------------------------------------------------------------------
  // Constructor
  //---------------------------------------------------------------------------

  /**
       *
       */
  public DigitalInput() {
    super();

    Record rec = instanciateRecord();
    rec.setValue(DEFAULT_DIGITAL);
    rec.quality = Record.Q_INVALID | Record.Q_FORCED;
    writeNewRecord(rec);
  }

  /**
   * Number of samples needed before trusting a measure.
   */

  // public short debounce = 5;

  /**
   * For use with Double inputs.
   */

  // protected boolean invalidDoubleInput = false;
  public static void main(String[] args) {
    DigitalInput d = new DigitalInput();

    try {
      Thread.sleep(100);
    } catch (Exception e) {
    }

    Record r = d.instanciateRecord();
    r.setValue(true);
    d.writeNewRecord(r);

    for (java.util.Enumeration e = d.allRecords(); e.hasMoreElements();) {
      System.out.println(e.nextElement());
    }
  }

  //---------------------------------------------------------------------------
  // Methods
  //---------------------------------------------------------------------------
  public String getName() {
    return Labels.DIGITAL_INPUT;
  }

  /**
   * DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public Object defaultValue() {
    return DEFAULT_DIGITAL;
  }

  /**
   * DOCUMENT ME!
   */
  public void setInvalid() {
    Record rec = instanciateRecord();
    rec.setValue(DEFAULT_DIGITAL);
    rec.quality = Record.Q_INVALID | Record.Q_FORCED;
    writeNewRecord(rec);
  }

  /**
   * DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public int getDefaultLogSize() {
    return 4;
  }

  /**
       *
       */
  public String servlet() {
    return "digin";
  }

  /**
   * <p>
   * See the general contract of the method <tt>instanciateRecord()</tt> in
   * the class DataElement.
   * </p>
   *
   * <p>
   * Return a Record compatible with the DigitalInput class.
   * </p>
   *
   * @return a Record compatible with the DigitalInput class.
   */
  public Record instanciateRecord() {
    return Record.instanciateRecord(Record.REC_DIGITAL);
  }

  //---------------------------------------------------------------------------
  // Interface XMLInterface
  //---------------------------------------------------------------------------

  /**
   * See the general contract of this method in the interface {@link
   * br.org.scadabr.io.XMLInterface}
   */
  public void inputFromXML(KXMLElement xml) throws IllegalArgumentException {
    try {
      logic = xml.getBooleanProperty(XML_LOGIC, true);
    } catch (Exception e) {
      throw ILLEGAL_ARGUMENT_EXCEPTION;
    }

    super.inputFromXML(xml);
  }

  /**
   * See the general contract of this method in the interface {@link
   * br.org.scadabr.io.XMLInterface}
   */
  public KXMLElement outputToXML() {
    KXMLElement root = super.outputToXML();

    root.setTagName(XML_DIGITAL_INPUT);
    root.addProperty(XML_LOGIC, logic ? "1" : "0");

    // root.addProperty("debounce", debounce);
    return root;
  }
}
