package br.org.scadabr.luciol.database;


/**
 * <p>
 * All Input/Output interfaces should implement this.
 * </p>
 *
 * <p>
 *
 * <ul>
 * <li>
 * Protocol applications
 * </li>
 * <li>
 * Luciol I/O cards application
 * </li>
 * <li>
 * Applications that calculate virtual Input/Outputs.
 * </li>
 * </ul>
 * </p>
 */
public interface IOInterface {
  //---------------------------------------------------------------------------
  // Constants
  //---------------------------------------------------------------------------
  public static final byte READ       = 0x1;
  public static final byte WRITE      = 0x2;
  public static final byte READ_WRITE = READ | WRITE;

  //---------------------------------------------------------------------------
  // Methods
  //---------------------------------------------------------------------------

  /**
   * <p>
   * Get the name of the device interfaced by this object.
   * </p>
   *
   * <p>
   * The name "Luciol" is reserved for the interface with Luciol I/O cards.
   * </p>
   *
   * @return the name of the device.
   */
  public abstract String deviceName();

  /**
   * <p>
   * Get the name of the protocol used to communicate with the device.
   * </p>
   *
   * @return the name of the protocol.
   */
  public abstract String protocolName();

  /**
   * <p>
   * Read a value from this interface. The value is identified with its
   * database ID.
   * </p>
   *
   * @return a Record Object.
   */
  public abstract Record read(int dataBaseID);

  /**
   * <p>
   * Gives the order to the interface to write a new value to the specified
   * data.
   * </p>
   *
   * @param dataBaseID a database wide identifier.
   * @param value a Value object.
   */
  public abstract void write(int dataBaseID, Record value);

  /**
   * Test if this IOInterface owns the Element identified by the parameter.
   *
   * @param dataBaseID a database wide identifier.
   *
   * @return <tt>true</tt> if this IOInterface owns the Element  identified by
   *         the parameter.
   */
  public abstract boolean ownsThisElement(int dataBaseID);
}
