package br.org.scadabr.luciol.database.templates;

import br.org.scadabr.luciol.database.AnalogInput;
import br.org.scadabr.luciol.database.Database;
import br.org.scadabr.luciol.database.DigitalInput;
import br.org.scadabr.luciol.database.DigitalOutput;
import br.org.scadabr.luciol.database.DoubleInput;
import br.org.scadabr.luciol.database.DoubleOutput;
import br.org.scadabr.luciol.database.IntegerInput;

/**
 * Utility for creating XML templates of database configuration: Basic RTU
 * model.
 *
 * @author Alexis BIETTI
 * @version 1.0
 */
public class BasicRTU {
  /**
   * Standard constructor.
   */
  public BasicRTU() {
  }

  /**
   * <p>
   * Construct a configuration file for the Luciol database with the given
   * options.
   * </p>
   *
   * <p>
   * Usage: BasicRTU ai acci si dbsi bo dco
   * </p>
   *
   * <ul>
   * <li>
   * ai:   analog inputs
   * </li>
   * <li>
   * acci: accumulator inputs
   * </li>
   * <li>
   * si:   status inputs
   * </li>
   * <li>
   * dbsi: double bit status inputs
   * </li>
   * <li>
   * bo:   binary outputs
   * </li>
   * <li>
   * dco:  double control outputs
   * </li>
   * </ul>
   */
  public static void main(String[] args) {
    try {
      int      analogInputs           = Integer.parseInt(args[0]);
      int      accumulatorInputs      = Integer.parseInt(args[1]);
      int      statusInput            = Integer.parseInt(args[2]);
      int      doubleBitStatusInputs  = Integer.parseInt(args[3]);
      int      binaryTripCloseOutputs = Integer.parseInt(args[4]);
      int      doubleControlOutputs   = Integer.parseInt(args[5]);

      BasicRTU basicRTU1 = new BasicRTU();
      basicRTU1.templateRTU(
        analogInputs, accumulatorInputs, statusInput, doubleBitStatusInputs,
        binaryTripCloseOutputs, doubleControlOutputs);
    } catch (Exception e) {
      System.out.println("Usage: BasicRTU ai acci si dbsi bo dco");
      System.out.println("  ai:   analog inputs");
      System.out.println("  acci: accumulator inputs");
      System.out.println("  si:   status inputs");
      System.out.println("  dbsi: double bit status inputs");
      System.out.println("  bo:   binary outputs");
      System.out.println("  dco:  double control outputs");
    }
  }

  /**
   * <p>
   * Create a configuration that have everything of an RTU and display it as
   * an XML file.
   * </p>
   */
  private void templateRTU(
    int analogInputs, int accumulatorInputs, int statusInput,
    int doubleBitStatusInputs, int binaryTripCloseOutputs,
    int doubleControlOutputs) {
    // Current unique database identifier
    int      id = 0;

    Database db = new Database();

    // Analog inputs
    for (int i = 0; i < analogInputs; ++i) {
      AnalogInput ai = new AnalogInput();
      ai.databaseID    = id++;
      ai.description   = "Analog Input #" + (i + 1);

      db.addElement(ai);
    }

    // Accumulator inputs
    for (int i = 0; i < accumulatorInputs; ++i) {
      IntegerInput ii = new IntegerInput();
      ii.databaseID    = id++;
      ii.description   = "Accumulator Input #" + (i + 1);

      db.addElement(ii);
    }

    // Status inputs
    for (int i = 0; i < statusInput; ++i) {
      DigitalInput di = new DigitalInput();
      di.databaseID    = id++;
      di.description   = "Digital Input #" + (i + 1);

      db.addElement(di);
    }

    // Double inputs
    for (int i = 0; i < doubleBitStatusInputs; ++i) {
      DoubleInput di = new DoubleInput();
      di.databaseID    = id++;
      di.description   = "Double Input #" + (i + 1);

      db.addElement(di);
    }

    // Binary outputs
    for (int i = 0; i < binaryTripCloseOutputs; ++i) {
      DigitalOutput dout = new DigitalOutput();
      dout.databaseID    = id++;
      dout.description   = "Digital Output #" + (i + 1);

      db.addElement(dout);
    }

    // Double outputs
    for (int i = 0; i < doubleControlOutputs; ++i) {
      DoubleOutput dout = new DoubleOutput();
      dout.databaseID    = id++;
      dout.description   = "Digital Output #" + (i + 1);

      db.addElement(dout);
    }

    // Other things
    System.out.println(db.outputToXML());
  }
}


// End BasicRTU.java
