/*
 * Class	br.org.scadabr.protocol.iec101.common.AppService
 * File		AppService.java
 * Author	lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;


/**
 * <p>
 * This class defines general application procedure.
 * <p>
 * AppService is handled by application layer and manager. It can be triggered
 * in an event way by database notification, must be able to build an asdu and
 * related link service in order to construct a compliant frame to send to a 
 * remote.
 * <p>
 * REMEMBER AppService ONLY CONCERNS APPLICATION LAYER. AT LINK LAYER LEVEL, WE
 * USE LnkService OBJECTS.
 * 
 * @author	lionnel cauvy
 * @version	1.0
 * @date	28.II.03
 */
public class AppService
{
	public final static int		NOTUSED		= -1;
	/**
	 * <p>
	 * Database locator to enable launching of this procedure by
	 * external process via a database notification. For each
	 * procedure, application MAY record a database event listener.
	 * Notice that it can be null for internal use.
	 */
	public DbLocator notifier;
	/**
	 * <p>
	 * The application LnkService Data Unit to build. If null, that means
	 * that procedure only contain a link service.
	 */
	public MapAsdu asdu;
	/**
	 * <p>
	 * The link service associated with this procedure.
	 */
	public LnkService svc;
	/**
	 * <p>
	 * Execution context. This is used to give some current values possibly
	 * needed to build an asdu.
	 */
	public Context context;
	/**
	 * <p>
	 * Instanciates a procedure object with an asdu to build.
	 *
	 * @param asdu			The asdu to send to a remote.
	 * @param svc			The related link service.
	 * @param notifier		The database element that will trigger the procedure.
	 * @param context		The execution context. Give current values to update during asdu building.
	 */
	public AppService(MapAsdu asdu, LnkService svc, DbLocator notifier, Context context)
	{
		this.asdu		= asdu;
		this.svc		= svc;
		this.notifier	= notifier;
		this.context	= context;
	}	
}	// end AppService.java