/*
 * Class    br.org.scadabr.protocol.iec101.common.ApplicationLayer
 * File     ApplicationLayer.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

/**
 * <p>
 * This class defines an implementation of IEC 608705 application layer from EPA
 * model.<br>
 * Application layer is in charge of:<br>
 * - application service data units (iec 60870-5-3).<br>
 * - information elements (iec 60870-5-4).
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public abstract class ApplicationLayer extends LayerThread {
	/**
	 * Asdu writer reference. A factory to code asdu to send.
	 */
	public AsduWriter writer;
	/**
	 * Asdu reader reference. A factory to decode received asdu.
	 */
	public AsduReader reader;

	/**
	 * Event raised by upper layer. Application layer is requested for a
	 * service. It must inform remote image, then build the related asdu and
	 * last request lower link layer to work.
	 * 
	 * @param object
	 *            data from upper layer.
	 */
	protected synchronized void upperEvent(Object object)
			throws IEC608705Exception {
		write(writer.build((AppService) object));
	}

	/**
	 * <p>
	 * Allocates an application layer. Such an object handles asdus. Read from
	 * lower link layer and to write to this lower layer.
	 */
	protected ApplicationLayer() {
		super("ApplicationLayer");
	}
} // end ApplicationLayer.java