/*
 * Class    br.org.scadabr.protocol.iec101.AsduLocator;
 * File     AsduLocator.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

/**
 * <p>
 * To write & read some data in luciol database, an application needs
 * to locate whatever value hold by an asdu. An asdu is composed of
 * information objects, each of them is composed of information elements,
 * and those last can contain several interesting VALUES.<br>
 * So an iec application uses three different informations to exactly
 * determine which is the value to read/write from/to the database.
 * This class defines the way to locate value into an asdu.
 * <p>
 * First, 'pio' locates index of Information Object into a sequence,
 * <p>
 * then, 'pie' locates Information Element into an Information Object,
 * <p>
 * and last, 'pval' locates the value element into Information Element.
 * <p>
 * Notice this mechanism is based on this implementation asdus framework.
 * (See the 'octetset' & 'dataset' arrays in Information Element implementation).
 * <p>
 * An instanciated object from this class is always part of a mapping asdu
 * element.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class AsduLocator
{
	/**
	 * Position of information object from a sequence of io into asdu.<br>
	 * From zero value.
	 */
	public int pio;
	/**
	 * Position of information element from a sequence of ie into io.<br>
	 * From zero value.
	 */
	public short pie;
	/**
	 * Position of value into ie.<br>
	 * From zero value.
	 */
	public short pval;
	/**
	 * <p>
	 * Instanciates a value locator for an asdu.
	 *
	 * @param pio		Position of Information Object into sequence of io.
	 * @param pie		Position of Information Element into information object.
	 * @param pval		Position of VALUE into information elt.
	 */	
	public AsduLocator(int pio, short pie, short pval)
	{
		this.pio = pio;
		this.pie = pie;
		this.pval = pval;
	}	
}	// end AsduLocator.java