/*
 * Class    br.org.scadabr.protocol.iec101.common.AsduWriter
 * File     AsduWriter.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;


/**
 * <p>
 * This interface defines backbone of an asdu writer factory. Role of
 * implementer will be to build a link service from an application procedure.
 * <p>
 * Notice that at link layer level, this link service will be translated
 * into an Application LnkService Data Unit (ASDU).
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.III.03
 */
public interface AsduWriter
{	
	/**
	 * Build method to construct an 'asdu' to write eg an asdu to send on
	 * the network. It will not be really an ASDU object but some data 
	 * lower link layer will understand. We use link services objects to
	 * perform this.
	 *
	 * @param procedure	An application procedure to turn into an asdu.
	 *
	 * @return a link service planed to be handled by link layer.
	 */
	public abstract LnkService build(AppService procedure);
	
};	// end AsduWriter.java