/*
 * Class    br.org.scadabr.protocol.iec101.common.BinaryTranslator
 * File     BinaryThreshold.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import br.org.scadabr.protocol.lib.math.Float;
import br.org.scadabr.protocol.lib.math.Math32FP;

/**
 * Translation rule is boolean cast.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class BinaryTranslator implements Translator
{	
	/**
	 * Upper bound threshold type. Object value must be inferior.
	 */
	public static final byte UPPERBOUND	= 0x01;

	/**
	 * Lower bound threshold type. Object value must be superior.
	 */
	public static final byte LOWERBOUND	= 0x02;

	/**
	 * Equality threshold type. Object value must be equal.
	 */
	public static final byte EQUALITY	= 0x03;

	/**
	 * Binary threshold type.
	 */	
	public byte type;

	/**
	 * Binary threshold value.
	 */
	public float threshold;

	/**
	 * Binary threshold logic.
	 */
	public boolean logic;


	/**
	 * Internal method for UPPERBOUND threshold.
	 * 
	 * @param obj	object to transform
	 *
	 * @return computed Boolean
	 */
	private Boolean sup(Object obj)
	{
		if (obj instanceof Integer)
		{
			if( ((Integer)obj).intValue() < threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Short)
		{
			if( ((Short)obj).shortValue() < threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Byte)
		{
			if( ((Byte)obj).byteValue() < threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Math32FP)
		{
			if( ((Math32FP)obj).floatValue() < threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Float)
		{
			if( ((Float)obj).floatValue() < threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
		
		else
			 return null;
	}

	/**
	 * Internal method for LOWERBOUND threshold.
	 * 
	 * @param obj	object to transform
	 *
	 * @return computed Boolean
	 */
	private Boolean inf(Object obj)
	{
		if (obj instanceof Integer)
		{
			if( ((Integer)obj).intValue() > threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Short)
		{
			if( ((Short)obj).shortValue() > threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Byte)
		{
			if( ((Byte)obj).byteValue() > threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Math32FP)
		{
			if( ((Math32FP)obj).floatValue() > threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Float)
		{
			if( ((Float)obj).floatValue() > threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
		
		else
			 return null;
	}

	/**
	 * Internal method for EQUALITY threshold.
	 * 
	 * @param obj	object to transform
	 *
	 * @return computed Boolean
	 */
	private Boolean val(Object obj)
	{
		if (obj instanceof Integer)
		{
			if( ((Integer)obj).intValue() == threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Short)
		{
			if( ((Short)obj).shortValue() == threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Byte)
		{
			if( ((Byte)obj).byteValue() == threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Math32FP)
		{
			if( ((Math32FP)obj).floatValue()== threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
				
		else if (obj instanceof Float)
		{
			if( ((Float)obj).floatValue() == threshold)
				return new Boolean(logic);
			else
				return new Boolean(!logic);
		}
		
		else
			 return null;
	}
	
	/**
	 * Applies a binary modification operation on a data element
	 * from iec information element.
	 *
	 * @param obj	the object to transform
	 *
	 * @return a Boolean object
	 */
	public Object apply(Object obj)
	{
		switch (type)
		{
		case UPPERBOUND:
			return sup(obj);
		case LOWERBOUND:
			return inf(obj);
		case EQUALITY:
			return val(obj);
		default:
			return obj;
		}
	}

	/**
	 * <p>
	 * allocates a binary translator object.
	 *
	 * @param type		type UPPERBOUND,LOWERBOUND or EQUALITY
	 * @param threshold	threshold value
	 * @param logic		true:+ false:-
	 */	
	public BinaryTranslator(byte type, float threshold, boolean logic)
	{
		this.type = type;
		this.threshold = threshold;
		this.logic = logic;
	}
};	// end BinaryTranslator.java