/*
 * Class    br.org.scadabr.protocol.iec101.common.FrameFactory
 * File     AbstractFrameFactory.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

/**
 * <p>
 * This class defines super class for frame builders.
 * <p>
 * Frame construction takes place at link layer level.
 * A frame factory can be a reader one for data from 
 * physical layer to send to upper application layer.
 * Or it can be a writer one for data from application
 * layer to send to lower physical layer.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
 
public abstract class  FrameFactory
{	
	/**
	 * control field construction MASK_PRM
	 * mask to evaluate PRM in control field
	 */
	public static final byte		MASK_PRM				= 0x40;
	
	/**
	 * control field construction MASK_FCB_ACD
	 * mask to evaluate FCB or ACD (according to PRM) in control field
	 */
	public static final byte		MASK_FCB_ACD			= 0x20;
	
	/**
	 * control field construction MASK_FCV_DFC
	 * mask to evaluate FCV or DFC (according to PRM) in control field
	 */
	public static final byte		MASK_FCV_DFC			= 0x10;
	
	/**
	 * control field construction MASK_FCT
	 * mask to evaluate function code (according to PRM) in control field
	 */
	public static final byte		MASK_FCT				= 0x0F;
	
	/**
	 * control field construction POS_PRM
	 * PRM position in control field
	 */
	public static final byte		POS_PRM					= 0x06;
	
	/**
	 * control field construction POS_FCB_ACD
	 * FCB or ACD position in control field
	 */
	public static final byte		POS_FCB_ACD				= 0x05;
	
	/**
	 * control field construction POS_FCV_DFC
	 * FCV or DFC position in control field
	 */
	public static final byte		POS_FCV_DFC				= 0x04;

	
	/**
	 * Frame object reference.
	 */
	protected static Frame frame;

}	// end FrameFactory.java