/*
 * Class    br.org.scadabr.protocol.iec101.common.Interoperability
 * File     Interoperability.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

/**
 * <p>
 * This interface defines general iec608705 settings for FT1.2 frame headers
 * and link services.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public interface Interoperability
{
	/**
	 * broadcast iec60870-5-10X address
	 */
	public static final short	BROADCAST								= 0xFF;
	/**
	 * start byte for a FT1.2 frame with fixed length
	 */
	public static final short	STARTBYTEFIXEDLENGTH					= 0x10;
	/**
	 * start byte for a FT1.2 frame with variable length
	 */
	public static final short	STARTBYTEVARIABLELENGTH					= 0x68;
	/**
	 * stop byte for a FT1.2 frame
	 */
	public static final short	STOPBYTE								= 0x16;
	/**
	 * single character frame
	 */
	public static final short	SINGLEBYTE								= 0xE5;
	public static final short	MAXFRAMELENGTH							= 0xFF;
	public static final short	FIXEDFRAMELENGTH						= 0x02;
	public static final byte	ADDRESSFIELDLENGTH						= 1;
	public static final byte	VARIABLE_FRAME_HEADER_SIZE				= 6;
	public static final byte	FIXED_FRAME_HEADER_SIZE					= 3;


	public static final byte	SERVICE_S0_INACTIVE						= 0x00;

	// local master
	public static final byte	SERVICE_S2_SC_RESET_REMOTE_LINK			= 0x00;
	public static final byte	SERVICE_S2_SC_RESET_USER_PROCESS		= 0x01;//T101 only
	public static final byte	SERVICE_S2_SC_RESET_TEST_FOR_LINK		= 0x02;//T101 only
	public static final byte	SERVICE_S2_SC_USER_DATA					= 0x03;
	public static final byte	SERVICE_S1_SN_USER_DATA					= 0x04;
	public static final byte	SERVICE_S2_SC_RESET_FCB					= 0x07;//T103 only
	public static final byte	SERVICE_S3_RR_ACCESS_DEMAND				= 0x08;//T101 only
	public static final byte	SERVICE_S3_RR_LINK_STATUS				= 0x09;
	public static final byte	SERVICE_S3_RR_USER_DATA_CLASS_1			= 0x0A;
	public static final byte	SERVICE_S3_RR_USER_DATA_CLASS_2			= 0x0B;

	// local slave
	public static final byte	SERVICE_S2_CONFIRM_ACK					= 0x00;
	public static final byte	SERVICE_S2_CONFIRM_NACK					= 0x01;
	public static final byte	SERVICE_S3_RESPOND_USER_DATA			= 0x08;
	public static final byte	SERVICE_S3_RESPOND_NACK					= 0x09;
	public static final byte	SERVICE_S3_RESPOND_LINK_STATUS_OR_ACD	= 0x0B;
	public static final byte	SERVICE_S3_XX_LINK_NOT_FUNCTIONING		= 0x0E;
	public static final byte	SERVICE_S3_XX_LINK_NOT_IMPLEMENTED		= 0x0F;

};	// end Interoperability.java