/*
 * Class    br.org.scadabr.protocol.iec101.common.LinearTranslator
 * File     LinearTranslator.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import br.org.scadabr.protocol.lib.math.Float;
import br.org.scadabr.protocol.lib.math.Math32FP;

/**
 * Translation rule is linear equation 'ax+b'.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class LinearTranslator implements Translator
{	
	/**
	 * Rate mutiplier a from linear equation 'ax+b'.
	 */
	public int rate;

	/**
	 * Offset from linear equation 'ax+b'.
	 */
	public int offset;

	/**
	 * Applies a linear equation 'ax+b' transformation
	 * on a data element from iec information element.
	 *
	 * @param obj	the object to transform
	 *
	 * @return a Boolean object
	 */	
	public Object apply(Object obj)
	{
		Object object = null;
		if (obj instanceof Integer)
		{
			int res = (rate * ((Integer)obj).intValue()) + offset;
			object = new Integer(res);
		}
		else if (obj instanceof Byte)
		{
			int res = (rate * ((Byte)obj).byteValue()) + offset;
			object = new Integer(res);
		}
		else if (obj instanceof Short)
		{
			int res = (rate * ((Short)obj).shortValue()) + offset;
			object = new Integer(res);
		}
		else if (obj instanceof Float)
		{
			float res = (rate * ((Float)obj).floatValue()) + offset;
			object = new Float(res);
		}
		else if (obj instanceof Math32FP)
		{
			float res = (rate * ((Math32FP)obj).floatValue()) + offset;
			object = new Float(res);
		}
		else
			object = obj;
			
		return object;
	}

	/**
	 * <p>
	 * Applies a linear translation to an Object value.
	 *
	 * @param rate		'a' from 'ax+b'
	 * @param offset	'b'
	 */	
	public LinearTranslator(int rate, int offset)
	{
		this.rate = rate;
		this.offset = offset;
	}
};	// end LinearTranslator.java