/*
 * Class    br.org.scadabr.protocol.iec101.common.LinkLayer
 * File     LinkLayer.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import br.org.scadabr.protocol.iec101.common.*;


/**
 * <p>
 * This class defines used implementation of IEC 608705 link layer from EPA model.<br>
 * Link layer from iec60870 has to:<br>
 * - check transmission frame format (IEC 60870-5-1).<br>
 * - provide transmission procedures (IEC 60870-5-2).<br>
 * <p>
 * Notice that FT1.2 format compliance verification has been moved to lower physical layel
 * for convenience and reasons of performance.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public abstract class LinkLayer extends LayerThread
{
	/**
	 *
	 */
	public boolean mode;

	/**
	 *
	 */	
	protected void buildAndWrite(Object service) throws IEC608705Exception
	{
		Frame frame = FrameWriter.build(service);
		if (frame==null)
		{
			CodeProperties.lnklog("Receive a null frame from upper application layer.");
			return;
		}
		write(FrameConverter.convert(frame));
		frame = null;
	}

		
	/**
	 * <p>
	 * Allocates a link layer.
	 */
	public LinkLayer()
	{
		super("LinkLayer");
	}

}	// end LinkLayer.java



