/*
 * Class    br.org.scadabr.protocol.iec101.common.LnkManager
 * File     LnkManager.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import br.org.scadabr.protocol.iec101.common.*;

/**
 * <p>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public abstract class LnkManager
{
	/**
	 * Final static flag for a lazy link service.
	 */
	public static final boolean LAZY	= false;
	/**
	 * Final static flag for a workin link service.
	 */
	public static final boolean WORK	= true;
	/**
	 * Related application manager. Related means it works
	 * for the same remote station.
	 */
	public AppManager app;
	/**
	 * Link layer reference.
	 */
	public LinkLayer layer;
	/**
	 * Flag LAZY/WORK of this manager.
	 */
	public boolean state;
	/**
	 * The current link service.
	 */
	public LnkService current;
	/**
	 * Access Demand bit hold by frame control field.
	 * ACD is raised (true/1) when class 1 data is available
	 * from a slave station.
	 */
	public boolean acd;
	/**
	 * Data Flow Control bit hold by frame control field.
	 * DFC is raised (true/1) when slave communication buffers
	 * are full.
	 */
	public boolean dfc;
	/**
	 * Frame Count Bit hold by frame control field.
	 * FCB is an alternate bit to avoid confusion between
	 * a master and a slave during exchange of user data.
	 * It's raised (true/1) for a master that wish to send
	 * data to a slave or poll data from this slave.
	 */
	public boolean fcb;
	/**
	 * Frame Count bit Valid bit hold by frame control field.
	 * Just indicates the validity of FCB. Therefore, a master
	 * MUST set this bit to 1/true for polling data or for sending
	 * data to a remote.
	 */
	public boolean fcv;
	/**
	 * Station address. Identification of the station that manager
	 * works for.
	 */	
	public int address;
	/**
	 * Allocates a link layer manager according to local station
	 * context for a slave application and in regards to a single remote
	 * slave station for a master application.
	 *
	 * @param address	Local or Remote station address.
	 */
	protected LnkManager(int address)
	{
		this.address = address;
		fcb = fcv = acd = dfc = false;
	}	
}	// end LnkManager.java
