/*
 * Class	br.org.scadabr.protocol.iec101.common.LnkService
 * File		LnkService.java
 * Author	lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;
/**
 * <p>
 * This class defines service at link layer level.
 * <p>
 * Link service is of course handled by link layer and link manager.
 * <p>
 * REMEMBER LnkService ONLY CONCERNS LINK LAYER. AT APPLICATION LAYER LEVEL, WE
 * USE AppService OBJECTS.
 * 
 * @author	lionnel cauvy
 * @version	1.0
 * @date	28.II.03
 */
public class LnkService
{
	/**
	 * Station address. Target address for a master application or local address
	 * for a slave one.
	 */
	public int address;
	/**
	 * Single character command indicator. Indeed, a special whole frame 
	 * could be only a single character 0xA5.
	 */
	public boolean single;
	/**
	 * Short array eventually containing user data. Can be null depending
	 * on service type.
	 */
	public short[] data;
	/**
	 * Control field hold by frame. This one contains informations such as:<br>
	 * - prm	primary message direction<br>
	 * - fcb	frame count bit (Master only)<br>
	 * - fcv	frame count bit validity (Master only)<br>
	 * - acd	access demand bit (Slave only)<br>
	 * - dfc	data flow control (Slave only)
	 */
	public ControlField control;
	/**
	 * Allocates a link service object with no expected answer.
	 *
	 * @param control	Control field reference. Contains PRM,FCB(orACD),FCV(or DFC) infos.
	 * @param address	Remote/local address for a master/slave application.
	 */
	public LnkService(ControlField control, int address)
	{
		this.control	= control;
		this.address	= address;
	}
}	// end LnkService.java

