/*
 * Class    br.org.scadabr.protocol.iec101.common.Mapping
 * File     Mapping.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import java.util.Enumeration;
import java.util.Hashtable;

import br.org.scadabr.luciol.database.DataElement;
import br.org.scadabr.luciol.database.DataEventListener;
import br.org.scadabr.luciol.database.Database;
import br.org.scadabr.luciol.database.DatabaseApp;
import br.org.scadabr.luciol.database.Record;

/**
 * <p>
 * This class defines basic Mapping implementation for iec package.<br>
 * The mapping is divided in two. One mapping for values an application is
 * expected to write; and another one for an application axpected to read values
 * from database OR to be notified by this database.<br>
 * <p>
 * It describes read and write methods to database, record listeners methods on
 * database.
 * <p>
 * Is also present a way to instanciate Keys according to the iec standard
 * implementation in use.
 * <p>
 * REQUEST LISTENER REGISTRATION IS NOT YET EFFECTIVE. TO SET UP ONLY FOR
 * WRITER.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public abstract class Mapping implements DataEventListener {
	/**
	 * <p>
	 * Table mapping keys to asdus. This table will be filled by asdus to write
	 * in luciol database eg in case application has decoded an asdu from a
	 * remote station.
	 * <p>
	 * Key: AsduKey<br>
	 * Value: MapAsdu
	 */
	private Hashtable Map_Key_Writer;
	/**
	 * Related application manager. Mapping implements DataEventListener to be
	 * notified by luciol database. Then the mapped data is passed to
	 * application manager.
	 */
	public AppManager manager;

	/**
	 * Map Asdu Key instanciation according to implementation of iec608705
	 * currently in use.
	 * 
	 * @param object
	 *            Can be a MapAsdu or a real Asdu object.
	 * 
	 * @return the adequate writer map key.
	 */
	protected abstract AsduKey instanciateKey(Object object);

	/**
	 * Gets the 'dbref' data element located in luciol datatbase.
	 * 
	 * @param dbref
	 *            a luciol database reference.
	 * 
	 * @return the data element object requested or null.
	 */
	protected DataElement getDataElement(int dbref) {
		Database db = DatabaseApp.ACTIVE;
		DataElement elt = null;
		if (db != null)
			elt = db.getDataElement(dbref);
		return elt;
	}

	/**
	 * Records the listener for all services & readers.
	 * 
	 * @param dbref
	 *            luciol database reference
	 */
	protected void recordEventListener(int db) {
		DataElement element = getDataElement(db);
		if (element != null) {
			CodeProperties.log("Luciol database element:" + db
					+ " event listener recorded.");
			element.addDataEventListener(this);
		}
	}

	/**
	 * This call will perform an attempt to a value in luciol database from a
	 * read 'decoded' asdu sent by a remote station.<br>
	 * The mechanism is the following:<br>
	 * - First, instanciation of a key according to standard implementation and
	 * master/slave mode running application. Notice that this instanciation is
	 * operated from an Asdu object.<br>
	 * - Second, by evaluating the presence of the so-created key in the related
	 * mapping between key and references, the write is allowed or not.<br>
	 * - Third and last, thanks to MapRwAsdu fields, we can retrieve all
	 * necessary informations to locate the value to write a new record in the
	 * correct database referenced area.
	 * 
	 * @param object
	 *            an application service data unit reference
	 * 
	 * @return a flag to indicate writing or not.
	 */
	public boolean write(Object object) {
		System.out.println("escrevendooo no databaseeeeeeeeeee");
		boolean result = false;
		Asdu asdu = (Asdu) object;
		AsduKey key = instanciateKey(object);
		MapAsdu writer = null;
		DbLocator db = null;
		AsduLocator loc = null;
		Translator trans = null;
		Object toWrite = null;
		DataElement dbelement = null;
		Record record = null;

		if (Map_Key_Writer.size() == 0)
			return false;
		System.out.println("Chegou aqui! " + Map_Key_Writer.get(key));
		if ((writer = (MapAsdu) Map_Key_Writer.get(key)) != null) {
			try {
				for (int i = 0; i < writer.value.length; i++) {
					loc = writer.value[i].locator;
					db = writer.value[i].db;
					trans = writer.value[i].translator;
					toWrite = asdu.value(loc.pio, loc.pie, loc.pval);
					if (trans != null)
						toWrite = trans.apply(toWrite);
					dbelement = getDataElement(db.index);
					result = true;
					record = dbelement.instanciateRecord();
					dbelement.writeNewRecord(record);
				}
			} catch (Exception e) {
				CodeProperties.log("Bad mapping:" + e);
				return false;
			}
		}
		return result;
	}

	/**
	 * <p>
	 * Maps a key to a writer asdu. Updates only Map_Key_Asdu. Request listener
	 * is not recorded for writer.
	 * 
	 * @param writer
	 *            a writer asdu.
	 */
	public void addWriter(MapAsdu writer) {
		Map_Key_Writer.put(writer.key, writer);
	}

	/**
	 * Get accessor on a single writer asdu from its key.
	 * 
	 * @param key
	 *            A writer mapping key.
	 * 
	 * @return the mapped writer asdu or null.
	 */
	public MapAsdu getWriter(AsduKey key) {
		return (MapAsdu) Map_Key_Writer.get(key);
	}

	/**
	 * Get accessor on all recorded writer asdus from this mapping.
	 * 
	 * @return an enumeration writer asdus.
	 */
	public Enumeration enumerateWriters() {
		return Map_Key_Writer.elements();
	}

	/**
	 * Allocates a mapping object.
	 * 
	 * @param manager
	 *            Application layer manager.
	 */
	protected Mapping(AppManager manager) {
		Map_Key_Writer = new Hashtable();
		this.manager = manager;
	}
} // end Mapping.java