/*
* Class    br.org.scadabr.protocol.iec101.common.PeriodicService
* File     PeriodicService.java
* Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common;

import br.org.scadabr.protocol.iec101.common.*;

import com.ajile.components.CallbackTimer;
import com.ajile.events.TriggerEventListener;

/**
 * <p>
 * A Periodic Service use the aJile CallbackTimer avoiding to use the
 * aJile Periodic Timer that creates a supplementary Thread.<br>
 * At each event triggered, callback is relaunched. Notice that time
 * passed as argument is RELATICE and not absolute as in CallbackTimer class.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 29.XII.02
 */
public class PeriodicService implements TriggerEventListener
{
	/**
	 * Restart flag. Avoids incontrolled restart
	 * when user pause a periodic task inside
	 * trigger method body.
	 */
	private boolean restart = true;
  /**
   * aJile Callback timer.
   */
  private CallbackTimer timer;
  /**
   * Service.
   */
  private AppService svc;
  /**
   * Trigger count.
   */
  private int count;
  /**
   * Absolute time.
   */
  private int time;
  /**
   * The object to periodically trigger.
   */
  private PeriodicEventListener handler;

  /**
   * This call disables the periodic trigger.
   */
  public void pause()
  {
    timer.removeCallback(this);
	restart = false;
  }

  /**
   * Enables the periodic trigger.
   */
  public void resume()
  {
    timer.callback(System.currentTimeMillis()+time,this);
  }

  /**
   * INTERNAL method bound to CallbackTimer.
   */
  public void triggerEvent()
  {
    handler.triggerEvent(svc,count++);
	if(restart)
    	timer.callback(System.currentTimeMillis()+time,this);
  }

  /**
   * Instanciates a new periodic task.
   *
   * @param time	RELATIVE time in ms.
   * @param id		task identifiant.
   * @param handler	the object to trigger.
   */
  public PeriodicService(int time, AppService svc, PeriodicEventListener handler)
  {
    this.svc = svc;
    this.handler = handler;
    this.time = time;
	this.count = 0;
    timer = CallbackTimer.getInstance();
  }
}	// end PeriodicService.java