package br.org.scadabr.protocol.iec101.common;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

import java.io.IOException;

import br.org.scadabr.luciol.ajile.components.CallbackTimer;
import br.org.scadabr.luciol.ajile.jem.rawJEM;

public class PhySerialLayer extends PhysicalLayer {

	public String port = "unknown";
	public int baud = 9600;
	public int data = 8;
	public int stop = 1;
	public int parity = 0;

	public PhySerialLayer(int retries, int timeout, String port, int baud,
			int data, int stop, int parity) {
		super();
		this.retries = retries;
		this.attempt = retries;
		this.timeout = timeout;
		this.port = port;
		this.baud = baud;
		this.data = data;
		this.stop = stop;
		this.parity = parity;
	}

	@Override
	protected void write(byte[] data) {
		// luciol rs485 RTS activation : write mode
		if (CodeProperties.RS485) {
			if (CodeProperties.AJILE)
				sport.setRTS(false);
			else
				sport.setRTS(true);
		}
		// write the data
		try {
			if (CodeProperties.PAUSE_BEFORE_WRITE > 0)
				Thread.sleep(CodeProperties.PAUSE_BEFORE_WRITE);

			output.write(data);
			output.flush();

			if (CodeProperties.DEBUG_PHYSICAL) {
				System.out.print(" --> ");
				for (int i = 0; i < data.length; i++) {
					System.out.print(data[i] + " ");
				}
				System.out.println();
			}
			// condition on TEMT et THRE from LSR register of aJile
			if (CodeProperties.AJILE) {
				while (((rawJEM.getInt(CodeProperties.AJILE_LSR_REGISTER) & 0x60) >> 5) != 3)
					Thread.yield();
			}

			if (CodeProperties.PAUSE_AFTER_WRITE > 0)
				Thread.sleep(CodeProperties.PAUSE_AFTER_WRITE);
		} catch (IOException e) {
			CodeProperties.phylog("[Transmitter] Writing Exception " + e);
		} catch (InterruptedException e) {
		}
		// luciol rs485 RTS deactivation : read mode
		if (CodeProperties.RS485) {
			if (CodeProperties.AJILE)
				sport.setRTS(true);
			else
				sport.setRTS(false);
		}
	}

	@Override
	public void launch() {
		if (port.equals("unknown")) {
			CodeProperties.log("exit application cause:unknown comm port.");
			kill();
			upper.lowerKill();
			return;
		}
		// variables initialization
		this.input = null;
		this.output = null;
		this.attempt = 0;
		this.timer = CallbackTimer.getInstance();
		if (!CodeProperties.AJILE)
			port = port.toUpperCase();
		// Communication port initialization
		try {
			System.out.println("AQUI " + port);
			sport = (SerialPort) CommPortIdentifier.getPortIdentifier(port)
					.open("CEI8705Serial", 0);
			System.out.println("PASSOU ");
			sport.setSerialPortParams(baud, data, stop, parity);
			input = sport.getInputStream();
			output = sport.getOutputStream();
		} catch (Exception e) {
			CodeProperties.log("Comm port:" + port + " not initialized.");
			kill();
			upper.lowerKill();
			return;
		}
		CodeProperties.phylog("Port " + port + " initialized.(" + baud + ":"
				+ parity + ":" + data + ":" + stop + ")");
		// RS485 Hardware initialization : READ MODE.
		if (CodeProperties.RS485) {
			if (CodeProperties.AJILE)
				sport.setRTS(true);
			else
				sport.setRTS(false);
		}
		try {
			output.flush();
		} catch (IOException e) {
			CodeProperties
					.phylog("[Transmitter] flush output stream exception " + e);
		}
		// instanciates a controller to validate frames, givin him
		// a reference to link layer to which controller will send
		// directly valid frames.
		controller = new Controller(this);
		// creates a new receiver object givin him reference to the
		// controller to wich receiver will send read bytes.
		receiver = new Receiver(controller, input);
		System.out.println("SAIU ");
	}

	public void kill() throws RuntimeException {
		try {
			if (timer != null)
				timer.removeCallback(this);

			if (controller != null) {
				// kills controller
				controller.kill();
				controller.join();
				// controller = null;
			}
			if (receiver != null) {
				// kills receiver
				receiver.kill();
				receiver.join();
				// receiver = null;
			}
			if (input != null) {
				// close streams
				input.close();
			}
			if (output != null) {
				output.close();
			}
			if (sport != null) {
				// close comm port
				sport.close();
			}
		} catch (Exception e) {
			return;
		}
	}

}
