/*
 * Class    br.org.scadabr.protocol.iec101.common101.FromBinaryFormatter101
 * File     FromBinaryFormatter101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common101;

import br.org.scadabr.protocol.iec101.common.Translator;

/**
 * Translation rule is 101 dpi or dco on:<1> off:<2> cast.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class FromBinaryFormatter101 implements Translator
{
	/**
	 * Binary POSITIVE/NEGATIVE logic.
	 */
	public boolean logic;
	
	private static final Byte TRUE = new Byte((byte)0x02);
	private static final Byte FALSE = new Byte((byte)0x01);
	
	/**
	 * Applies a binary modification operation on a data element
	 * from iec information element.
	 *
	 * @param obj	the object to transform
	 *
	 * @return a byte <0x02> ON	<0x01> OFF
	 */
	public Object apply(Object obj)
	{
		if (logic)
		{
			if(((Boolean)obj).booleanValue())
				return TRUE;
			else
				return FALSE;
		}
		else
		{
			if(((Boolean)obj).booleanValue())
				return FALSE;
			else
				return TRUE;
		}	
	}

	/**
	 * <p>
	 * Constructs a 101 formatter object.
	 *
	 * @param logic		true:+ false:-
	 */	
	public FromBinaryFormatter101(boolean logic)
	{
		this.logic = logic;
	}
	
};	// end FromBinaryFormatter101.java