/*
 * Class    br.org.scadabr.protocol.iec101.common101.PrivateRange101
 * File     PrivateRange101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common101;

import br.org.scadabr.protocol.iec101.common.PrivateRange;
import br.org.scadabr.protocol.iec101.common101.information.*;

import java.util.Hashtable;

/**
 * <p>
 * This class adds Private Information Numbers & Function Types to
 * abstract super class Private Range. These data are only present
 * in 101 implementation from iec 608705 standard.
 * <p>
 * In the same way that for super class, we use hashtable to store
 * these new private elements from their identification octet value.
 * There's also related methods to easilyu add and get informations.
 * <p>
 * Private Range is member of a Station object.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
 
public class PrivateRange101 extends PrivateRange
{
	/**
	 * Table classes to allow dynamic creation of private asdus.
	 */
	public static final Hashtable CLASSES = new Hashtable();

	static
	{
		CLASSES.put("AFQ",				new AFQ().getClass());
		CLASSES.put("BCR",				new BCR().getClass());
		CLASSES.put("BSI",				new BSI().getClass());
		CLASSES.put("CHS",				new CHS().getClass());
		CLASSES.put("COI",				new COI().getClass());
		CLASSES.put("CP16Time2a",		new CP16Time2a().getClass());
		CLASSES.put("CP24Time2a",		new CP24Time2a().getClass());
		CLASSES.put("CP56Time2a",		new CP56Time2a().getClass());
		CLASSES.put("DCO",				new DCO().getClass());
		CLASSES.put("DIQ",				new DIQ().getClass());
		CLASSES.put("FBP",				new FBP().getClass());
		CLASSES.put("FRQ",				new FRQ().getClass());
		CLASSES.put("LOF",				new LOF().getClass());
		CLASSES.put("LOS",				new LOS().getClass());
		CLASSES.put("LSQ",				new LSQ().getClass());
		CLASSES.put("NOF",				new NOF().getClass());
		CLASSES.put("NOS",				new NOS().getClass());
		CLASSES.put("NVA",				new NVA().getClass());
		CLASSES.put("OCI",				new OCI().getClass());
		CLASSES.put("OCTET",			new OCTET().getClass());
		CLASSES.put("QCC",				new QCC().getClass());
		CLASSES.put("QDP",				new QDP().getClass());
		CLASSES.put("QDS",				new QDS().getClass());
		CLASSES.put("QOC",				new QOC().getClass());
		CLASSES.put("QOI",				new QOI().getClass());
		CLASSES.put("QOS",				new QOS().getClass());
		CLASSES.put("QPA",				new QPA().getClass());
		CLASSES.put("QPM",				new QPM().getClass());
		CLASSES.put("QRP",				new QRP().getClass());
		CLASSES.put("R32_23",			new R32_23().getClass());
		CLASSES.put("RCO",				new RCO().getClass());
		CLASSES.put("SCD",				new SCD().getClass());
		CLASSES.put("SCO",				new SCO().getClass());
		CLASSES.put("SCQ",				new SCQ().getClass());
		CLASSES.put("SEP",				new SEP().getClass());
		CLASSES.put("SIQ",				new SIQ().getClass());
		CLASSES.put("SOF",				new SOF().getClass());
		CLASSES.put("SPE",				new SPE().getClass());
		CLASSES.put("SRQ",				new SRQ().getClass());
		CLASSES.put("SVA",				new SVA().getClass());
		CLASSES.put("VTI",				new VTI().getClass());
	}


	/**
	 * Instanciates a private range for 101 remote station.
	 */	
	public PrivateRange101()
	{
	}
}	// end PrivateRange101.java