/*
 * Class    br.org.scadabr.protocol.iec101.common101.information.R32_23
 * File     R32_23.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common101.information;

import br.org.scadabr.protocol.lib.math.Float;
import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * IEEE STD 754 7.2.6.8<br>
 * (information element used by asdus id 13,14,36,50,112)<br>
 * <p><i>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 12.II.03
 */

public class R32_23 extends InformationElement
{
	//private int fraction; // 2(-23) to 2(-1) (octet 1 ... octet 3)
	//private short exposant; // 2(0) to 2(7) (octet 3 ... octet 4)
	//private boolean signe; // (octet 4 8eme bit)
	

	/**
	 * constructs R32_23 object<br>
	 * @param octet value of first octet of 'physical' data
	 * @param octet second octet
	 * @param octet third octet
	 * @param octet fourth octet
	 */
	public R32_23(short octet1, short octet2, short octet3, short octet4)
	{
		super(4,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		octetset[2] = octet3;
		octetset[3] = octet4;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */		
	protected void decode()
	{
		//int fraction = octetset[0] | (octetset[1] << 8) | ((octetset[2] & 0x7F) << 16);
		//short exposant = (short)(((octetset[2] & 0x80) >> 7) | ((octetset[3] & 0x7F) << 1));
		//boolean signe = (((octetset[3]&0x80)>>7)==1)?true:false;
		dataset[0] = new Float(Float.intBitsToFloat(octetset[0] | (octetset[1]<<8) | (octetset[2]<<16) | (octetset[3]<<24)));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = Float.floatToIntBits(((Float)dataset[0]).floatValue());
		octetset[0] = (short)((integer & 0x000000FF));
		octetset[0] = (short)((integer & 0x0000FF00) >> 8);
		octetset[0] = (short)((integer & 0x00FF0000) >> 16);
		octetset[0] = (short)((integer & 0xFF000000) >> 24);
	}
	
	/**
	 * default constructor
	 */
	public R32_23()
	{
		super(4,1);
	}

};	// end R32_23.java