/*
 * Class    br.org.scadabr.protocol.iec101.common103.AsduKey103
 * File     AsduKey103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103;

import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.AsduKey;

/**
 * <p>
 * This class defines specialized key asdu mapping for 103
 * both master & slave applications.
 * <p>
 * So, finally, such an object use same fields than iec asdu:
 * <p>
 * - type identification
 * <p>
 * - variable structure qualifier
 * <p>
 * - cause of transmission
 * <p>
 * - common address.
 * <p>
 * - function type.
 * <p>
 * - information number.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class AsduKey103 extends AsduKey
{
	/**
	 * Function type of asdu.
	 */
	public short ft;
	/**
	 * Information number of asdu.
	 */
	public short in;	
	/**
	 * Object method equals redefinition
	 *
	 * @param obj	the object to compare
	 *
	 * @return equality indicator
	 */
	public boolean equals(Object obj)
	{
	    if (obj instanceof AsduKey103)
		{
			AsduKey103 data = (AsduKey103)obj;
			
			if(id != data.id)
				return false;
			if (vsq != data.vsq)
				return false;
			if (caa != data.caa)
				return false;
			if(ft != data.ft)
				return false;
			if(in != data.in)
				return false;
				
			return true;
		}
	    else
	        return false;
	}	
	/**
	 * Hash code generator. MapKey is used in hashtable.
	 *
	 * @return the hashcode based on asdu type identification, function type and info number.
	 */
	public int hashCode()
	{
		return ((in << 16) | (ft << 8) | id);
	}
	/**
	 * Check asdu matching with this key.
	 *
	 * @param _asdu	The asdu to evaluate.
	 *
	 * @return true if matched, false else.
	 */	
	public boolean check(Asdu _asdu)
	{
		System.out.println("check");
		Asdu103 asdu = (Asdu103)_asdu;
		if (id!=-1 && id!=asdu.id)
		{
		System.out.println(":(");
			return false;
		}
		if(vsq!=-1 && vsq!=asdu.vsq)
		{
		System.out.println(":((");
			return false;
		}
		if(cot!=-1 && cot!=asdu.cot)
		{
		System.out.println(":(((");
			return false;
		}
		if(caa!=-1 && vsq!=asdu.caa)
		{
		System.out.println(":((((");
			return false;
		}
		if(ft!=-1 && ft!=asdu.ft)
		{
		System.out.println(":(((((");
			return false;
		}
		if(in!=-1 && in!=asdu.in)
		{
		System.out.println(":((((((");
			return false;
		}
		System.out.println(":)");	
		return true;
	}
	/**
	 * <p>
	 * Allocates a key for handling asdus in 103 application.
	 * Specialization is based on fields 'Information Number' and 'Function
	 * Type' addons.
	 *
	 * @param id		Asdu type identification.
	 * @param vsq		Asdu variable structure qualifier.
	 * @param cot		Asdu cause of transmission.
	 * @param caa		Asdu common address.
	 * @param ft		Function Type of asdu.
	 * @param in		Information Number of asdu.
	 */	
	public AsduKey103(short id, short vsq, short cot, int caa, short ft, short in)
	{
		super(id,vsq,cot,caa);
		this.ft = ft;
		this.in = in;
	}
	
};	// end AsduKey103.java