/*
 * Class    br.org.scadabr.protocol.iec101.common103.Interoperability103
 * File     Interoperability103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103;

import br.org.scadabr.protocol.iec101.common.Interoperability;

/**
 * This interface defines interoperability for 103 protocol
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public interface Interoperability103 extends Interoperability
{			
	/**
	 * function type
	 */
	/**
	 * FUNCTION TYPE := UI8 [1..8] <0..255>
	 * <0..127> := private range
	 * <128..129> := compatible range
	 * <130..143> := private range
	 * <144..145> := compatible range
	 * <146..159> := private range
	 * <160..161> := compatible range
	 * <162..175> := private range
	 * <176..177> := compatible range
	 * <178..191> := private range
	 * <192..193> := compatible range
	 * <194..207> := private range
	 * <208..209> := compatible range
	 * <210..223> := private range
	 * <224..225> := compatible range
	 * <226..239> := private range
	 * <240..241> := compatible range
	 * <242..253> := private range
	 * <254..255> := compatible range
	 * 
	 * <128> := distance protection t(z)
	 * <160> := overcurrent protection I>>
	 * <176> := transformer differential protection DI T
	 * <192> := line differential protection DI L
	 * <254> := generic function type GEN
	 * <255> := global function type GLB
	 */
	public static final byte FT_REMOTE_PROT			= (byte)0x80;
	public static final byte FT_OVERCURRENT_PROT	= (byte)0xA0;
	public static final byte FT_DI_T				= (byte)0xB0;
	public static final byte FT_DI_L				= (byte)0xC0;
	public static final byte FT_GEN					= (byte)0xFE;
	public static final byte FT_GLB					= (byte)0xFF;
	
	public static final byte FT_ALSTOM				= (byte)0xA8;

	/**
	 * Information number - can be extended by private inf
	 */
	/**
	 * INFORMATION NUMBER := UI8 [1..8] <0..255>
	 * with monitor direction := <0..255>
	 * <0..15> := system functions
	 * <16..31> := status
	 * <32..47> := supervision
	 * <48..63> := earth fault
	 * <64..127> := short circuit
	 * <128..143> := auto-reclosure
     * <144..159> := measurands
	 * <160..239> := not used
	 * <240..255> := generic functions
	 * control direction := <0..255>
	 * <0..15> := system functions
	 * <16..31> := general commands
	 * <32..239> := not used
	 * <240..255> := generic functions
	 */
	 
	 
	/**
	 * <0> := end of general interrogation			MC
	 * <0> := time synchronization					MC
	 * <1> := not used
	 * <2> := reset FCB
	 * <3> := reset CU
	 * <4> := start/restart
	 * <5> := power on
	 */
	public static final byte IN_GI						=	0x00;
	public static final byte IN_SYNCHRO					=	0x00;
	public static final byte IN_RESET_FCB				=	0x02;
	public static final byte IN_RESET_CU				=	0x03;
	public static final byte IN_START_RESTART			=	0x04;
	public static final byte IN_POWER_ON				=	0x05;
	/**
	 * <16> := auto-recloser active					MC
	 * <17> := teleprotection active				MC
	 * <18> := protection active					MC
	 * <19> := LED reset							MC
	 * <20> := monitor direction blocked
	 * <21> := test mode
	 * <22> := local parameter setting
	 * <23> := characteristic 1						MC
	 * <24> := characteristic 2						MC
	 * <25> := characteristic 3						MC
	 * <26> := characteristic 4						MC
	 * <27> := auxiliary input 1
	 * <28> := auxiliary input 2
	 * <29> := auxiliary input 3
	 * <30> := auxiliary input 4
	 */
	public static final byte IN_AR_ACTIVE				=	0x10;
	public static final byte IN_TP_ACTIVE				=	0x11;
	public static final byte IN_P_ACTIVE				=	0x12;
	public static final byte IN_LED_RESET				=	0x13;
	public static final byte IN_MONITOR_BLOCKED			=	0x14;
	public static final byte IN_TEST_MODE				=	0x15;
	public static final byte IN_LOCAL_PARAM				=	0x16;
	public static final byte IN_CHAR_1					=	0x17;
	public static final byte IN_CHAR_2					=	0x18;
	public static final byte IN_CHAR_3					=	0x19;
	public static final byte IN_CHAR_4					=	0x1A;
	public static final byte IN_AUX_IN_1				=	0x1B;
	public static final byte IN_AUX_IN_2				=	0x1C;
	public static final byte IN_AUX_IN_3				=	0x1D;
	public static final byte IN_AUX_IN_4				=	0x1E;
	/**
	 * <32> := measurand supervision I
	 * <33> := measurand supervision V
	 * <35> := phase sequence supervision
	 * <36> := trip circuit supervision
	 * <37> := I>> back-up operation
	 * <38> := VT fuse failure*
	 * <39> := teleprotection disturbed
	 * <46> := group warning
	 * <47> := group alarm
	 */
	public static final byte IN_MEA_SUP_I				=	0x20;
	public static final byte IN_MEA_SUP_V				=	0x21;
	public static final byte IN_PHASE_SEQ_SUP			=	0x23;
	public static final byte IN_TRIP_CIRCUIT_SUP		=	0x24;
	public static final byte IN_I_BACKUP				=	0x25;
	public static final byte IN_VT_FUSE_FAILURE			=	0x26;
	public static final byte IN_TP_DISTURBED			=	0x27;
	public static final byte IN_GROUP_WARNING			=	0x2E;
	public static final byte IN_GROUP_ALARM				=	0x2F;
	/**
	 * <48> := earth fault L 1
	 * <49> := earth fault L 2
	 * <50> := earth fault L 3
	 * <51> := earth fault forward, i.e. line
	 * <52> := earth fault reverse, i.e. busbar
	 */
	public static final byte IN_EARTH_FAULT_L1			=	0x30;
	public static final byte IN_EARTH_FAULT_L2			=	0x31;
	public static final byte IN_EARTH_FAULT_L3			=	0x32;
	public static final byte IN_EARTH_FAULT_FORWARD		=	0x33;
	public static final byte IN_EARTH_FAULT_REVERSE		=	0x34;
	/**
	 * <64> := start /pick-up L 1
	 * <65> := start /pick-up L 2
	 * <66> := start /pick-up L 3
	 * <67> := start /pick-up N
	 * <68> := general trip
	 * <69> := trip L 1
	 * <70> := trip L 2
	 * <71> := trip L 3
	 * <72> := trip I>> (back-up operation)
	 * <73> := fault location X in ohms
	 * <74> := fault forward/line
	 * <75> := fault reverse/busbar
	 * <76> := teleprotection signal transmitted
	 * <77> := teleprotection signal received
	 * <78> := zone 1
	 * <79> := zone 2
	 * <80> := zone 3
	 * <81> := zone 4
	 * <82> := zone 5
	 * <83> := zone 6
	 * <84> := general start/pick-up
	 * <85> := breaker failure
	 * <86> := trip measuring system L 1
	 * <87> := trip measuring system L 2
	 * <88> := trip measuring system L 3
	 * <89> := trip measuring system E
	 * <90> := trip I>
	 * <91> := trip I>>
	 * <92> := trip IN>
	 * <93> := trip IN>>
	 */
	public static final byte IN_START_PICKUP_L1			=	0x40;
	public static final byte IN_START_PICKUP_L2			=	0x41;
	public static final byte IN_START_PICKUP_L3			=	0x42;
	public static final byte IN_START_PICKUP_N			=	0x43;
	public static final byte IN_GENERAL_TRIP			=	0x44;
	public static final byte IN_TRIP_L1					=	0x45;
	public static final byte IN_TRIP_L2					=	0x46;
	public static final byte IN_TRIP_L3					=	0x47;
	public static final byte IN_TRIP_I2_BACKUP			=	0x48;
	public static final byte IN_FAULT_LOC_X				=	0x49;
	public static final byte IN_FAULT_FORWARD			=	0x4A;
	public static final byte IN_FAULT_REVERSE			=	0x4B;
	public static final byte IN_TP_TRANSMITTED			=	0x4C;
	public static final byte IN_TP_RECEIVED				=	0x4D;
	public static final byte IN_ZONE_1					=	0x4E;
	public static final byte IN_ZONE_2					=	0x4F;
	public static final byte IN_ZONE_3					=	0x50;
	public static final byte IN_ZONE_4					=	0x51;
	public static final byte IN_ZONE_5					=	0x52;
	public static final byte IN_ZONE_6					=	0x53;
	public static final byte IN_GENERAL_START_PICKUP	=	0x54;
	public static final byte IN_BREAKER_FAILURE			=	0x55;
	public static final byte IN_TRIP_MEA_L1				=	0x56;
	public static final byte IN_TRIP_MEA_L2				=	0x57;
	public static final byte IN_TRIP_MEA_L3				=	0x58;
	public static final byte IN_TRIP_MEA_E				=	0x59;
	public static final byte IN_TRIP_I1					=	0x5A;
	public static final byte IN_TRIP_I2					=	0x5B;
	public static final byte IN_TRIP_IN1				=	0x5C;
	public static final byte IN_TRIP_IN2				=	0x5D;
	/**
	 * <128> := Circuit Breaker on by AR
	 * <129> := CB on by long-time AR
	 * <130> := AR blocked
	 */	
	public static final byte IN_CB_ON					=	(byte)0x80;	
	public static final byte IN_CB_ON_LONG				=	(byte)0x81;
	public static final byte IN_AR_BLOCKED				=	(byte)0x82;
	/**
	 * <144> := measurand I
	 * <145> := measurands I, V
	 * <146> := measurands I, V, P, Q
	 * <147> := measurands I N , V EN
	 * <148> := measurands I
	 */	
	public static final byte IN_MEA_I					=	(byte)0x90;	
	public static final byte IN_MEA_I_V					=	(byte)0x91;
	public static final byte IN_MEA_I_V_P_Q				=	(byte)0x92;	
	public static final byte IN_MEA_IN_VEN				=	(byte)0x93;
	public static final byte IN_MEA_IL_VL_P_Q_F			=	(byte)0x94;
	 
	/**
	 * <240> := read headings of all defined groups						MC
	 * <241> := read values or attributes of all entries of one group	MC
	 * <242> := not used												MC
	 * <243> := read directory of a single entry						MC
	 * <244> := read value or attribute of a single entry				MC
	 * <245> := end of general interrogation of generic data			MC
	 * <249> := write entry with confirmation							MC
	 * <250> := write entry with execution								MC
	 * <251> := write entry aborted										MC
	 */	
	public static final byte IN_READ_ALL_HEAD			=	(byte)0xF0;
	public static final byte IN_READ_ALL_VAL_ATTR		=	(byte)0xF1;
	public static final byte IN_NOT_USED				=	(byte)0xF2;	
	public static final byte IN_READ_DIR				=	(byte)0xF3;
	public static final byte IN_READ_VAL_ATTR			=	(byte)0xF4;
	public static final byte IN_END_GI_GENERIC_DATA		=	(byte)0xF5;	
	public static final byte IN_WRITE_ENTRY_CONFIRMATION=	(byte)0xF9;
	public static final byte IN_WRITE_ENTRY_EXECUTION	=	(byte)0xFA;
	public static final byte IN_WRITE_ENTRY_ABORTED		=	(byte)0xFB;

	/**
	 * Cause of transmission - can be extended by private cot
	 */	
	public static final byte	COT_SPONTANEOUS			=	0x01;
	public static final byte	COT_CYCLIC				=	0x02;
	public static final byte	COT_RESET_FCB			=	0x03;
	public static final byte	COT_RESET_CU			=	0x04;
	public static final byte	COT_START_RESTART		=	0x05;
	public static final byte	COT_POWER				=	0x06;
	public static final byte	test_mode				=	0x07;
	public static final byte	COT_SYNCHRO				=	0x08;
	public static final byte	COT_GI					=	0x09;
	public static final byte	COT_END_GI				=	0x0A;
	public static final byte	local_op				=	0x0B;
	public static final byte	COT_REMOTE				=	0x0C;
	public static final byte	COT_ACK_COM				=	0x14;// or general command
	public static final byte	COT_NACK_COM			=	0x15;
	public static final byte	disturbance_data		=	0x1F;
	public static final byte	ack_write_com			=	0x28;
	public static final byte	nack_write_com			=	0x29;
	public static final byte	valid_read_com			=	0x2A;
	public static final byte	invalid_read_com		=	0x2B;
	public static final byte	generic_write_conf		=	0x2C;

	/**
	 * Asdus - can be extended by private asdu
	 * asdus in monitor direction M
	 * asdus in control direction C
	 */
	public static final byte	ID_TIME_MSG						=	0x01;	// M
	public static final byte	ID_RELATIVE_TIME_MSG			=	0x02;	// M
	public static final byte	ID_MEASURANDSI					=	0x03;	// M
	public static final byte	RelativeTimeTaggedMeasurands	=	0x04;	// M
	public static final byte	ID_IDENTIFICATION				=	0x05;	// M
	public static final byte	ID_SYNCHRO						=	0x06;	// MC
	public static final byte	ID_INIT_GI						=	0x07;	// C
	public static final byte	ID_END_GI						=	0x08;	// M
	public static final byte	ID_MEASURANDSII					=	0x09;	// M
	public static final byte	ID_GENERIC_DATA					=	0x0A;	// MC
	public static final byte	GenericIdentification			=	0x0B;	// M
	public static final byte	ID_GENERAL_CD					=	0x14;	// C
	public static final byte	ID_GENERIC_CD					=	0x15;	// C			
	public static final byte	ID_LIST_RECORD_DISTURBANCE		=	0x17;	// M
	public static final byte	ID_ORDER_DISTURBANCE			=	0x18;	// C
	public static final byte	ID_ACK_DISTURBANCE				=	0x19;	// C
	public static final byte	ReadyTransmissionDisturbanceData=	0x1A;	// M
	public static final byte	ReadyTransmissionChannel		=	0x1B;	// M
	public static final byte	ReadyTransmissionTags			=	0x1C;	// M
	public static final byte	TransmissionTags				=	0x1D;	// M
	public static final byte	TransmissionDisturbanceValues	=	0x1E;	// M
	public static final byte	EndTransmission					=	0x1F;	// M

	
	public static final int		HEADER_ASDU_SIZE				= 6;

	
};	// end Interoperability103.java