/*
 * Class    br.org.scadabr.protocol.iec101.common103.PrivateAsdu103
 * File     PrivateAsdu103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103;

import br.org.scadabr.protocol.iec101.common.PrivateAsdu;

/**
 * <p>
 * This class specializes a private asdu for 103 implementation.
 * <p>
 * Adds Function Type and Information Number fields.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
 
public class PrivateAsdu103 extends PrivateAsdu
{
	/**
	 * Variable structure qualifier.
	 */
	public short vsq;
	/**
	 * Single or Sequence indicator.
	 */
	public boolean sq;
	/**
	 * Number of information objects or elements according to sq value.
	 */
	public byte nio;
	/**
	 * Cause of transmission.
	 */
	public short cot;
	/**
	 * Common address of asdu.
	 */
	public int caa;
	/**
	 * Function type.
	 */
	public short ft;
	/**
	 * Information number.
	 */
	public short in;

	/**
	 * Allocates a new private asdu for iec 103 protocol.
	 *
	 * @param label		label to comment this asdu
	 * @param id		type identification
	 * @param vsq		Variable Structure Qualifier.
	 * @param cot		Cause of Transmission.
	 * @param caa		Common address of asdu.
	 * @param ft		Function Type.
	 * @param in		Information Number.
	 * @param element	array of information elements
	 */	
	public PrivateAsdu103(String label, short id, short vsq, short cot, int caa, short ft, short in, String[] ie)
	{
		super(id,label,ie);
		this.vsq	= vsq;
		this.nio	= (byte)(vsq & 0x7F);
		this.sq		= (((vsq & 0x80) >> 7)==1)?true:false;
		this.cot	= cot;
		this.caa	= caa;
		this.ft		= ft;
		this.in		= in;
	}
};	// end PrivateAsdu103.java