/*
 * Class    br.org.scadabr.protocol.iec101.common103.PrivateRange103
 * File     PrivateRange103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103;

import br.org.scadabr.protocol.iec101.common.PrivateRange;
import br.org.scadabr.protocol.iec101.common103.information.*;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * <p>
 * This class adds Private Information Numbers & Function Types to
 * abstract super class Private Range. These data are only present
 * in 103 implementation from iec 608705 standard.
 * <p>
 * In the same way that for super class, we use hashtable to store
 * these new private elements from their identification octet value.
 * There's also related methods to easilyu add and get informations.
 * <p>
 * Private Range is member of a Station object.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
 
public class PrivateRange103 extends PrivateRange
{	/**
	 * Table classes to allow dynamic creation of private asdus.
	 */
	public static final Hashtable CLASSES = new Hashtable();

	static
	{
		CLASSES.put("ACC",				new ACC().getClass());
		CLASSES.put("ASC",				new ASC().getClass());
		CLASSES.put("COL",				new COL().getClass());
		CLASSES.put("CP32Time2a",		new CP32Time2a().getClass());
		CLASSES.put("CP56Time2a",		new CP56Time2a().getClass());
		CLASSES.put("DCO",				new DCO().getClass());
		CLASSES.put("DPI",				new DPI().getClass());
		CLASSES.put("FAN",				new FAN().getClass());
		CLASSES.put("GDD",				new GDD().getClass());
		CLASSES.put("GID",				new GID().getClass());
		CLASSES.put("GRC",				new GRC().getClass());
		CLASSES.put("INT",				new INT().getClass());
		CLASSES.put("KOD",				new KOD().getClass());
		CLASSES.put("MEA",				new MEA().getClass());
		CLASSES.put("NDE",				new NDE().getClass());
		CLASSES.put("NDV",				new NDV().getClass());
		CLASSES.put("NGD",				new NGD().getClass());
		CLASSES.put("NOC",				new NOC().getClass());
		CLASSES.put("NOE",				new NOE().getClass());
		CLASSES.put("NOF",				new NOF().getClass());
		CLASSES.put("NOG",				new NOG().getClass());
		CLASSES.put("NOT",				new NOT().getClass());
		CLASSES.put("NUD",				new NUD().getClass());
		CLASSES.put("RET",				new RET().getClass());
		CLASSES.put("RFA",				new RFA().getClass());
		CLASSES.put("RII",				new RII().getClass());
		CLASSES.put("RPV",				new RPV().getClass());
		CLASSES.put("RSV",				new RSV().getClass());
		CLASSES.put("SCL",				new SCL().getClass());
		CLASSES.put("SCN",				new SCN().getClass());
		CLASSES.put("SDV",				new SDV().getClass());
		CLASSES.put("SIN",				new SIN().getClass());
		CLASSES.put("SOF",				new SOF().getClass());
		CLASSES.put("TAG",				new TAG().getClass());
		CLASSES.put("TAP",				new TAP().getClass());
		CLASSES.put("TOO",				new TOO().getClass());
		CLASSES.put("TOV",				new TOV().getClass());
	}
	
	/**
	 * Private information number hashtable.
	 */
	public Hashtable Table_Id_In;
	
	/**
	 * Private function type hashtable.
	 */
	public Hashtable Table_Id_Ft;
	
	/**
	 * Add an information number to private range.
	 *
	 * @param pin	private information number reference
	 */
	public void addIn(PrivateIn103 pin)
	{
		Table_Id_In.put(new Integer(pin.id),pin);
	}
		
	/**
	 * Add an function type to private range.
	 *
	 * @param pft	private function type reference.
	 */
	public void addFt(PrivateFt103 pft)
	{
		Table_Id_Ft.put(new Integer(pft.id),pft);
	}

	/**
	 * Get accessor on a single private information number from its id.
	 *
	 * @param id	Id of private information number.
	 *
	 * @return the private in or null.
	 */		
	public PrivateIn103 getIn(int id)
	{
		return (PrivateIn103)Table_Id_In.get(new Integer(id));
	}

	/**
	 * Get accessor on a single private function type from its id.
	 *
	 * @param id	Id of private function type.
	 *
	 * @return the private ft or null.
	 */		
	public PrivateFt103 getFt(int id)
	{
		return (PrivateFt103)Table_Id_Ft.get(new Integer(id));
	}

	/**
	 * Get accessor on all private information numbers.
	 *
	 * @return enumeration of private in.
	 */		
	public Enumeration enumerateIns()
	{
		return Table_Id_In.elements();
	}

	/**
	 * Get accessor on all private function type.
	 *
	 * @return enumeration of private ft.
	 */		
	public Enumeration enumerateFts()
	{
		return Table_Id_Ft.elements();
	}

	/**
	 * Instanciates a private range for 103 remote station.
	 */	
	public PrivateRange103()
	{
		Table_Id_In	= new Hashtable();
		Table_Id_Ft	= new Hashtable();
	}
}	// end PrivateRange103.java