/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.ACC
 * File     ACC.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Actual channel 7.2.6.1</b><br>
 * (information element used by asdu id 24)<br>
 * <p><i>
 * ACC := UI8[1..8] <1..255><br>
 * with:<br>
 * <i><0> := global<br>
 * <1> := I L1<br>
 * <2> := I L2<br>
 * <3> := I L3<br>
 * <4> := I N<br>
 * <5> := V L1E<br>
 * <6> := V L2E<br>
 * <7> := V L3E<br>
 * <8> := V EN<br>
 * <9..63> := reserved for future compatible use<br>
 * <64...255> := reserved for private use</i><br>
 * This octet indicates the actual channel to be processed within the transmission of disturbance data.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class ACC extends InformationElement
{
	public static final byte	GLOBAL			= 0x00;
	public static final byte	IL1				= 0x01;
	public static final byte	IL2				= 0x02;
	public static final byte	IL3				= 0x03;
	public static final byte	IN				= 0x04;
	public static final byte	VL1E			= 0x05;
	public static final byte	VL2E			= 0x06;
	public static final byte	VL3E			= 0x07;
	public static final byte	VEN				= 0x08;

	/**
	 * constructs ACC object<br>
	 * actual channel 7.2.6.1
	 *
	 * @param octet value of 'physical' data
	 */
	public ACC(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public ACC()
	{
		super(1,1);
	}
	
};	// end ACC.java