/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.ASC
 * File     ASC.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>ASCII character 7.2.6.2</b><br>
 * (information element used by asdu id 24) <br>
 * <p><i>
 * ASC := UI8 [1..8] <ASCII 8-bit code> <br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class ASC extends InformationElement
{
	/**
	 * constructs ASC object<br>
	 * ASCII character 7.2.6.2
	 *
	 * @param octet value of 'physical' data
	 */
	public ASC(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new String(Integer.toHexString((int)octetset[0]));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = (Short.parseShort(((String)dataset[0])));
	}
	
	/**
	 * default constructor
	 */	
	public ASC()
	{
		super(1,1);
	}
	
};	// end ASC.java