/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.COL
 * File     COL.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Compatibility level 7.2.6.3</b><br>
 * (information element used by asdu id 24) <br>
 * <p><i>
 * COL := UI8 [1..8] <0..255> <br>
 * The compatibility level of protection equipment based on this companion standard is 2 without use of
 * generic services, and 3 with use of generic services.
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class COL extends InformationElement
{
	/**
	 * <2> no generic services
	 */	
	private static byte SERVICES_OFF	= 0x02;
	/**
	 * <3> generic services
	 */
	private static byte SERVICES_ON		= 0x03;	


	/**
	 * constructs COL object<br>
	 * compatibility level 7.2.6.3
	 * @param octet value of 'physical' data
	 */
	public COL(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public COL()
	{
		super(1,1);
	}
	
};	// end COL.java