/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.CP32Time2a
 * File     CP32Time2a.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

import java.util.Calendar;

/**
 * <p>
 * <b>CP32Time2a four octet binary time 7.2.6.4</b><br>
 * <p><i>
 * CP32Time2a := CP32 {Milliseconds, Minutes, RES1, Invalid, Hours, RES2, Summer time}<br>
 * This binary time is defined in 6.8 of CEI 60870-5-4. It is used for the TIME TAG of an INFORMATION
 * OBJECT.<br>
 * </i>
 * (information element used by asdu id 30,31,32,33,34,35,36,37,38,39,40,103,126)<br>
 * <p><i>
 * milliseconds <br>
 * 2 octets <br>
 * Milliseconds	--> dataset[0] <br>
 * <br>
 * minutes <br>
 * 6 bits <br>
 * Minutes		--> dataset[1] <br>
 * <br>
 * Reserved 1 <br>
 * 1 bit <br>
 * RES1			--> dataset[2] <br>
 * <br>
 * invalid <br>
 * 1 bit <br>
 * IV			--> dataset[3] <br>
 * <br>
 * hours <br>
 * 5 bits <br>
 * Hours		--> dataset[4] <br>
 * <br>
 * Reserved 2 <br>
 * 2 bits <br>
 * RES2			--> dataset[5] <br>
 * <br>
 * summer time <br>
 * 1 bit <br>
 * SU			--> dataset[6] <br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class CP32Time2a extends InformationElement
{
	/**
	 * constructs CP56Time2a object (reader factory)<br>
	 * four octet binary time 7.2.6.28
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 * @param octet3 third octet
	 * @param octet4 fourth octet
	 */
	public CP32Time2a(short octet1,
	                  short octet2,
					  short octet3,
					  short octet4)
	{
		super(4,7);
		octetset[0] = octet1;
		octetset[1] = octet2;
		octetset[2] = octet3;
		octetset[3] = octet4;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */		
	protected void decode()
	{		
		// milliseconds
		dataset[0] = new Integer(octetset[0] | (octetset[1] << 8));
		// minutes
		dataset[1] = new Byte((byte) ( octetset[2] & 0x3F ));
		// reserve1
		dataset[2] = new Boolean(((( octetset[2] & 0x40 ) >> 6)== 1)?true:false);
		// invalid
		dataset[3] = new Boolean(((( octetset[2] & 0x80 ) >> 7) == 1)?true:false);
		// hours
		dataset[4] = new Byte((byte) ( octetset[3] & 0x1F ));
		// reserve2
		dataset[5] = new Byte((byte) ( ( octetset[3] & 0x60 ) >> 5 ));
		// summer time
		dataset[6] = new Boolean(((( octetset[3] & 0x80 ) >> 7) == 1)?true:false);
		// computes invalidity
		invalidity = (short)((((Boolean)dataset[3]).booleanValue()?1:0) << 1);
		
		if (invalidity!=0)invalidity |= 2;
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = (short)(((Integer)dataset[0]).intValue() & 0xFF);
		octetset[1] = (short)((((Integer)dataset[0]).intValue() & 0xFF00)>>8);
		octetset[2] = (short)( ((Byte)dataset[1]).byteValue() | ((((Boolean)dataset[2]).booleanValue()?1:0)<<6) | ((((Boolean)dataset[3]).booleanValue()?1:0)<<7) );
		octetset[3] = (short)( ((Byte)dataset[4]).byteValue() | (((Byte)dataset[5]).byteValue()<<5) | (((((Boolean)dataset[6]).booleanValue())?1:0)<<7) );
	}

	/**
	 * constructs CP32Time2a object (writer factory)<br>
	 * four octet binary time 7.2.6.28
	 */	
	public CP32Time2a()
	{
		super(4,7);
		Calendar calendar = Calendar.getInstance();
		dataset[0] = new Integer(calendar.get(Calendar.MILLISECOND)+1000*calendar.get(Calendar.SECOND));
		dataset[1] = new Byte((byte)calendar.get(Calendar.MINUTE));
		dataset[2] = new Boolean(false);
		dataset[3] = new Boolean(false);
		dataset[4] = new Byte((byte)calendar.get(Calendar.HOUR));
		dataset[5] = new Byte((byte)0x00);
		dataset[6] = new Boolean(false);
		code();
	}
	
};	// end CP32Time2a.java