/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.DCO
 * File     DCO.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Double command (see IEV 371-03-03) 7.2.6.4</b><br>
 * <p><i>
 * DCO := UI2 [1..2] <0..3> <br>
 * with <0> := not used <br>
 * 		<1> := OFF <br>
 * 		<2> := ON <br>
 * 		<3> := not used <br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class DCO extends InformationElement
{
	/**
	 * constructs DCO object<br>
	 * double command 7.2.6.4
	 *
	 * @param octet octet from link user data
	 */
	public DCO(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Byte((byte)(octetset[0] & 0x03));
		// computes invalidity
		invalidity = (short)( (((((Byte)dataset[0]).byteValue()==0)?1:0) << 5) | (((((Byte)dataset[0]).byteValue()==3)?1:0) << 5) );
		if (invalidity!=0 )invalidity |= 2;
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Byte)dataset[0]).byteValue();
	}
	
	/**
	 * default constructor
	 */	
	public DCO()
	{
		super(1,1);
		//octetset[0] = 0x01;//default off
	}
	
};	// end DCO.java