/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.FAN
 * File     FAN.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Fault number 7.2.6.6<b><br>
 * (information element used by asdu id 23,24,26) <br>
 * <p><i>
 * FAN := UI16 [1..16] <0..65535>
 * The fault number is used to identify an event associated with a protection function, for example, a
 * start/pick-up from a protection equipment will increment a fault number. This means that a sequence
 * with an unsuccessful auto-reclosure will be recorded as two separate fault numbers. The fault number
 * need not be reset nor preset.
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class FAN extends InformationElement
{
	/**
	 * constructs FAN object<br>
	 * fault number 7.2.6.6
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public FAN(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public FAN()
	{
		super(2,1);
	}
	
};	// end FAN.java