/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.GDD
 * File     GDD.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Generic data description 7.2.6.32</b><br>
 * <p><i>
 * GDD := CP24 {DATATYPE, DATASIZE, NUMBER, CONT}<br>
 * with<br><br>
 * DATATYPE := UI8[1..8] <0..255>
 * <0>	:= No data<br>
 * <1>	:= OS8ASCII := OS8[1..8]<ASCII 8-bit-code><br>
 * <2>	:= PACKEDBITSTRING := BS1<br>
 * <3>	:= UI<br>
 * <4>	:= I<br>
 * <5>	:= UF<br>
 * <6>	:= F<br>
 * <7>	:= R32.23 := Short real IEEE 754<br>
 * <8>	:= R64.53 := Real IEEE 754<br>
 * <9>	:= DOUBLE POINT INFORMATION (see DPI 7.2.6.5)<br>
 * <10> := SINGLE POINT INFORMATION<br>
 * <11> := DOUBLE POINT INFORMATION WITH TRANSIENT AND ERROR := UI2[1..2] <0..3> <br>
 * <0>	:= TRANSIENT <br>
 * <1>	:= OFF <br>
 * <2>	:= ON <br>
 * <3>	:= ERROR <br>
 * <12> := MEASURAND WITH QUALITY DESCRIPTOR (see 7.2.6.8) <br>
 * <13> := Reserved <br>
 * <14> := BINARY TIME (see 7.2.6.29) <br>
 * <15> := GENERIC IDENTIFICATION NUMBER (see 7.2.6.31) <br>
 * <16> := RELATIVE TIME (see 7.2.6.15) <br>
 * <17> := FUNCTION TYPE AND INFORMATION NUMBER := CP16 {Type, INF} <br>
 * Type := UI8[1..8] <0..255> <br>
 * INF	:= UI8[1..8] <0..255> (see annex A.2.8) <br>
 * <18> := TIME TAGGED MESSAGE := CP48 {DPI, RES, TIME, SIN} <br>
 * with <br>
 * DPI	:= UI2[1..2] (see 7.2.6.5) <br>
 * RES	:= BS6[3..8] <0> <br>
 * TIME := CP32Time2a[9..40] (see 7.2.6.28) <br>
 * SIN	:= UI8[41..48] (see 7.2.6.23) <br>
 * <19> := TIME TAGGED MESSAGE WITH RELATIVE TIME <br>
 * := CP80 {DPI, RES, RET, FAN, TIME, SIN} <br>
 * with <br>
 * DPI	:= UI2[1..2] (see 7.2.6.5) <br>
 * RES	:= BS6[3..8] <0> <br>
 * RET	:= UI16[9..24] (see 7.2.6.15) <br>
 * FAN	:= UI16[25..40] (see 7.2.6.6) <br>
 * TIME := CP32Time2a[41..72] (see 7.2.6.28) <br>
 * SIN	:= UI8[73..80] (see 7.2.6.23) <br>
 * <20>	:= TIME TAGGED MEASURAND WITH RELATIVE TIME <br>
 * := CP96 {VAL, RET, FAN, TIME}
 * with <br>
 * VAL	:= R32.23[1..32] <br>
 * RET	:= UI16[33..48] (see 7.2.6.15) <br>
 * FAN	:= UI16[49..64] (see 7.2.6.6) <br>
 * TIME	:= CP32Time2a[65..96] (see 7.2.6.28) <br>
 * <21>	:= EXTERNAL TEXT NUMBER := UIi <br>
 * <22>	:= GENERIC REPLY CODE (see 7.2.6.36) <br>
 * <23>	:= DATA STRUCTURE := CPii{(GDD, GID)} (see 7.2.6.32 and 7.2.6.33) <br>
 * <24>	:= INDEX <br>
 * <25..255>	:= Reserved <br>
 * DATASIZE		:= UI8[9..16] <1..255> <br>
 * NUMBER		:= UI7[17..23] <1..127> <br>
 * CONT			:= BS1[24] <0..1> <br>
 * <0> := no following data elements <br>
 * <1> := following data in the next ASDU with the same RII <br>
 * Data types are defined in accordance with CEI 60870-5-4. <br>
 * The EXTERNAL TEXT NUMBER is used as a reference to a description. The text is specific for every
 * protection equipment and shall be distributed separately, for example as a file with external description
 * numbers and corresponding text. <br>
 * The PACKEDBITSTRING type has a special meaning. If an array or a combination with consecutive
 * PACKEDBITSTRINGS is defined, they are packed to octets. For example an array of 8 PACKEDBITSTRINGS
 * has the size of one octet, a combination of arrays with 7 and 2 PACKEDBITSTRINGS has the size of two
 * octets. The octets are each filled from bit position 1 to 8 without gaps, only the last octet may have less
 * then 8 filled bits. <br>
 * DATASIZE defines the number of octets used for the data type defined by DATATYPE. In the case of a
 * PACKEDBITSTRING it defines the number of bits. <br>
 * NUMBER defines the quantity of data elements defined by DATATYPE and DATASIZE in the actual ASDU.<br><br>
 * NOTE 1  The TRANSIENT and ERROR states within the DOUBLE POINT INFORMATION WITH TRANSIENT AND ERROR are used for ex-ample
 * with switchgear. TRANSIENT can be used to indicate a temporary unknown state during normal operating conditions,
 * ERROR can be used to indicate a permanent indeterminate state.<br>
 * NOTE 2  Complex values (e.g. R + jX) may be treated as an array of two fields, starting with the real part, in polar form (e.g.
 * Z, q) starting with the argument.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class GDD extends InformationElement
{
	/** GDD value
	  *
	  */
	//private short DATATYPE;	--> dataset[0]
	//private short DATASIZE;	--> dataset[1]
	//private short NUMBER;		--> dataset[2]
	//private boolean CONT;		--> dataset[3]

	public short getDATASIZE()
	{
		return ((Short)dataset[0]).shortValue();
	}
	
	public short getNUMBER()
	{
		return (short)(((Byte)dataset[2]).byteValue());
	}

	/**
	 * constructs GDD object<br>
	 * generic data description 7.2.6.32
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 * @param octet3 third octet
	 */
	public GDD(short octet1, short octet2, short octet3)
	{
		super(3,4);
		octetset[0] = octet1;
		octetset[1] = octet2;
		octetset[2] = octet3;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		dataset[1] = new Short(octetset[1]);
		dataset[2] = new Byte((byte)(octetset[2] & 0x7F));
		dataset[3] = new Boolean((((octetset[2]&0x80)>>7)==0)?false:true);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
		octetset[1] = ((Short)dataset[1]).shortValue();
		octetset[2] = (short)( ((Byte)dataset[2]).byteValue() | ((((Boolean)dataset[0]).booleanValue()?1:0) << 7) );
	}
	
	/**
	 * default constructor
	 */	
	public GDD()
	{
		super(3,4);
	}
	
};	// end GDD.java