/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.GIN
 * File     GIN.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Generic identification number 7.2.6.31</b><br>
 * <p><i> 
 * GIN := CP16{GROUP, ENTRY}<br>
 * with<br>
 * GROUP := UI8[1..8] <0..255><br>
 * ENTRY := UI8[9..16] <0..255><br>
 * <0> := GROUP identifier<br>
 * <1..255> := ENTRY identifier<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class GIN extends InformationElement
{
	/**
	 * GIN value
	 */
	//private short GROUP;	--> dataset[0]
	//private short ENTRY;	--> dataset[1]
	

	/**
	 * constructs GIN object<br>
	 * generic information number 7.2.6.31
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public GIN(short octet1, short octet2)
	{
		super(2,2);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		dataset[1] = new Short(octetset[1]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
		octetset[1] = ((Short)dataset[1]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public GIN()
	{
		super(2,2);
	}
	
};	// end GIN.java