/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.GRC
 * File     GRC.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Generic reply code 7.2.6.36</b><br>
 * <p><i> 
 * GRC := UI8[1..8] <0..255><br>
 * with<br>
 * <0> := acknowledgement<br>
 * <1> := invalid GIN specified<br>
 * <2> := requested data does not exist<br>
 * <3> := data is not available, try again later<br>
 * <4> := verify error whilst changing setting<br>
 * <5> := out of range whilst changing setting<br>
 * <6> := size of entry too large<br>
 * <7> := too many commands<br>
 * <8> := entry is read-only<br>
 * <9> := setting is password protected<br>
 * <10> := local setting in progress<br>
 * <11> := error with following description<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class GRC extends InformationElement
{
	/**
	 * constructs GRC object<br>
	 * generic reply code 7.2.6.36
	 *
	 * @param octet octet from link user data
	 */
	public GRC(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		// computes invalidity
		invalidity = (short)(((((Short)dataset[0]).shortValue()!=0)?1:0) << 5);
		if (invalidity!=0 )invalidity |= 2;
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */
	public GRC()
	{
		super(1,1);
	}
	
};	// end GRC.java