/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.INT
 * File     INT.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Interval between information elements 7.2.6.7<b><br>
 * (information element used by asdu id 26) <br>
 * <p><i>
 * INT := UI16[1..16] <1..65535> <br>
 * The interval for acquisition of the single information elements is the same for all disturbance data. It is
 * listed in microseconds.
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class INT extends InformationElement
{
	/**
	 * constructs INT object<br>
	 * interval between information elements 7.2.6.7
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public INT(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public INT()
	{
		super(2,1);
	}
	
};	// end INT.java