/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.MEA
 * File     MEA.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.lib.math.Math32FP;
import br.org.scadabr.protocol.iec101.common.InformationElement;


/**
 * <p>
 * <b>Measurand with quality descriptor 7.2.6.8</b><br>
 * <p><i>
 * MEA := CP16 {OV, ER, RES, MVAL}<br>
 * with<br>
 * OV := BS1[1]<br>
 * <0> := no overflow<br>
 * <1> := overflow<br>
 * ER := BS1 [2] <0..1><br>
 * <0> := MVAL valid<br>
 * <1> := MVAL invalid<br>
 * RES := BS1 [3] not used (always <0>)<br>
 * MVAL := F13 [4..16] <-1..+1-2 12 ><br>
 * In the case of overflow of MVAL it is set to its maximum positive or maximum negative value<br>
 * respectively, in addition to OV :=1. The maximum MVAL can either be 1,2 or 2,4  the rated value.<br>
 * Other formats and ranges can be used with generic services.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class MEA extends InformationElement
{
	/**
	 * MEA value
	 */
	
	/**
	 * overflow
	 */
	//private boolean OV;		--> dataset[0]
	
	/**
	 * error on measured value
	 */
	//private boolean ER;		--> dataset[1]
	
	/**
	 * reserved
	 * must be 0
	 */
	//private boolean RES;		--> dataset[2]

	/**
	 * constructs MEA object<br>
	 * measurand with quality descriptor 7.2.6.8
	 *
	 * @param octet1 first octet from link user data
	 * @param octet2 second octet from link user data
	 */
	public MEA(short octet1, short octet2)
	{
		super(2,4);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
	
	protected void decode()
	{
		dataset[0] = new Boolean(((octetset[0] & 0x01)==1)?true:false);
		dataset[1] = new Boolean((((octetset[0] & 0x02) >> 1)==1)?true:false);
		dataset[2] = new Boolean((((octetset[0] & 0x04) >> 2)==1)?true:false);
		// 1.2 or 2.4 factor
		dataset[3] = new Math32FP( (( octetset[0] & 0xF8 ) >> 3) | (octetset[1] << 5));
		// computes invalidity
		invalidity = (short)( (((((Boolean)dataset[0]).booleanValue())?1:0) << 4) | (((((Boolean)dataset[1]).booleanValue())?1:0) << 1) | (((((Boolean)dataset[2]).booleanValue())?1:0) << 5));
		if (invalidity!=0)invalidity |= 2;

	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = (short)((((Boolean)dataset[0]).booleanValue()?1:0) | ((((Boolean)dataset[1]).booleanValue()?1:0) << 1) | ((((Boolean)dataset[2]).booleanValue()?1:0) << 2) | ((((Math32FP)dataset[3]).fpValue() & 0x1F) << 3) );
		// rapport 1.2 ou 2.4
		octetset[1] = (short)( (((Math32FP)dataset[3]).fpValue() & 0xFFE0) >> 5);
	}
	
	/**
	 * default constructor
	 */	
	public MEA()
	{
		super(2,4);
	}
};	// end MEA.java