/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NDV
 * File     NDV.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of relevant disturbance values per ASDU 7.2.6.14</b><br>
 * (information element used by asdu id ..) <br>
 * <p><i>
 * NDV := UI8[1..8] <1..255> <br>
 * with	<1 .. 25> := used <br>
 * 		<26 .. 255> := not used <br>
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NDV extends InformationElement
{
	/**
	 * constructs NDV object<br>
	 * number of relevant disturbance values per ASDU 7.2.6.14
	 *
	 * @param octet octet from link user data
	 */
	public NDV(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		// computes invalidity
		invalidity = (short)(((((Short)dataset[0]).shortValue()>25)?1:0) << 5);
		if (invalidity!=0 )invalidity |= 2;
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public NDV()
	{
		super(1,1);
	}
	
};	// end NDV.java