/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NFE
 * File     NFE.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of the ASDU first information element 7.2.6.9</b><br>
 * <p><i>
 * NFE := UI16[1..16] <0..65535><br>
 * All single disturbance values of a channel (file) have consecutive numbers and are transmitted in
 * uniform portions. Within an ASDU they are transmitted with consecutive increasing numbers. To be
 * able to correctly reassemble the file, the number of the first disturbance value (first information ele-ment)
 * of the ASDU is shown.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NFE extends InformationElement
{
	/**
	 * NFE value
	 *
	 */


	/** constructs NFE object<br>
	  * number of asdu first information element 7.2.6.9
	  *
	  * @param octet1 first octet from link user data
	  * @param octet2 second octet from link user data
	  */
	public NFE(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public NFE()
	{
		super(2,1);
	}
	
};	// end NFE.java