/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NGD
 * File     NGD.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of generic data sets 7.2.6.30</b><br>
 * <p><i>
 * NGD := CP8 {NO, COUNT, CONT}<br>
 * with<br>
 * NO := UI6[1..6] <0..63><br>
 * COUNT := BS1[7] <0..1><br>
 * <0..1> := one bit counter of ASDUs with same RII<br>
 * CONT := BS1[8] <0..1><br>
 * <0> := no further ASDU with same RII will follow<br>
 * <1> := further ASDUs with same RII following<br>
 * The initial value of COUNT is zero.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NGD extends InformationElement
{
	/** NGD octet value
	  *
	  */
	
	/**
	  *
	  */
	//private byte NO;			--> dataset[0]
	
	/**
	  */
	//private boolean COUNT;	--> dataset[1]
	
	/**
	  */
	//private boolean CONT;		--> dataset[2]
	
	public byte getNO()
	{
		return ((Byte)dataset[0]).byteValue();
	}
	
	/** constructs NGD object<br>
	  * number of generic datasets 7.2.6.30
	  * @param octet octet from link user data
	  */
	public NGD(short octet)
	{
		super(1,3);
		octetset[0] = octet;
		decode();
	}
			
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{		
		dataset[0] = new Byte((byte)(octetset[0] & 0x003F));
		dataset[1] = new Boolean((((octetset[0] & 0x0040)>>6)==0)?false:true);
		dataset[2] = new Boolean((((octetset[0] & 0x0080)>>7)==0)?false:true);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = (short) (((Byte)dataset[0]).byteValue() | ((((Boolean)dataset[1]).booleanValue()?1:0) << 6) |((((Boolean)dataset[2]).booleanValue()?1:0) << 7));
	}
	
	/**
	 * default constructor _ called by asdu writer?
	 */
	public NGD()
	{
		super(1,3);
		
		// selon l'info qu'on a  envoyer, doit avoir un max pour une
		// trame, alors on dcoupe en plusieurs et on maj les vars NO COUNT CONT
	}
	
};	// end NGD.java