/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NOC
 * File     NOC.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of channels 7.2.6.10</b><br>
 * (information element used by asdu id 26)<br>
 * <p><i>
 * NOC := UI8[1..8] <0..255><br>
 * This octet indicates the number of analogue channels of a set of transmission data being ready for
 * transmission.
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NOC extends InformationElement
{
	/**
	 * constructs NOC object<br>
	 * number of channels 7.2.6.10
	 *
	 * @param octet octet from link user data
	 */
	public NOC(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public NOC()
	{
		super(1,1);
	}
	
};	// end NOC.java