/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NOE
 * File     NOE.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of information elements of a channel 7.2.6.11</b><br>
 * <p><i>
 * NOE := UI16[1..16] <1..65535><br>
 * All channels contain the same number of information elements. This number is transmitted in ASDU 26
 * ready for transmission of disturbance data, being valid for all channels.
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NOE extends InformationElement
{
	/**
	 * <p>
	 * constructs NOE object<br>
	 * number of information elements of a channel 7.2.6.11
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public NOE(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public NOE()
	{
		super(2,1);
	}
	
};	// end NOE.java