/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NOF
 * File     NOF.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * <b>Number of grid faults 7.2.6.12</b><br>
 * (information element used by asdu id 26) <br>
 * <p><i>
 * NOF := UI16[1..16] <0..65535> <br>
 * NOTE  A grid fault, for example a short circuit, might cause several faults with trip and auto-reclosing, each fault being
 * identified by an increased fault number. In this case, the grid fault number remains the same. The grid fault number need not
 * be reset nor preset.<br>
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NOF extends InformationElement
{
	/**
	 * constructs NOF object<br>
	 * number of grid faults 7.2.6.12
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public NOF(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */
	public NOF()
	{
		super(2,1);
	}
	
};	// end NOF.java