/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NOG
 * File     NOG.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Number of generic identifications 7.2.6.37</b><br>
 * <p><i>
 * NOG := UI8[1..8] <0..255><br>
 * A generic identification always consists of the generic identification number (GIN) and the kind of
 * description (KOD).<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NOG extends InformationElement
{
	/**
	 * <p>
	 * constructs NOG object<br>
	 * number of generic identifications 7.2.6.37
	 *
	 * @param octet value of 'physical' data
	 */
	public NOG(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public NOG()
	{
		super(1,1);
	}
	
};	// end NOG.java