/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.NOT
 * File     NOT.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Number of tags 7.2.6.13</b><br>
 * <p><i>
 * NOT := UI8[1..8] <1..255><br>
 * This octet indicates the number of tags to be transmitted per ASDU.<br>
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class NOT extends InformationElement
{
	/**
	 * NOT value
	 */
	public short value()
	{
		return octetset[0];
	}


	/**
	 * constructs NOT object<br>
	 * number of tags 7.2.6.13
	 *
	 * @param octet value of 'physical' data
	 */
	public NOT(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public NOT()
	{
		super(1,1);
	}
	
};	// end NOT.java