/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.RET
 * File     RET.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Relative time 7.2.6.15</b><br>
 * <p><i>
 * RET := UI16 [1..16] <0..65535><br>
 * The relative time is reset at the beginning of a short circuit. It indicates the time in milliseconds from the
 * start/pick-up of the protection equipment up to the present time.<br>
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class RET extends InformationElement
{
	/**
	 * <p>
	 * constructs RET object<br>
	 * relative time 7.2.6.15
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public RET(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 )); 
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */
	public RET()
	{
		super(2,1);
	}
	
};	// end RET.java