/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.RFA
 * File     RFA.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.lib.math.Float;
import br.org.scadabr.protocol.iec101.common.InformationElement;
/**
 * <b>Reference factor 7.2.6.16</b><br>
 * <p><i>
 * RFA := R32.23{Fraction, Exponent, Sign}<br>
 * The disturbance values are transmitted as raw values. The reference factor shows the relationship
 * between raw value and secondary value.<br>
 * Reference factor F = raw value / secondary value<br>
 * The primary value is the secondary value multiplied with the ratio 'rated primary value' and 'rated
 * secondary value'.<br>
 * Primary value  = secondary value x (rated primary value / rated secondary value) = ( raw value / reference factor ) x (rated primary value / rated secondary value)<br>
 * </i>
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class RFA extends InformationElement
{
	/**
	 * <p>
	 * constructs RFA object<br>
	 * reference factor 7.2.6.16
	 *
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 * @param octet3 third octet
	 * @param octet4 fourth octet
	 */
	public RFA(short octet1, short octet2, short octet3, short octet4)
	{
		super(4,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		octetset[2] = octet3;
		octetset[3] = octet4;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */		
	protected void decode()
	{
		//int fraction = octetset[0] | (octetset[1] << 8) | ((octetset[2] & 0x7F) << 16);
		//short exposant = (short)(((octetset[2] & 0x80) >> 7) | ((octetset[3] & 0x7F) << 1));
		//boolean signe = (((octetset[3]&0x80)>>7)==1)?true:false;
		dataset[0] = new Float(Float.intBitsToFloat(octetset[0] | (octetset[1]<<8) | (octetset[2]<<16) | (octetset[3]<<24)));
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = Float.floatToIntBits(((Float)dataset[0]).floatValue());
		octetset[0] = (short)((integer & 0x000000FF));
		octetset[0] = (short)((integer & 0x0000FF00) >> 8);
		octetset[0] = (short)((integer & 0x00FF0000) >> 16);
		octetset[0] = (short)((integer & 0xFF000000) >> 24);
	}
	
	/**
	 * default constructor
	 */
	public RFA()
	{
		super(4,1);
	}

};	// end RFA.java