/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.RII
 * File     RII.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <p>
 * Return Information Identifier 7.2.6.19<br>
 * <p><i>
 * RII := UI8 [1..8] <0..255><br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class RII extends InformationElement
{
	/** 
	 * RII value<br>
	 * general command case: not signifiant for slave station
	 */

	/**
	 * constructs RII object (reader factory)<br>
	 * return information identifier 7.2.6.19
	 * @param octet value of 'physical' data
	 */
	public RII(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * constructs RII object (writer factory)<br>
	 * return information identifier 7.2.6.19
	 */	
	public RII()
	{
		super(1,1);
	}
	
};	// end RII.java