/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.RSV
 * File     RSV.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.lib.math.Float;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.InformationElement;
/**
 * <b>Rated secondary value 7.2.6.18</b><br>
 * <p><i>
 * RSV := R32.23{Fraction, Exponent, Sign}<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class RSV extends InformationElement
{
	//private int fraction; // 2(-23) to 2(-1)
	//private short exposant; // 2(0) to 2(7)
	//private boolean signe;
	

	/** constructs RSV object<br>
	  * rated secondary value 7.2.6.18
	  * @param octet1 value of first octet of 'physical' data
	  * @param octet2 second octet
	  * @param octet3 third octet
	  * @param octet4 fourth octet
	  */
	public RSV(short octet1, short octet2, short octet3, short octet4)
	{
		super(4,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		octetset[2] = octet3;
		octetset[3] = octet4;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */		
	protected void decode()
	{
		//int fraction = octetset[0] | (octetset[1] << 8) | ((octetset[2] & 0x7F) << 16);
		//short exposant = (short)(((octetset[2] & 0x80) >> 7) | ((octetset[3] & 0x7F) << 1));
		//boolean signe = (((octetset[3]&0x80)>>7)==1)?true:false;
		if(CodeProperties.AJILE)
			dataset[0] = new Float(Float.intBitsToFloat(octetset[0] | (octetset[1]<<8) | (octetset[2]<<16) | (octetset[3]<<24)));
		else
			dataset[0] = new java.lang.Float(java.lang.Float.intBitsToFloat(octetset[0] | (octetset[1]<<8) | (octetset[2]<<16) | (octetset[3]<<24)));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = Float.floatToIntBits(((Float)dataset[0]).floatValue());
		octetset[0] = (short)((integer & 0x000000FF));
		octetset[0] = (short)((integer & 0x0000FF00) >> 8);
		octetset[0] = (short)((integer & 0x00FF0000) >> 16);
		octetset[0] = (short)((integer & 0xFF000000) >> 24);
	}
	
	/**
	 * default constructor
	 */
	public RSV()
	{
		super(4,1);
	}

};	// end RSV.java