/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.SDV
 * File     SDV.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.lib.math.Math32FP;
import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Single disturbance value 7.2.6.22</b><br>
 * <p><i>
 * SDV := F16[1..16] <-1..+1-2 15 ><br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class SDV extends InformationElement
{
	/**
	 * SDV octet value
	 */
	//private int value;			// 2(-15) to 2(-1)
	//private boolean signe;
	

	/**
	 * constructs SDV object<br>
	 * single disturbance value 7.2.6.22
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public SDV(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		
		decode();
	}

	/**					 
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */
	protected void decode()
	{
		dataset[0] = new Math32FP((short)( octetset[0] | (octetset[1] <<8)));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
 		octetset[0] = (short)(((Math32FP)dataset[0]).fpValue() & 0x00FF);
		octetset[1] = (short)((((Math32FP)dataset[0]).fpValue() & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public SDV()
	{
		super(2,1);
	}
	
};	// end SDV.java