/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.SIN
 * File     SIN.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Supplementary information 7.2.6.23</b><br>
 * <p><i>
 * SIN := UI8 [1..8] <0..255><br>
 * SUPPLEMENTARY INFORMATION is used as follows:<br>
 * 		CAUSE OF TRANSMISSION = general interrogation<br>
 * SIN := SCAN NUMBER of the GI initiating ASDU<br>
 * 		CAUSE OF TRANSMISSION = positive or negative acknowledgement of command<br>
 * SIN := RETURN INFORMATION IDENTIFIER of the command message<br>
 * 		CAUSES OF TRANSMISSION = other<br>
 * SIN := not relevant<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class SIN extends InformationElement
{	 
	/**
	 * TODO: if sin != actual gi's scn -> ignore received frame
	 */


	/**
	 * constructs SIN object<br>
	 * supplementary information 7.2.6.23
	 * @param octet value of 'physical' data
	 */
	public SIN(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public SIN()
	{
		super(1,1);
	}
	
};	// end SIN.java