/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.TAP
 * File     TAP.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Tag position 7.2.6.25</b><br>
 * <p><i>
 * TAP := UI16[1..16] <0..65535>
 * These two octets show the position of the tag within the disturbance data set. The number is the
 * distance of the tag from the first element of the disturbance data set, coded as number of information
 * elements modulo 65536. The position of the first tag is zero.<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class TAP extends InformationElement
{
	/**
	 * constructs TAP object<br>
	 * tag position 7.2.6.25
	 * @param octet1 value of first octet of 'physical' data
	 * @param octet2 second octet
	 */
	public TAP(short octet1, short octet2)
	{
		super(2,1);
		octetset[0] = octet1;
		octetset[1] = octet2;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Integer(( octetset[0] ) | ( octetset[1] << 8 ));
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		int integer = ((Integer)dataset[0]).intValue();
		octetset[0] = (short)((integer & 0x00FF));
		octetset[1] = (short)((integer & 0xFF00) >> 8);
	}
	
	/**
	 * default constructor
	 */	
	public TAP()
	{
		super(2,1);
	}
	
};	// end TAP.java