/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.TOO
 * File     TOO.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Type of order 7.2.6.26</b><br>
 * <p><i>
 * TOO := UI8[1..8] <1..255> <br>
 * with <1> := selection of fault <br>
 * <2> 		:= request for disturbance data <br>
 * <3> 		:= abortion of disturbance data <br>
 * <4..7> 	:= reserved <br>
 * <8> 		:= request for channel <br>
 * <9> 		:= abortion of channel <br>
 * <10..15> := reserved <br>
 * <16> 	:= request for tags <br>
 * <17> 	:= abortion of tags <br>
 * <18..23> := reserved <br>
 * <24> 	:= request for list of recorded disturbances <br>
 * <25..31> := reserved <br>
 * <32> 	:= end of disturbance data transmission without abortion <br>
 * <33> 	:= end of disturbance data transmission with abortion by control system <br>
 * <34> 	:= end of disturbance data transmission with abortion by the protection equipment <br>
 * <35> 	:= end of channel transmission without abortion <br>
 * <36> 	:= end of channel transmission with abortion by control system <br>
 * <37> 	:= end of channel transmission with abortion by the protection quipment <br>
 * <38> 	:= end of tag transmission without abortion <br>
 * <39> 	:= end of tag transmission with abortion by control system <br>
 * <40> 	:= end of tag transmission with abortion by the protection equipment <br>
 * <41..63> := reserved <br>
 * <64> 	:= disturbance data transmitted successfully (positive) <br>
 * <65> 	:= disturbance data transmitted not successfully (negative) <br>
 * <66> 	:= channel transmitted successfully (positive) <br>
 * <67> 	:= channel transmitted not successfully (negative) <br>
 * <68> 	:= tags transmitted successfully (positive) <br>
 * <69> 	:= tags transmitted not successfully (negative) <br>
 * <70..255>:= reserved <br>
 * TOO specifies the type of order, for example selection, request, abort of transmission of disturbance
 * data, channels, tags, and list of recorded disturbances. The following ranges of TOO are used with the
 * different ASDUs: <br>
 * TOO <1..31>	used with ASDU 24: order for disturbance data transmission <br>
 * TOO <32..63>	used with ASDU 31: end of disturbance data transmission <br>
 * TOO <64..95>	used with ASDU 25: acknowledgement for disturbance data transmission <br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class TOO extends InformationElement
{
	/**
	 * constructs TOO object<br>
	 * type of order 7.2.6.26
	 * @param octet value of 'physical' data
	 */
	public TOO(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		// computes invalidity
		switch(((Short)dataset[0]).shortValue())
		{
		case 3:
		case 9:
		case 17:
		case 33:
		case 34:
		case 36:
		case 37:
		case 39:
		case 40:
		case 65:
		case 67:
		case 69:
			invalidity = 0x02;
			break;
		default:
			break;
		}
		if (invalidity!=0 )invalidity |= 2;
	}
	
	/**
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public TOO()
	{
		super(1,1);
	}
	
};	// end TOO.java