/*
 * Class    br.org.scadabr.protocol.iec101.common103.information.TOV
 * File     TOV.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.common103.information;

import br.org.scadabr.protocol.iec101.common.InformationElement;

/**
 * <b>Type of disturbance values 7.2.6.27</b><br>
 * <p><i>
 * TOV := UI8[1..8] <0..255><br>
 * with	<0> := not used<br>
 * 		<1> := instantaneous values<br>
 * 		<2..255> := not used<br>
 * </i>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class TOV extends InformationElement
{
	/**
	 * constructs TOV object<br>
	 * type of disturbance values 7.2.6.27
	 * @param octet value of 'physical' data
	 */
	public TOV(short octet)
	{
		super(1,1);
		octetset[0] = octet;
		decode();
	}
		
	/**
	 * <p>
	 * builds sub information data from 'physical' data.
	 * called when physical data is all known
	 */	
	protected void decode()
	{
		dataset[0] = new Short(octetset[0]);
		// computes invalidity
		invalidity = (short)(((((Short)dataset[0]).shortValue()!=1)?1:0) << 5);
		if (invalidity!=0 )invalidity |= 2;
	}
	
	/**
	 * <p>
	 * builds  'physical' data from sub information data.
	 * called when sub information data is all known
	 */	
	protected void code()
	{
		octetset[0] = ((Short)dataset[0]).shortValue();
	}
	
	/**
	 * default constructor
	 */	
	public TOV()
	{
		super(1,1);
	}
	
};	// end TOV.java