/*
 * Class	br.org.scadabr.protocol.iec101.master.MAppService
 * File		MAppService.java
 * Author	lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduKey;
import br.org.scadabr.protocol.iec101.common.Context;
import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;

/**
 * <p>
 * This class specializes general application procedure for master application.
 * <p>
 * Master specialization implies add of an expected asdu to be returned by
 * the secondary (slave) station.
 * 
 * @author	lionnel cauvy
 * @version	1.0
 * @date	28.II.03
 */
public class MAppService extends AppService
{
	/**
	 *
	 */
	public final static short	DEFAULT_VSQ	= 129;
	public final static int		DEFAULT_CAA	= 1;
	/**
	 * <p>
	 * Required asdu to be returned.
	 */
	public AsduKey expectedAsdu;
	/**
	 * <p>
	 * Related identifier. Contains a type & a priority.
	 */
	public MSvcId identifier;
	/**
	 * <p>
	 * Automatically launches polling data class 1 to ensure procedure
	 * completion.
	 */
	public boolean autoPoll = true;
	/**
	 * <p>
	 * This allows to maintain polling data class 1 until a confirm nack
	 * (eg data not available) is returned by secondary station. Beware,
	 * doing this implies that application doesn't matter ACD bit value
	 * in frame returned by remote secondary. In fact, this is very useful
	 * to face real equipments...
	 */
	public boolean pollUntilNack = false;
	/**
	 * <p>
	 * Instanciates a procedure for master application.
	 *
	 * @param asdu			The asdu to send to a remote.
	 * @param svc			The related link service.
	 * @param identifier	The procedure identification.
	 * @param notifier		The database element that will trigger the procedure.
	 * @param context		The execution context. Give current values to update during asdu building.
	 * @param expectedAsdu	Required asdu to validate procedure.
	 */
	public MAppService(MapAsdu asdu, MLnkService svc, MSvcId identifier, DbLocator notifier, Context context, AsduKey expectedAsdu)
	{
		super(asdu,svc,notifier,context);
		this.identifier		= identifier;
		this.expectedAsdu	= expectedAsdu;
		if(expectedAsdu==null)
			autoPoll = false;
	}	
}	// end MAppService.java