/*
 * Class    br.org.scadabr.protocol.iec101.master.MApplicationLayer
 * File     MApplicationLayer.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master;

import java.util.Enumeration;
import java.util.Hashtable;

import br.org.scadabr.protocol.iec101.common.ApplicationLayer;
import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.UserData;
import br.org.scadabr.protocol.iec101.session.Response;

/**
 * <p>
 * This class defines an implementation of IEC 608705 application layer from EPA
 * model.<br>
 * Application layer is in charge of:<br>
 * - application service data units (iec 60870-5-3).<br>
 * - information elements (iec 60870-5-4).
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class MApplicationLayer extends ApplicationLayer {
	/**
	 * Table of LnkManager referenced by address.
	 */
	public Hashtable manager;

	/**
	 * Enumerate all application managers and therefore all remote statiions
	 * defined for this application layer.
	 * 
	 * @return an enumeration of AppManger objects.
	 */
	public Enumeration enumerateManagers() {
		return manager.elements();
	}

	/**
	 * Start/restart application.
	 */
	public void start() {
		launch();
		// SINCE TESTS HAVE NOT BEEN MADE ON SEVERAL REMOTES
		// WE LET MANAGER PLAY....
		System.out.println("Starting REMOTES!");
		for (Enumeration e = manager.elements(); e.hasMoreElements();) {
			MAppManager mngr = (MAppManager) e.nextElement();
			mngr.start();
		}
	}

	/**
	 * Event raised by lower layer. Application layer must transform this data
	 * into an asdu before to evaluate it.
	 * 
	 * @param object
	 *            data from lower layer.
	 */
	protected synchronized void lowerEvent(Object object)
			throws IEC608705Exception {
		try {
			UserData data = (UserData) object;
			MAppManager mngr = (MAppManager) manager.get(new Integer(
					data.address));
			Asdu asdu = null;
			try {
				asdu = reader.build(object, mngr.prange);
			} catch (Exception e) {
				e.printStackTrace();
				System.out.println("Erro ao decodificar frame!");
			}
			Response response = new Response();
			response.setCot(asdu.cot);
			mngr.responseReceived(response);
			asdu = null;
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("Erro ao desconhecido!");
		}
	}

	/**
	 * Allocates a new application layer for master application.
	 * 
	 * @param size
	 *            Number of managers (eg remotes) to take into account.
	 */
	public MApplicationLayer(int size) {
		// remote managers
		manager = new Hashtable(size);
	}
} // end MApplicationLayer.java