/*
 * Class    br.org.scadabr.protocol.iec101.master.MLnkManager
 * File     MLnkManager.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master;

import br.org.scadabr.protocol.iec101.common.LnkManager;

/**
 * <p>
 * This class defines specialization of a link manager for master applications.
 * <p>
 * Only fixes a primary direction to set on control field hold by frame to send
 * to a remote station. Adds also a flag to indicate polling measures (data
 * class 2) state of application according to the remote we manage here.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class MLnkManager extends LnkManager {
	/**
	 * Final static variable to identify primary direction to set.
	 */
	public static final boolean PRM = true;
	/**
	 * Final static flag to indicate link service fail case.
	 */
	public static final boolean KO = false;
	/**
	 * Final static flag to indicate link service success.
	 */
	public static final boolean OK = true;
	/**
	 * Maximum number of asdus to get during a data class 2 polling. Fixed by
	 * configuration.
	 */
	public int nbToPoll = 100;
	/**
	 * Flag to indicate polling data class 2 state.
	 */
	public boolean polling = true;
	/**
	 * Polling data class 2 counter.
	 */
	public int pollCount;
	/**
	 * Success or fail case link service flag.
	 */
	public boolean result;

	/**
	 * This method is called when link layer succeed last service. Notice link
	 * manager informs related application manager of its success. This MUST be
	 * done to allow application manager to launch a new procedure.
	 */
	public void succeed() {
		state = MLnkManager.LAZY;
		result = MLnkManager.OK;
		pollCount = nbToPoll;
		polling = true;
		((MAppManager) app).manage();
	}

	public void poll1() {
		System.out.println("MLnkManager - poll1");
		state = MLnkManager.WORK;
		current = new MPoll1LnkSvc(this);
		layer.receive(current);
	}

	/**
	 * Instanciates a link manager for master local link layer.
	 * 
	 * @param nbToPoll
	 *            Max number of asdus to poll during a data class 2 polling.
	 * @param address
	 *            Remote address this manager will handle.
	 */
	public MLnkManager(MAppManager app, int nbToPoll, int address) {
		super(address);
		this.app = app;
		this.pollCount = this.nbToPoll = nbToPoll;
	}
} // end MLnkManager.java
