/*
 * Class	br.org.scadabr.protocol.iec101.master.MLnkService
 * File		MLnkService.java
 * Author	lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master;

import br.org.scadabr.protocol.iec101.common.ControlField;
import br.org.scadabr.protocol.iec101.common.LnkService;

/**
 * <p>
 * This class defines service at link layer level for master application.
 * <p>
 * Master link services add on one hand Frame Count Bit & Frame Count Bit
 * Validity to the super class. Also defines both right and wrong answers
 * that a remote (slave one) can return to local master application.
 * <p>
 * For example, if we instanciate a service SEND CONFIRM USER DATA, we can
 * expect CONFIRM ACK (right) or CONFIRM NACK (wrong) answers from a remote secondary station.
 * 
 * @author	lionnel cauvy
 * @version	1.0
 * @date	28.II.03
 */
public class MLnkService extends LnkService
{
	/**
	 * Final static identifier for not used field.
	 */
	protected final static byte NOTUSED	= 0x7F;
	/**
	 * Expected 'right' answer for a master application.
	 */
	public byte expectedOk;
	/**
	 * Expected 'wrong' answer for a master application.
	 */
	public byte expectedKo;
	/**
	 * Check real answer according to expected right or wrong answers.
	 *
	 * @param answer	The function returned by remote.
	 *
	 * @return a boolean true if answer matches, false if there's a problem
	 */
	public int check(byte answer)
	{
		if(answer==expectedOk)
			return 0;
		else if(answer==expectedKo)
			return 1;
		else
			return 2;
	}
	/**
	 * Allocates a link service object with expected ok and ko answers.
	 *
	 * @param order			The link service to build.
	 * @param address		Remote/local address for a master/slave application.
	 * @param fcb			Frame count bit.
	 * @param fcv			Frame count bit validity.
	 * @param expectedOk	Expected OK link service contained by answer.
	 * @param expectedKo	Expected KO link service contained by answer.
	 */
	public MLnkService(byte order, int address, boolean fcb, boolean fcv, byte expectedOk, byte expectedKo)
	{
		super(new ControlField(true,order,fcb,fcv),address);
		this.expectedOk	= expectedOk;
		this.expectedKo	= expectedKo;
	}
}	// end MLnkService.java