/*
 * Class    br.org.scadabr.protocol.iec101.master103.MAsduReader101
 * File     MAsduReader101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import java.util.Calendar;
import java.util.Enumeration;

import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.AsduReader;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.InformationElement;
import br.org.scadabr.protocol.iec101.common.InformationObject;
import br.org.scadabr.protocol.iec101.common.PrivateAsdu;
import br.org.scadabr.protocol.iec101.common.PrivateCot;
import br.org.scadabr.protocol.iec101.common.PrivateRange;
import br.org.scadabr.protocol.iec101.common.UserData;
import br.org.scadabr.protocol.iec101.common101.Asdu101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.common101.PrivateRange101;
import br.org.scadabr.protocol.iec101.common101.information.AFQ;
import br.org.scadabr.protocol.iec101.common101.information.BCR;
import br.org.scadabr.protocol.iec101.common101.information.BSI;
import br.org.scadabr.protocol.iec101.common101.information.CHS;
import br.org.scadabr.protocol.iec101.common101.information.COI;
import br.org.scadabr.protocol.iec101.common101.information.CP16Time2a;
import br.org.scadabr.protocol.iec101.common101.information.CP24Time2a;
import br.org.scadabr.protocol.iec101.common101.information.CP56Time2a;
import br.org.scadabr.protocol.iec101.common101.information.DIQ;
import br.org.scadabr.protocol.iec101.common101.information.FRQ;
import br.org.scadabr.protocol.iec101.common101.information.LOF;
import br.org.scadabr.protocol.iec101.common101.information.LOS;
import br.org.scadabr.protocol.iec101.common101.information.LSQ;
import br.org.scadabr.protocol.iec101.common101.information.NOF;
import br.org.scadabr.protocol.iec101.common101.information.NOS;
import br.org.scadabr.protocol.iec101.common101.information.NVA;
import br.org.scadabr.protocol.iec101.common101.information.OCI;
import br.org.scadabr.protocol.iec101.common101.information.OCTET;
import br.org.scadabr.protocol.iec101.common101.information.QDP;
import br.org.scadabr.protocol.iec101.common101.information.QDS;
import br.org.scadabr.protocol.iec101.common101.information.QOI;
import br.org.scadabr.protocol.iec101.common101.information.QPM;
import br.org.scadabr.protocol.iec101.common101.information.R32_23;
import br.org.scadabr.protocol.iec101.common101.information.SCD;
import br.org.scadabr.protocol.iec101.common101.information.SCQ;
import br.org.scadabr.protocol.iec101.common101.information.SEP;
import br.org.scadabr.protocol.iec101.common101.information.SIQ;
import br.org.scadabr.protocol.iec101.common101.information.SOF;
import br.org.scadabr.protocol.iec101.common101.information.SPE;
import br.org.scadabr.protocol.iec101.common101.information.SRQ;
import br.org.scadabr.protocol.iec101.common101.information.SVA;
import br.org.scadabr.protocol.iec101.common101.information.VTI;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.session.database.DataElement;

/**
 * This class describes mechanisms to read 101 asdus as a master station.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 18.II.03
 */

public class MAsduReader101 implements AsduReader {
	/**
	 * Cause of transmission field length.
	 */
	private byte szCot = 1;
	/**
	 * Common Address of Asdus field length.
	 */
	private byte szCaa = 2;
	/**
	 * Information Object Address field length.
	 */
	private byte szIoa = 2;
	/**
	 * Link address field length.
	 */
	private byte szLnk = 1;
	/**
	 * Private range reference.
	 */
	private PrivateRange prange;
	/**
	 * Current private asdu (if so) reference.
	 */
	private PrivateAsdu pasdu;
	/**
	 * Private reading flag
	 */
	private boolean privateRead;
	/**
	 * Asdu reference.
	 */
	private Asdu101 asdu;

	private MAppManager manager;

	/**
	 * Fills cause of transmission field in 101 asdu.
	 * 
	 * @param octet
	 *            cause of transmission value.
	 * 
	 * @return true or false according to this information standard compliance.
	 */
	protected boolean setCauseTransmission(short octet) {
		// there's a confirmation bit
		boolean valid = true;
		switch (octet) {
		case Interoperability101.COT_PERIODIC_CYCLIC:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ periodic, cyclic");
			break;
		case Interoperability101.COT_BACKGROUND_SCAN:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ background scan");
			break;
		case Interoperability101.COT_SPONTANEOUS:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ spontaneous");
			break;
		case Interoperability101.COT_INITIALIZED:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ initialized");
			break;
		case Interoperability101.COT_REQUEST:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ request or requested");
			break;
		case Interoperability101.COT_ACTIVATION:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ activation");
			break;
		case Interoperability101.COT_ACTIVATIONCONF:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ activation confirmation");
			break;
		case Interoperability101.COT_DEACTIVATION:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ deactivation");
			break;
		case Interoperability101.COT_DEACTIVATIONCONF:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ deactivation confirmation");
			break;
		case Interoperability101.COT_ACTIVATION_TERMINATION:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ activation termination");
			break;
		case Interoperability101.COT_RETURN_INFORMATION_REMOTE:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ return information caused by a remote command");
			break;
		case Interoperability101.COT_RETURN_INFORMATION_LOCAL:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ return information caused by a local command");
			break;
		case Interoperability101.COT_FILE_TRANSFER:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ file transfer");
			break;
		case Interoperability101.COT_INTERROGATION_GENERAL:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by general interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_1:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 1 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_2:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 2 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_3:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 3 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_4:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 4 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_5:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 5 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_6:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 6 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_7:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 7 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_8:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 8 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_9:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 9 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_10:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 10 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_11:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 11 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_12:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 12 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_13:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 13 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_14:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 14 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_15:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 15 interrogation");
			break;
		case Interoperability101.COT_INTERROGATION_GROUP_16:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ interrogated by group 16 interrogation");
			break;
		case Interoperability101.COT_REQUEST_GENERAL_COUNTER:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ requested by general counter request");
			break;
		case Interoperability101.COT_REQUEST_COUNTER_GROUP_1:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ requested by group 1 counter request");
			break;
		case Interoperability101.COT_REQUEST_COUNTER_GROUP_2:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ requested by group 2 counter request");
			break;
		case Interoperability101.COT_REQUEST_COUNTER_GROUP_3:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ requested by group 3 counter request");
			break;
		case Interoperability101.COT_REQUEST_COUNTER_GROUP_4:
			// CodeProperties.applog("[MAsduReader101] cause of transmission _ requested by group 4 counter request");
			break;
		default:
			// is this a private cause of transmission?
			if (prange != null) {
				PrivateCot pcot = prange.getCot(octet);
				if (pcot != null) {
					CodeProperties
							.applog("[MAsduReader101] cause of transmission");
					break;
				}
			}
			CodeProperties
					.applog("[MAsduReader101] cause of transmission _ unknown COT");
			valid = false;
			break;
		}

		if (valid)
			asdu.cot = octet;
		else
			reset();

		return valid;
	}

	/**
	 * Fills type identification field in 101 asdu. Private asdus can be
	 * detected here.
	 * 
	 * @param octet
	 *            type identification value.
	 * 
	 * @return true or false according to this information standard compliance.
	 */
	protected boolean setTypeIdentification(short octet) {
		boolean valid = true;
		switch (octet) {
		/** process information monitor direction */
		case Interoperability101.ID_M_SP_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ single point");
			break;
		case Interoperability101.ID_M_SP_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ single point");
			break;
		case Interoperability101.ID_M_SP_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ single point");
			break;
		case Interoperability101.ID_M_DP_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ double point");
			break;
		case Interoperability101.ID_M_DP_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ double point");
			break;
		case Interoperability101.ID_M_DP_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ double point");
			break;
		case Interoperability101.ID_M_ST_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ step position");
			break;
		case Interoperability101.ID_M_ST_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ step position");
			break;
		case Interoperability101.ID_M_ST_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ step position");
			break;
		case Interoperability101.ID_M_BO_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ bitstring");
			break;
		case Interoperability101.ID_M_BO_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ bitstring");
			break;
		case Interoperability101.ID_M_BO_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ bitstring");
			break;
		case Interoperability101.ID_M_ME_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ normalized measure");
			break;
		case Interoperability101.ID_M_ME_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ normalized measure");
			break;
		case Interoperability101.ID_M_ME_TD_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ normalized measure");
			break;
		case Interoperability101.ID_M_ME_NB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ scaled measure");
			break;
		case Interoperability101.ID_M_ME_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ scaled measure");
			break;
		case Interoperability101.ID_M_ME_TE_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ scaled measure");
			break;
		case Interoperability101.ID_M_ME_NC_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ floating point measure");
			break;
		case Interoperability101.ID_M_ME_TC_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ floating point measure");
			break;
		case Interoperability101.ID_M_ME_TF_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ floating point measure");
			break;
		case Interoperability101.ID_M_IT_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ integrating totals");
			break;
		case Interoperability101.ID_M_IT_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ integrating totals");
			break;
		case Interoperability101.ID_M_IT_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ integrating totals");
			break;
		case Interoperability101.ID_M_EP_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ event protection equipment");
			break;
		case Interoperability101.ID_M_EP_TD_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ event protection equipment");
			break;
		case Interoperability101.ID_M_EP_TB_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ packed start events protection equipment");
			break;
		case Interoperability101.ID_M_EP_TE_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ packed start events protection equipment");
			break;
		case Interoperability101.ID_M_EP_TC_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ packed output circuit protection equipmentt");
			break;
		case Interoperability101.ID_M_EP_TF_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ packed output circuit protection equipment");
			break;
		case Interoperability101.ID_M_PS_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ packed single point with status change detection");
			break;
		case Interoperability101.ID_M_ME_ND_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ measured value, normalized, no quality descriptor");
			break;
		case Interoperability101.ID_M_EI_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ end of initialization");
			break;
		case Interoperability101.ID_F_FR_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ file ready");
			break;
		case Interoperability101.ID_F_SR_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ section ready");
			break;
		case Interoperability101.ID_F_SC_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ call directory, select file, call file, call section");
			break;
		case Interoperability101.ID_F_LS_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ last section, last segment");
			break;
		case Interoperability101.ID_F_AF_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ last section, last segmentack file, ack section");
			break;
		case Interoperability101.ID_F_SG_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ segment");
			break;
		case Interoperability101.ID_F_DR_TA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ directory");
			break;
		case Interoperability101.ID_C_SC_NA_1:
		case Interoperability101.ID_C_DC_NA_1:
		case Interoperability101.ID_C_RC_NA_1:
		case Interoperability101.ID_C_SE_NA_1:
		case Interoperability101.ID_C_SE_NB_1:
		case Interoperability101.ID_C_SE_NC_1:
		case Interoperability101.ID_C_BO_NA_1:
			// case Interoperability101.ID_C_IC_NA_1:
		case Interoperability101.ID_C_CI_NA_1:
		case Interoperability101.ID_C_RD_NA_1:
			// case Interoperability101.ID_C_CS_NA_1:
		case Interoperability101.ID_C_TS_NA_1:
		case Interoperability101.ID_C_RP_NA_1:
		case Interoperability101.ID_C_CD_NA_1:
			// case Interoperability101.ID_P_ME_NA_1:
		case Interoperability101.ID_P_ME_NB_1:
		case Interoperability101.ID_P_ME_NC_1:
		case Interoperability101.ID_P_AC_NA_1:
			// CodeProperties
			// .applog("[MAsduReader101] type identification _ master receives control direction!");
			valid = true;
			break;
		case Interoperability101.ID_P_ME_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ parameter of measured value, normalized value");
			break;
		case Interoperability101.ID_C_CS_NA_1:
			// CodeProperties.applog("[MAsduReader101] type identification _ clock synchronization command");
			break;
		case Interoperability101.ID_C_IC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ interrogation command");
			break;
		default:
			// is this a private asdu?
			if (prange != null) {
				pasdu = prange.getAsdu(octet);
				if (pasdu != null) {
					privateRead = true;
					CodeProperties.applog("[MAsduReader101] Asdu type:" + octet
							+ " from private range.");
					break;
				}
			}
			// no sorry.....
			CodeProperties
					.applog("[MAsduReader101] type identification _ unknow type! "
							+ octet);
			valid = false;
			break;
		}
		asdu.id = octet;
		return valid;
	}

	/**
	 * Build main method inherited from asdu factory. Here, this completes a 101
	 * asdu.
	 * 
	 * @param object
	 *            short array to turn into an asdu.
	 * @param prange
	 *            The private range of related remote that sent this asdu.
	 * 
	 * @return the built asdu.
	 */
	public Asdu build(Object object, PrivateRange prange)
			throws IEC608705Exception {
		reset();
		this.prange = prange;
		if (object == null)
			return null;
		UserData user = (UserData) object;
		asdu = new Asdu101();
		asdu.address = user.address;
		if (user.data == null)
			return asdu;
		if (user.data.isEmpty())
			return asdu;

		short num = 0;
		byte numelt = 0;
		Enumeration data = user.data.elements();
		// DATA UNIT IDENTIFIER
		// TYPE IDENTIFICATION
		if (!setTypeIdentification(((Short) data.nextElement()).shortValue())) {
			reset();
			return null;
		}

		// VARIABLE STRUCTURE QUALIFIER
		short octet = ((Short) data.nextElement()).shortValue();

		asdu.vsq = octet;
		asdu.nio = (byte) (octet & MASK_NB);
		asdu.sq = (((short) (octet & MASK_SQ) >> 7) == 1) ? false : true;
		// CAUSE OF TRANSMISSION
		// (T101 COT SIZE UP TO 2 OCTETS)

		switch (szCot) {
		case 1:
			if (!setCauseTransmission(((Short) data.nextElement()).shortValue())) {
				reset();
				return null;
			}
			break;
		case 2:
			if (!setCauseTransmission(((Short) data.nextElement()).shortValue())) {
				reset();
				return null;
			}
			asdu.oa = ((Short) data.nextElement()).shortValue();
			break;
		}
		// COMMON ADDRESS
		// (T101 CAA SIZE UP TO 2 OCTETS)
		switch (szCaa) {
		case 1:
			asdu.caa = ((Short) data.nextElement()).shortValue();
			break;
		case 2:
			asdu.caa = (((Short) data.nextElement()).shortValue() & 0xFF)
					| ((((Short) data.nextElement()).shortValue() & 0xFF) << 8);
			break;
		}
		// System.out.println("VARIABLE STRUCTURE QUALIFIED " + octet);
		// System.out.println("Size of COT " + szCot);
		// System.out.println("ASDU COMMON ADDRESS: " + asdu.caa);
		// System.out.println("VARIABLE STRUCTURE QUALIFIER: " + octet);
		// System.out.println("VARIABLE STRUCTURE QUALIFIER: " + asdu.sq);
		if (!asdu.sq) {
			// 1 information object, n information elements
			// number from variable structure qualifier defines number n of
			// information elements
			// setInformationObjectStructure((byte)1,number);
			num = 1;
			numelt = (byte) asdu.nio;

			// System.out.println("Recebeu " + numelt + " ELEMENTOS!");
		} else {
			// n information objects, 1 information element per object
			// number from variable structure qualifier defines number n of
			// information objects
			// setInformationObjectStructure(number,(byte)1);
			num = asdu.nio;
			numelt = 1;
			// System.out.println("Recebeu " + num + " OBJETOS!");
		}
		if (!asdu.sq & asdu.nio == 0) {
			num = 0x00;
		}
		try {
			// id from type identification defines which elements to use
			for (short i = 0; i < num; i++) {
				InformationObject info = new InformationObject();
				switch (szIoa) {
				case 1:
					info.address = ((Short) data.nextElement()).shortValue();
					break;
				case 2:
					info.address = (((Short) data.nextElement()).shortValue() | (((Short) data
							.nextElement()).shortValue() << 8));
					break;
				case 3:
					info.address = (((Short) data.nextElement()).shortValue()
							| (((Short) data.nextElement()).shortValue() << 8) | (((Short) data
							.nextElement()).shortValue() << 16));
					break;
				default:
					CodeProperties
							.log("[MAsduReader101] Information Object Address Field size incorrect: "
									+ szIoa);
					reset();
					return null;
				}
				switch (asdu.id) {
				case Interoperability101.ID_P_ME_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QPM(((Short) data.nextElement())
								.shortValue()));
					}
					break;
				case Interoperability101.ID_C_CS_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;
				case Interoperability101.ID_C_IC_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new QOI(((Short) data.nextElement())
								.shortValue()));
					}
					break;
				case Interoperability101.ID_M_SP_NA_1:
					for (byte j = 0; j < numelt; j++) {
						DataElement dataElement = new DataElement();
						SIQ element;
						try {
							element = new SIQ(((Short) data.nextElement())
									.shortValue());
						} catch (Exception e) {
							element = null;
						}

						if (element != null) {
							dataElement.setTypeIdentification(asdu.id);
							dataElement.setAddress(info.address + j);
							dataElement.setValue(element.getValue());
							dataElement.setTimestamp(Calendar.getInstance()
									.getTimeInMillis());
							manager.dataReceived(dataElement);
						} else {
							System.out.println("Dado descartado!");
						}
					}
					break;

				case Interoperability101.ID_M_SP_TA_1:
					// SQ must be 0
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SIQ(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_SP_TB_1:
					// SQ must be 0
					for (byte j = 0; j < numelt; j++) {
						DataElement dataElement = new DataElement();
						SIQ element = new SIQ(((Short) data.nextElement())
								.shortValue());
						//
						CP56Time2a timestamp = new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue());

						dataElement.setTypeIdentification(asdu.id);
						dataElement.setAddress(info.address + j);
						dataElement.setValue(element.getValue());
						dataElement
								.setTimestamp(timestamp.getValue().getTime());
						manager.dataReceived(dataElement);
					}
					break;

				case Interoperability101.ID_M_DP_NA_1:
					for (byte j = 0; j < numelt; j++) {
						DataElement dataElement = new DataElement();
						DIQ element = null;
						try {
							element = new DIQ(((Short) data.nextElement())
									.shortValue());
							dataElement.setTypeIdentification(asdu.id);
							dataElement.setAddress(info.address + j);
							dataElement.setValue(element.getValue());
							dataElement.setTimestamp(Calendar.getInstance()
									.getTimeInMillis());
							manager.dataReceived(dataElement);
						} catch (Exception e) {
							e.printStackTrace();
							System.out.println("Erro");
						}
					}
					break;

				case Interoperability101.ID_M_DP_TA_1:
					// SQ must be 0
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new DIQ(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_DP_TB_1:
					// SQ must be 0
					for (byte j = 0; j < numelt; j++) {
						DataElement dataElement = new DataElement();
						DIQ element = null;
						try {
							element = new DIQ(((Short) data.nextElement())
									.shortValue());
							CP56Time2a timestamp = new CP56Time2a(((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue(), ((Short) data
									.nextElement()).shortValue());
							dataElement.setTypeIdentification(asdu.id);
							dataElement.setAddress(info.address + j);
							dataElement.setValue(element.getValue());
							dataElement.setTimestamp(timestamp.getValue()
									.getTime());
							manager.dataReceived(dataElement);
						} catch (Exception e) {
							System.out.println("Erro!");
						}
					}
					break;

				case Interoperability101.ID_M_ST_NA_1:
					for (byte j = 0; j < numelt; j++) {
						VTI element = null;
						QDS quality = null;
						try {
							DataElement dataElement = new DataElement();
							element = new VTI(((Short) data.nextElement())
									.shortValue());
							quality = new QDS(((Short) data.nextElement())
									.shortValue());
							dataElement.setTypeIdentification(asdu.id);
							dataElement.setAddress(info.address + j);
							dataElement.setValue(element.getValue());
							dataElement.setTimestamp(Calendar.getInstance()
									.getTimeInMillis());
							manager.dataReceived(dataElement);
						} catch (Exception e) {
							System.out.println("Erro");
						}

					}
					break;

				case Interoperability101.ID_M_ST_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new VTI(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ST_TB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new VTI(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_BO_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BSI(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_BO_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BSI(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_BO_TB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BSI(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_NA_1:
					for (byte j = 0; j < numelt; j++) {
						NVA element = null;
						QDS quality = null;
						try {
							DataElement dataElement = new DataElement();
							element = new NVA(((Short) data.nextElement())
									.shortValue(), ((Short) data.nextElement())
									.shortValue());
							quality = new QDS(((Short) data.nextElement())
									.shortValue());
							dataElement.setTypeIdentification(asdu.id);
							dataElement.setAddress(info.address + j);
							dataElement.setValue(element.getValue());
							dataElement.setTimestamp(Calendar.getInstance()
									.getTimeInMillis());
							manager.dataReceived(dataElement);
						} catch (Exception e) {
							System.out.println("Erro!");
						}
					}
					break;

				case Interoperability101.ID_M_ME_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_TD_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_NB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_TB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_TE_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_NC_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new R32_23(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_TC_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new R32_23(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_TF_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new R32_23(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_IT_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BCR(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_IT_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BCR(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_IT_TB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new BCR(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SEP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TD_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SEP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TB_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SPE(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TE_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SPE(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TC_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new OCI(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP24Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_EP_TF_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new OCI(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDP(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP16Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				case Interoperability101.ID_M_PS_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new SCD(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new QDS(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_ME_ND_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NVA(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_M_EI_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new COI(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_FR_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new LOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new FRQ(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_SR_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new NOS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new LOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new SRQ(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_SC_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new NOS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new SCQ(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_LS_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new NOS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new LSQ(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CHS(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_AF_NA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new NOS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new AFQ(((Short) data.nextElement())
								.shortValue()));
					}
					break;

				case Interoperability101.ID_F_SG_NA_1:
					for (byte j = 0; j < numelt; j++) {
						LOS los;
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new NOS(((Short) data.nextElement())
								.shortValue()));
						info.setElement(los = new LOS(((Short) data
								.nextElement()).shortValue()));
						for (short k = 0; k < los.octetset[0]; k++) {
							info.setElement(new OCTET(((Short) data
									.nextElement()).shortValue()));
						}
					}
					break;

				case Interoperability101.ID_F_DR_TA_1:
					for (byte j = 0; j < numelt; j++) {
						info.setElement(new NOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new LOF(((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue(), ((Short) data.nextElement())
								.shortValue()));
						info.setElement(new SOF(((Short) data.nextElement())
								.shortValue()));
						info.setElement(new CP56Time2a(((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue(), ((Short) data
								.nextElement()).shortValue()));
					}
					break;

				default:
					// is this private asdu?
					if (privateRead) {
						try {
							for (int z = 0; z < pasdu.ie.length; z++) {
								Class clazz = (Class) (PrivateRange101.CLASSES
										.get(pasdu.ie[z]));
								InformationElement elt = (InformationElement) clazz
										.newInstance();
								for (short y = 0; y < elt.octetset.length; y++)
									elt.setOctet(y,
											((Short) data.nextElement())
													.shortValue());
								info.setElement(elt);
							}
						} catch (InstantiationException e2) {
							CodeProperties
									.applog("[MAsduReader101] not instanciable "
											+ e2);
						} catch (IllegalAccessException e3) {
							CodeProperties
									.applog("[MAsduReader101] bad access " + e3);
						}
					}
					// no so sorry...
					else {
						// CodeProperties
						// .applog("[MAsduReader101] Unknown asdu type identification");
					}
					break;
				}

				/*
				 * for (Object iE : info.ie) { InformationElement x =
				 * (InformationElement) iE; System.out.println("Bytes Length: "
				 * +x.octetset.length); System.out.print ("Value: "); for (int j
				 * = 0; j < x.octetset.length; j++) {
				 * System.out.print(x.octetset[j]+" "); } System.out.println();
				 * 
				 * }
				 */
				asdu.add(info);
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new IEC608705Exception("IEC Exception: bad asdu format");
		}

		return asdu;

	}

	/**
	 * Reset factory
	 */
	protected void reset() {
		asdu = null;
		pasdu = null;
		privateRead = false;
	}

	/**
	 * Constructs a reader factory specialized for 101 master station.
	 * 
	 * @param szCot
	 *            Cause of transmission length (1-2).
	 * @param szCaa
	 *            Common address of asdus length (1-2).
	 * @param szIoa
	 *            Information object address length (1-2-3).
	 * @param szLnk
	 *            Link address size (1-2).
	 */
	public MAsduReader101(byte szCot, byte szCaa, byte szIoa, byte szLnk) {
		this.szCot = szCot;
		this.szCaa = szCaa;
		this.szIoa = szIoa;
		this.szLnk = szLnk;
	}

	public void setManager(MAppManager manager) {
		this.manager = manager;
	}

	public MAppManager getManager() {
		return manager;
	}
} // end MAsduReader101.java