/*
 * Class    br.org.scadabr.protocol.iec101.master101.MAsduWriter101
 * File     MAsduWriter101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduWriter;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.InformationElement;
import br.org.scadabr.protocol.iec101.common.InformationObject;
import br.org.scadabr.protocol.iec101.common.LnkService;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.common101.information.AFQ;
import br.org.scadabr.protocol.iec101.common101.information.BSI;
import br.org.scadabr.protocol.iec101.common101.information.CHS;
import br.org.scadabr.protocol.iec101.common101.information.CP16Time2a;
import br.org.scadabr.protocol.iec101.common101.information.CP56Time2a;
import br.org.scadabr.protocol.iec101.common101.information.DCO;
import br.org.scadabr.protocol.iec101.common101.information.FBP;
import br.org.scadabr.protocol.iec101.common101.information.FRQ;
import br.org.scadabr.protocol.iec101.common101.information.LOF;
import br.org.scadabr.protocol.iec101.common101.information.LOS;
import br.org.scadabr.protocol.iec101.common101.information.LSQ;
import br.org.scadabr.protocol.iec101.common101.information.NOF;
import br.org.scadabr.protocol.iec101.common101.information.NOS;
import br.org.scadabr.protocol.iec101.common101.information.NVA;
import br.org.scadabr.protocol.iec101.common101.information.OCTET;
import br.org.scadabr.protocol.iec101.common101.information.QCC;
import br.org.scadabr.protocol.iec101.common101.information.QOI;
import br.org.scadabr.protocol.iec101.common101.information.QOS;
import br.org.scadabr.protocol.iec101.common101.information.QPA;
import br.org.scadabr.protocol.iec101.common101.information.QPM;
import br.org.scadabr.protocol.iec101.common101.information.QRP;
import br.org.scadabr.protocol.iec101.common101.information.R32_23;
import br.org.scadabr.protocol.iec101.common101.information.RCO;
import br.org.scadabr.protocol.iec101.common101.information.SCO;
import br.org.scadabr.protocol.iec101.common101.information.SCQ;
import br.org.scadabr.protocol.iec101.common101.information.SOF;
import br.org.scadabr.protocol.iec101.common101.information.SRQ;
import br.org.scadabr.protocol.iec101.common101.information.SVA;
import br.org.scadabr.protocol.lib.math.Math32FP;

/**
 * This class defines mechanisms to build a 101 asdu to write to lower link
 * layer.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 18.II.03
 */

public class MAsduWriter101 implements AsduWriter {
	/**
	 * Cause of transmission field length.
	 */
	private byte szCot = 1;
	/**
	 * Common Address of Asdus field length.
	 */
	private byte szCaa = 1;
	/**
	 * Information Object Address field length.
	 */
	private byte szIoa = 1;
	/**
	 * Link address field length.
	 */
	private byte szLnk = 1;

	private int ioa = 0;

	/**
	 * Fills information objects in 103 asdu.
	 * 
	 * @param proc
	 *            Application procedure reference
	 */
	private InformationObject getIo(AppService proc) {
		InformationObject io = null;

		// information object address & numelt????

		MContext101 context = (MContext101) proc.context;
		ioa = 0;
		switch (proc.asdu.key.id) {
		case Interoperability101.ID_C_SC_NA_1:
			MCommandAppSvc101 params = (MCommandAppSvc101) proc;
			// CodeProperties.applog("[MAsduWriter101] type identification _ single command");
			io = new InformationObject();
			SCO command = new SCO();

			boolean value = true;
			if (params.state == 0)
				value = false;
			command.setData((short) 0, value);
			command.setData((short) 1, (Boolean) false);
			command.setData((short) 2, (byte) params.qualifier);
			command.setData((short) 3, !(Boolean) params.se);

			ioa = params.ioa;

			io.setElement(command);

			break;
		case Interoperability101.ID_C_DC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ double command");
			MCommandAppSvc101 dcoParams = (MCommandAppSvc101) proc;
			// CodeProperties.applog("[MAsduWriter101] type identification _ single command");
			DCO dco = new DCO();
			dco.setData((short) 0, (dcoParams.state == 0x01) ? (byte) 0x02
					: (byte) 0x01);
			dco.setData((short) 1, (Byte) dcoParams.qualifier);
			dco.setData((short) 2, !(Boolean) dcoParams.se);
			io = new InformationObject();
			io.setElement(dco);
			ioa = dcoParams.ioa;
			break;
		case Interoperability101.ID_C_RC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ regulating step command");
			io.setElement(new RCO());
			break;
		case Interoperability101.ID_C_SE_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ set point command, normalized value");
			MCommandAppSvc101 spParams = (MCommandAppSvc101) proc;
			NVA nva = new NVA();
			QOS qos = new QOS();
			Math32FP v = new Math32FP(spParams.setPointValue);
			nva.setData((short) 0, v);
			qos.setData((short) 0, (byte) spParams.qualifier);
			qos.setData((short) 1, !(Boolean) spParams.se);
			io = new InformationObject();
			io.setElement(nva);
			io.setElement(qos);
			ioa = spParams.ioa;
			break;
		case Interoperability101.ID_C_SE_NB_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ set point command, scaled value");
			io.setElement(new SVA());
			io.setElement(new QOS());
			break;
		case Interoperability101.ID_C_SE_NC_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ set point command, short floating point number");
			io.setElement(new R32_23());
			io.setElement(new QOS());
			break;
		case Interoperability101.ID_C_BO_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ bitsrting 32 bits");
			io.setElement(new BSI());// map
			break;
		case Interoperability101.ID_C_IC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ interrogation command");
			MGeneralInterrogationAppSvc101 svc = (MGeneralInterrogationAppSvc101) proc;
			io = new InformationObject();
			io.setElement(new QOI(svc.qoi));
			break;
		case Interoperability101.ID_C_CI_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ counter interrogation command");
			io.setElement(new QCC());
			break;
		case Interoperability101.ID_C_RD_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ read command");
			// nada
			break;
		case Interoperability101.ID_C_CS_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ clock synchronization command");
			io = new InformationObject();
			io.setElement(new CP56Time2a());
			break;
		case Interoperability101.ID_C_TS_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ test command");
			io = new InformationObject();
			io.setElement(new FBP());
			break;
		case Interoperability101.ID_C_RP_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ reset process command");
			io.setElement(new QRP());
			break;
		case Interoperability101.ID_C_CD_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ delay acquisition command");
			io = new InformationObject();
			io.setElement(new CP16Time2a());
			break;
		case Interoperability101.ID_P_ME_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ parameter of measured value, normalized value");
			io.setElement(new NVA());
			io.setElement(new QPM());
			break;
		case Interoperability101.ID_P_ME_NB_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ parameter of measured value, scaled value");
			io.setElement(new SVA());
			io.setElement(new QPM());
			break;
		case Interoperability101.ID_P_ME_NC_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ parameter of measured value, short floating point number");
			io.setElement(new R32_23()); // map
			io.setElement(new QPM()); // proc
			break;
		case Interoperability101.ID_P_AC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ parameter activation");
			io.setElement(new QPA());
			break;
		case Interoperability101.ID_F_FR_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ file ready");
			io.setElement(new NOF());
			io.setElement(new LOF());
			io.setElement(new FRQ());
			break;
		case Interoperability101.ID_F_SR_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ section ready");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new LOF());
			io.setElement(new SRQ());
			break;
		case Interoperability101.ID_F_SC_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ call directory, select file, call file, call section");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new SCQ());
			break;
		case Interoperability101.ID_F_LS_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ last section, last segment");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new LSQ());
			io.setElement(new CHS());
			break;
		case Interoperability101.ID_F_AF_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ last section, last segmentack file, ack section");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new AFQ());
			break;
		case Interoperability101.ID_F_SG_NA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ segment");
			LOS los;
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(los = new LOS());
			for (short j = 0; j < los.octetset[0]; j++) {
				io.setElement(new OCTET());
			}
			break;
		case Interoperability101.ID_F_DR_TA_1:
			// CodeProperties.applog("[MAsduWriter101] type identification _ directory");
			// VSQ gives number of elements !!!!
			io.setElement(new NOF());
			io.setElement(new LOF());
			io.setElement(new SOF());
			io.setElement(new CP56Time2a());
			break;
		default:
			CodeProperties.log("FUNCTION NOT IMPLEMENTED.");
			break;
		}

		return io;
	}

	/**
	 * Main method of this asdu writer factory for master 101 station.
	 * 
	 * @param proc
	 *            An application procedure to turn into link service.
	 * 
	 * @return the built asdu hold by a link service object.
	 */
	public LnkService build(AppService proc) {
		int size = 0;
		LnkService lnk = proc.svc;
		if (proc.asdu != null)// user data
		{
			AsduKey101 key = (AsduKey101) proc.asdu.key;
			InformationObject io = getIo(proc);
			for (int i = 0; i < io.ie.size(); i++) {
				InformationElement ie = (InformationElement) io.ie.elementAt(i);
				size += ie.octetset.length;
			}
			int i = 0;
			lnk.data = new short[2 + szCot + szCaa + szIoa + size];
			lnk.data[i++] = key.id;
			lnk.data[i++] = key.vsq;
			lnk.data[i++] = key.cot;
			if (szCot == 2)
				lnk.data[i++] = key.oa;
			switch (szCaa) {
			case 1:
				lnk.data[i++] = (short) key.caa;
				break;
			case 2:
				lnk.data[i++] = (short) ((key.caa & 0x00FF));
				lnk.data[i++] = (short) ((key.caa & 0xFF00) >> 8);
				break;
			}

			switch (szIoa) {
			case 1:
				lnk.data[i++] = (short) 1;
				break;
			case 2:
				lnk.data[i++] = (short) ((ioa & 0x00FF));
				lnk.data[i++] = (short) ((ioa & 0xFF00) >> 8);
				break;
			case 3:
				lnk.data[i++] = (short) ((ioa & 0x00FF));
				lnk.data[i++] = (short) ((ioa & 0xFF00) >> 8);
				lnk.data[i++] = (short) ((ioa & 0xFF0000) >> 8);
				break;
			}
			// for (int j = 0; j < szIoa; j++) {
			// lnk.data[i++] = 1;
			// }
			for (int j = 0; j < io.ie.size(); j++) {
				InformationElement ie = (InformationElement) io.ie.elementAt(j);
				for (int k = 0; k < ie.octetset.length; k++)
					lnk.data[i++] = ie.octetset[k];
			}
		}
		return lnk;
	}

	/**
	 * Constructs a writer factory specialized for 101 master station.
	 * 
	 * @param szCot
	 *            Cause of transmission length (1-2).
	 * @param szCaa
	 *            Common address of asdus length (1-2).
	 * @param szIoa
	 *            Information object address length (1-2-3).
	 * @param szLnk
	 *            Link address field length (1-2).
	 */
	public MAsduWriter101(byte szCot, byte szCaa, byte szIoa, byte szLnk) {
		this.szCot = szCot;
		this.szCaa = szCaa;
		this.szIoa = szIoa;
	}
} // end MAsduWriter101.java