/*
 * Class    br.org.scadabr.protocol.iec101.master101.MCommandAppSvc101
 * File     MCommandAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.IV.03
 */
public class MCommandAppSvc101 extends MAppService {
	/**
	 * Information object address.
	 */
	public int ioa;
	public byte state;
	public byte qualifier;
	public boolean se;
	public int setPointValue;

	/**
	 * Instanciates a new triggered single command asdu.
	 * 
	 * @param manager
	 *            Application layer manager.
	 * @param notifier
	 *            Luciol database notifier.
	 * @param id
	 *            Type identification. Single, double, regulating step, ...
	 *            commands.
	 * @param caa
	 *            Common address of asdus.
	 * @param ioa
	 *            Information object address to read.
	 * @param act
	 *            Activation/Deactivation flag.
	 * @param state
	 *            ON/OFF or LOW/HIGH according to id.
	 * @param qualifier
	 *            short pulse, long pulse, persistent or no additional...
	 * @param se
	 *            False for select, true for execute
	 */
	public MCommandAppSvc101(MAppManager manager, DbLocator notifier, byte id,
			int caa, int ioa, boolean act, byte state, byte qualifier,
			boolean se) {
		super(
		// asdu to build
				new MapAsdu(new AsduKey101(id, (short) 1,
						(act) ? Interoperability101.COT_ACTIVATION
								: Interoperability101.COT_DEACTIVATION,
						(short) NOTUSED, caa)),
				// related link service
				new MSendConfirmLnkSvc((MLnkManager) manager.lnk),
				// procedure identifier
				new MSvcId(MAppServiceIfc101.S9_COMMAND,
						MAppServiceIfc101.S9_PRIORITY),
				// database locator as luciol notifier
				notifier,
				// execution context
				manager.context,
				// expected ok asdu
				new AsduKey101(id, (short) NOTUSED,
						(act) ? Interoperability101.COT_ACTIVATION_TERMINATION
								: Interoperability101.COT_DEACTIVATIONCONF,
						(short) NOTUSED, NOTUSED));
		this.ioa = ioa;
		this.state = state;
		this.qualifier = qualifier;
		this.se = se;
	}
} // end MCommandAppSvc101.java
