/*
 * Class    br.org.scadabr.protocol.master101.MConfiguration101;
 * File     MConfiguration101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import java.util.Enumeration;
import java.util.Vector;

import br.org.scadabr.luciol.Configurable;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.LinkLayer;
import br.org.scadabr.protocol.iec101.common.PhySerialLayer;
import br.org.scadabr.protocol.iec101.common.PhysicalLayer;
import br.org.scadabr.protocol.iec101.common.PrivateRange;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MApplicationLayer;
import br.org.scadabr.protocol.iec101.master.MLinkLayer;
import br.org.scadabr.protocol.lib.xml.KXMLElement;

/**
 * <p>
 * This class defines common methods to instanciate mapping & private objects
 * from an xml configuration. And these to instanciate KXMLElement from
 * instanciated objects.
 * <p>
 * These mechanisms are OBLIGATORY for a Luciol application in order to be
 * compatible with HMI and central application manager.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class MConfiguration101 implements Configurable {

	public class Communication {
		public String port = "COM4";
		public int baud = 9600;
		public final static int parity = 0;
		public final static int data = 8;
		public final static int stop = 2;

		public Communication() {
			port = "COM4";
			baud = 9600;
		}

		public Communication(String port, int baud) {
			this.port = port;
			this.baud = baud;
		}
	}

	/**
	 * This class defines protocol settings.
	 * <p>
	 * Timeout is a watchdog on remote answer.
	 * <p>
	 * Retries stand for allowed number of retries in transmission fail case.
	 * <p>
	 * Polling defines polling data class 2 cyclic period in seconds.
	 * <p>
	 * NbToPoll defines maximum number of data class 2 to poll during 1 cycle.
	 * <p>
	 * Synchro defines synchronization cyclic period in minutes.
	 * <p>
	 * Caa stands for Common Address of Asdus.
	 * <p>
	 * oa stands for Originator address took in account only if cause of
	 * transmission field size equals 2.
	 * <p>
	 * gi is cycle duration for general interrogation.
	 * <p>
	 * giX is cycle duration for group X interrogation.
	 * <p>
	 * ciTime is cycle duration for counter global interrogation.
	 * <p>
	 * ciXTime is cycle duration for counter group X interrogation.
	 * <p>
	 * ciRaf is reset after freeze status for all counters.
	 * <p>
	 * ciXRaf is reset after freeze status for counters from group X.
	 */
	public class Protocol {
		public int timeout = 2;
		public int retries = 100;
		public int polling = 5;
		public int nbToPoll = 10;
		public int synchro = 60;
		public int caa = 1;
		public int oa = 254;
		public int gi = 0;
		public int gi1 = 0;
		public int gi2 = 0;
		public int gi3 = 0;
		public int gi4 = 0;
		public int gi5 = 0;
		public int gi6 = 0;
		public int gi7 = 0;
		public int gi8 = 0;
		public int gi9 = 0;
		public int gi10 = 0;
		public int gi11 = 0;
		public int gi12 = 0;
		public int gi13 = 0;
		public int gi14 = 0;
		public int gi15 = 0;
		public int gi16 = 0;
		public int ciTime = 0;
		public int ci1Time = 0;
		public int ci2Time = 0;
		public int ci3Time = 0;
		public int ci4Time = 0;
		public boolean ciRaf = false;
		public boolean ci1Raf = false;
		public boolean ci2Raf = false;
		public boolean ci3Raf = false;
		public boolean ci4Raf = false;
		public byte caaSz = 2;
		public byte ioaSz = 2;
		public byte cotSz = 1;
		public byte lnkSz = 1;
	}

	public Communication comm;
	public Protocol protocol;
	public MApplicationLayer app;
	private KXMLElement[] backup;

	private String author = "l.cauvy";
	private String version = "1.0";
	private String date = "10.IV.03";
	private String fileId = "unknown";

	public String getAuthor() {
		return author;
	}

	public String getVersion() {
		return version;
	}

	public String getFileId() {
		return fileId;
	}

	public String getDate() {
		return date;
	}

	private static final String APPNAME = "Master101";

	public String applicationName() {
		return APPNAME;
	}

	protected static final String ROOT = "Master101";
	protected static final String AUTHOR = "author";
	protected static final String VERSION = "version";
	protected static final String DATE = "date";

	protected static final String COMMUNICATION = "Communication";
	protected static final String PORT = "port";
	protected static final String BAUDRATE = "baudrate";

	protected static final String PROTOCOL = "Protocol";
	protected static final String TIMEOUT = "timeOut";
	protected static final String RETRIES = "retries";
	protected static final String POLLING = "polling";
	protected static final String SYNCHRO = "synchro";
	protected static final String OA = "oa";
	protected static final String IOA = "ioa";
	protected static final String GENERALINTERRO = "GeneralInterrogation";
	protected static final String GI = "global";
	protected static final String GI1 = "group1";
	protected static final String GI2 = "group2";
	protected static final String GI3 = "group3";
	protected static final String GI4 = "group4";
	protected static final String GI5 = "group5";
	protected static final String GI6 = "group6";
	protected static final String GI7 = "group7";
	protected static final String GI8 = "group8";
	protected static final String GI9 = "group9";
	protected static final String GI10 = "group10";
	protected static final String GI11 = "group11";
	protected static final String GI12 = "group12";
	protected static final String GI13 = "group13";
	protected static final String GI14 = "group14";
	protected static final String GI15 = "group15";
	protected static final String GI16 = "group16";
	protected static final String COUNTERINTERRO = "CounterInterrogation";
	protected static final String CITIME = "globalTime";
	protected static final String CI1TIME = "group1Time";
	protected static final String CI2TIME = "group2Time";
	protected static final String CI3TIME = "group3Time";
	protected static final String CI4TIME = "group4Time";
	protected static final String CIRAF = "globalResetAfterFreeze";
	protected static final String CI1RAF = "group1ResetAfterFreeze";
	protected static final String CI2RAF = "group2ResetAfterFreeze";
	protected static final String CI3RAF = "group3ResetAfterFreeze";
	protected static final String CI4RAF = "group4ResetAfterFreeze";
	protected static final String FIELDSIZE = "FieldSize";
	protected static final String CAASZ = "caaSz";
	protected static final String IOASZ = "ioaSz";
	protected static final String LNKSZ = "lnkSz";
	protected static final String COTSZ = "cotSz";
	protected static final String NBTOPOLL = "nbToPoll";
	protected static final String TRUE = "1";
	protected static final String FALSE = "0";

	protected static final String REMOTE = "Remote";
	protected static final String NAME = "name";
	protected static final String ADDRESS = "address";

	protected static final String PRIVATERANGE = "PrivateRange";
	protected static final String PRVTASDU = "PrvtAsdu";
	protected static final String PRVTELEMENT = "PrvtElement";
	protected static final String PRVTCOT = "PrvtCot";
	protected static final String PRVTIN = "PrvtIn";
	protected static final String PRVTFT = "PrvtFt";

	protected static final String MAPPING = "Mapping";

	protected final static String PROCEDURE = "AppService";
	protected final static String RESETCU = "RESETCU";
	protected final static String RESETFCB = "RESETFCB";
	protected final static String SYNCHROPROC = "SYNCHRO";
	protected final static String GENERALINTERROGATION = "GI";
	protected final static String POLLINGPROC = "POLLING";
	protected final static String GENERALCOMMAND = "GENERALCOMMAND";
	protected final static String DCO = "dco";

	protected final static String PROCEDURES = "Procedures";
	protected static final String WRITERS = "Writers";
	protected static final String ASDU = "Asdu";
	protected static final String KEY = "Key";
	protected static final String ID = "id";
	protected static final String VSQ = "vsq";
	protected static final String COT = "cot";
	protected static final String CAA = "caa";

	protected final static String MAPVALUE = "MapValue";
	protected final static String PIO = "pio";
	protected final static String PIE = "pie";
	protected final static String PVAL = "pval";
	protected final static String DBREF = "db";
	protected final static String LABEL = "label";

	protected static final String LINEARTRANSLATOR = "LinearTranslator";
	protected static final String BINARYTRANSLATOR = "BinaryTranslator";
	protected static final String TOBINARYTRANSLATOR = "Formatter101ToBinary";
	protected static final String FROMBINARYTRANSLATOR = "FormatterBinaryTo101";
	protected static final String RATE = "rate";
	protected static final String OFFSET = "offset";
	protected static final String TYPE = "type";
	protected static final String THRESHOLD = "threshold";
	protected static final String LOGIC = "logic";
	protected static final String UPPERBOUND = "UPPERBOUND";
	protected static final String LOWERBOUND = "LOWERBOUND";
	protected static final String EQUALITY = "EQUALITY";
	protected static final String POSITIVE = "POSITIVE";
	protected static final String NEGATIVE = "NEGATIVE";

	public MAppManager configureRemote(int address) {
		MContext101 context = new MContext101(address, "REMOTE");
		MAppManager manager = new MAppManager(app, context, protocol.nbToPoll);
		manager.periodic = new Vector();
		int duration = 1000 * protocol.polling;

		manager.initial = new MAppService[3];
		manager.initial[0] = new MReqStatusAppSvc101(manager, null);
		manager.initial[1] = new MResetCuAppSvc101(manager, null);
		manager.initial[2] = new MPoll2AppSvc101(manager, null);
		// manager.initial[1] = new MTestAppSvc101(manager, null, protocol.caa);
		// 
		// manager.initial[2] = new MGeneralInterrogationAppSvc101(manager,
		// null,
		// protocol.caa, (short) 0);

		manager.layer = app;
		manager.lnk.layer = (LinkLayer) app.lower;
		app.manager.put(address, manager);
		((MLinkLayer) app.lower).manager.put(address, manager.lnk);
		manager.prange = new PrivateRange();
		// manager.periodic.addElement(new PeriodicService(duration,
		// new MPoll2AppSvc101(manager, null), manager));
		return manager;
	}

	/**
	 * Instanciate an application manager from kXML configuration.
	 * 
	 * @param node
	 *            a KXMLElement Remote.
	 * 
	 * @return an instance of AppManager.
	 */
	public MAppManager inputRemote() {
		MContext101 context = new MContext101(1, "REMOTE");
		MAppManager manager = new MAppManager(app, context, protocol.nbToPoll);

		manager.periodic = new Vector();

		// initial procedures
		MAppService proc;
		manager.initial = new MAppService[3];
		manager.initial[0] = new MResetCuAppSvc101(manager, null);
		manager.initial[1] = new MReqStatusAppSvc101(manager, null);
		// manager.initial[1] = new MTestAppSvc101(manager,null,protocol.caa);
		// manager.initial[1] = new
		// MDelayAppSvc101(manager,null,Interoperability101.COT_ACTIVATION);
		// manager.initial[2] = new
		// MDelayAppSvc101(manager,null,Interoperability101.COT_SPONTANEOUS);
		// manager.initial[2] = new
		// MSynchroAppSvc101(manager,null,protocol.caa);
		manager.initial[2] = new MGeneralInterrogationAppSvc101(manager, null,
				protocol.caa, (short) 0);

		return manager;
	}

	protected Protocol inputProtocol(KXMLElement node) {
		Protocol proto = new Protocol();
		proto.timeout = node.getProperty(TIMEOUT, 0);
		proto.retries = node.getProperty(RETRIES, 0);
		proto.polling = node.getProperty(POLLING, 0);
		proto.nbToPoll = node.getProperty(NBTOPOLL, 0);
		proto.synchro = node.getProperty(SYNCHRO, 0);
		proto.caa = node.getProperty(CAA, 0);
		// proto.oa = node.getProperty(OA,0);
		for (Enumeration e = node.enumerateChildren(); e.hasMoreElements();) {
			KXMLElement child = (KXMLElement) e.nextElement();
			String name = child.getTagName();
			if (name.equals(GENERALINTERRO)) {
				proto.gi = child.getProperty(GI, 0);
				proto.gi1 = child.getProperty(GI1, 0);
				proto.gi2 = child.getProperty(GI2, 0);
				proto.gi3 = child.getProperty(GI3, 0);
				proto.gi4 = child.getProperty(GI4, 0);
				proto.gi5 = child.getProperty(GI5, 0);
				proto.gi6 = child.getProperty(GI6, 0);
				proto.gi7 = child.getProperty(GI7, 0);
				proto.gi8 = child.getProperty(GI8, 0);
				proto.gi9 = child.getProperty(GI9, 0);
				proto.gi10 = child.getProperty(GI10, 0);
				proto.gi11 = child.getProperty(GI11, 0);
				proto.gi12 = child.getProperty(GI12, 0);
				proto.gi13 = child.getProperty(GI13, 0);
				proto.gi14 = child.getProperty(GI14, 0);
				proto.gi15 = child.getProperty(GI15, 0);
				proto.gi16 = child.getProperty(GI16, 0);
			} else if (name.equals(COUNTERINTERRO)) {
				proto.ciTime = child.getProperty(CITIME, 0);
				proto.ciRaf = child.getProperty(CIRAF, "1", "0", false);
				proto.ci1Time = child.getProperty(CI1TIME, 0);
				proto.ci1Raf = child.getProperty(CI1RAF, "1", "0", false);
				proto.ci2Time = child.getProperty(CI2TIME, 0);
				proto.ci2Raf = child.getProperty(CI2RAF, "1", "0", false);
				proto.ci3Time = child.getProperty(CI3TIME, 0);
				proto.ci3Raf = child.getProperty(CI3RAF, "1", "0", false);
				proto.ci4Time = child.getProperty(CI4TIME, 0);
				proto.ci4Raf = child.getProperty(CI4RAF, "1", "0", false);
			} else if (name.equals(FIELDSIZE)) {
				proto.caaSz = (byte) child.getProperty(CAASZ, 1);
				proto.ioaSz = (byte) child.getProperty(IOASZ, 1);
				proto.cotSz = (byte) child.getProperty(COTSZ, 1);
				proto.lnkSz = (byte) child.getProperty(LNKSZ, 1);
			}
		}
		return proto;
	}

	protected KXMLElement outputProtocol(Protocol protocol) {
		KXMLElement node = new KXMLElement();
		KXMLElement ginode = new KXMLElement();
		KXMLElement cinode = new KXMLElement();
		KXMLElement fsnode = new KXMLElement();

		node.setTagName(PROTOCOL);
		ginode.setTagName(GENERALINTERRO);
		cinode.setTagName(COUNTERINTERRO);
		fsnode.setTagName(FIELDSIZE);

		node.addProperty(TIMEOUT, protocol.timeout);
		node.addProperty(RETRIES, protocol.retries);
		node.addProperty(POLLING, protocol.polling);
		node.addProperty(NBTOPOLL, protocol.nbToPoll);
		node.addProperty(SYNCHRO, protocol.synchro);
		node.addProperty(CAA, protocol.caa);
		// node.addProperty(OA,protocol.oa);

		ginode.addProperty(GI, protocol.gi);
		ginode.addProperty(GI1, protocol.gi1);
		ginode.addProperty(GI2, protocol.gi2);
		ginode.addProperty(GI3, protocol.gi3);
		ginode.addProperty(GI4, protocol.gi4);
		ginode.addProperty(GI5, protocol.gi5);
		ginode.addProperty(GI6, protocol.gi6);
		ginode.addProperty(GI7, protocol.gi7);
		ginode.addProperty(GI8, protocol.gi8);
		ginode.addProperty(GI9, protocol.gi9);
		ginode.addProperty(GI10, protocol.gi10);
		ginode.addProperty(GI11, protocol.gi11);
		ginode.addProperty(GI12, protocol.gi12);
		ginode.addProperty(GI13, protocol.gi13);
		ginode.addProperty(GI14, protocol.gi14);
		ginode.addProperty(GI15, protocol.gi15);
		ginode.addProperty(GI16, protocol.gi16);

		node.addChild(ginode);

		cinode.addProperty(CITIME, protocol.ciTime);
		cinode.addProperty(CIRAF, (protocol.ciRaf) ? TRUE : FALSE);
		cinode.addProperty(CI1TIME, protocol.ci1Time);
		cinode.addProperty(CI1RAF, (protocol.ci1Raf) ? TRUE : FALSE);
		cinode.addProperty(CI2TIME, protocol.ci2Time);
		cinode.addProperty(CI2RAF, (protocol.ci2Raf) ? TRUE : FALSE);
		cinode.addProperty(CI3TIME, protocol.ci3Time);
		cinode.addProperty(CI3RAF, (protocol.ci3Raf) ? TRUE : FALSE);
		cinode.addProperty(CI4TIME, protocol.ci4Time);
		cinode.addProperty(CI4RAF, (protocol.ci4Raf) ? TRUE : FALSE);

		node.addChild(cinode);

		fsnode.addProperty(CAASZ, protocol.caaSz);
		fsnode.addProperty(IOASZ, protocol.ioaSz);
		fsnode.addProperty(COTSZ, protocol.cotSz);
		fsnode.addProperty(LNKSZ, protocol.lnkSz);

		node.addChild(fsnode);

		return node;
	}

	protected Communication inputCommunication(KXMLElement node) {
		return new Communication(node.getProperty(PORT).toLowerCase(), node
				.getProperty(BAUDRATE, 9600));

	}

	protected KXMLElement outputCommunication(Communication comm) {
		KXMLElement node = new KXMLElement();
		node.setTagName(COMMUNICATION);
		node.addProperty(PORT, comm.port.toUpperCase());
		node.addProperty(BAUDRATE, comm.baud);
		return node;
	}

	public void inputFromXML(KXMLElement node) {
		if (!node.getTagName().equals(ROOT))
			return;

		author = node.getProperty(AUTHOR);
		version = node.getProperty(VERSION);
		date = node.getProperty(DATE);
		fileId = node.getProperty("fileId", "unknown");

		int size = node.countChildren() - 2;
		int i = 0;
		MLinkLayer lnk = null;
		PhysicalLayer phy = null;
		backup = new KXMLElement[size];
		for (Enumeration e = node.enumerateChildren(); e.hasMoreElements();) {
			KXMLElement child = (KXMLElement) e.nextElement();
			String name = child.getTagName();
			if (name.equals(COMMUNICATION)) {
				comm = inputCommunication(child);
			} else if (name.equals(PROTOCOL)) {

				protocol = inputProtocol(child);

				app = new MApplicationLayer(size);
				lnk = new MLinkLayer(size);
				phy = new PhySerialLayer(protocol.retries, protocol.timeout,
						comm.port, comm.baud, Communication.data, 2, 0);
				app.reader = new MAsduReader101(protocol.cotSz, protocol.caaSz,
						protocol.ioaSz, protocol.lnkSz);
				app.writer = new MAsduWriter101(protocol.cotSz, protocol.caaSz,
						protocol.ioaSz, protocol.lnkSz);
				app.lower = lnk;
				lnk.upper = app;
				lnk.lower = phy;
				phy.upper = lnk;
			} else if (name.equals(REMOTE)) {
				backup[i++] = child;
				MAppManager manager = inputRemote();
				Integer address = new Integer(manager.context.address);
				manager.layer = app;
				manager.lnk.layer = lnk;
				app.manager.put(address, manager);
				lnk.manager.put(address, manager.lnk);
			}
		}
	}

	public KXMLElement outputToXML() {
		KXMLElement node = new KXMLElement();
		node.setTagName(ROOT);

		node.addProperty(AUTHOR, author);
		node.addProperty(VERSION, version);
		node.addProperty(DATE, date);
		node.addProperty("fileId", fileId);

		node.addChild(outputCommunication(comm));
		node.addChild(outputProtocol(protocol));
		// for(Enumeration e = app.enumerateManagers();e.hasMoreElements();)
		// node.addChild(outputRemote((MAppManager101)e.nextElement()));
		for (int i = 0; i < backup.length;)
			node.addChild(backup[i++]);
		return node;
	}

	/**
	 * Kills all threads in current session.
	 */
	public void kill() {
		if (app != null) {
			app.kill();
			try {
				app.join();
			} catch (Exception e) {
			}
			CodeProperties.log("M101 really killed.");
		}
	}

	/**
	 * starts application.
	 */
	public void start() {
		if (!comm.port.equals("unknown")) {
			app.start();
		}
	}

	/**
	 * Instanciate a new configurator for master 101 application.
	 */
	public MConfiguration101() {
		comm = new Communication();
		protocol = new Protocol();
		app = new MApplicationLayer(0);
		backup = new KXMLElement[0];
	}

	@Override
	public int getParameterCount() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public String[] getParameterNames() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public byte[] getParameterTypes() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Object[] getParameterValues() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String[] getPossibleValues(int param)
			throws IllegalArgumentException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean[] setNewValues(Object[] newValues)
			throws IllegalArgumentException {
		// TODO Auto-generated method stub
		return null;
	}

} // end MConfiguration101.java