/*
 * Class    br.org.scadabr.protocol.iec101.master101.MDelayAppSvc101
 * File     MDelayAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'synchronization' application procedure for
 * master 101 application.
 * <p>
 * Here we set procedure identification, an asdu key to build a synchro
 * asdu, another asdu key as an expected answer from slave remote, the
 * related link service to implement,
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.IV.03
 */
public class MDelayAppSvc101 extends MAppService
{	
	/**
	 * Instanciates a new triggered delay acquisiton asdu. Cause of transmission can be
	 * activation or spontaneous.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Luciol database notifier.
	 * @param cot		Parametrable cause of transmission.
	 */		
	public MDelayAppSvc101(MAppManager manager, DbLocator notifier, short cot)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_C_CD_NA_1,
					DEFAULT_VSQ,
					cot,
					(short)NOTUSED,
					DEFAULT_CAA
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S2_DELAY_ACQ,
				MAppServiceIfc101.S2_PRIORITY					
			),
			// database locator as luciol notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			null			
		);
		
		switch(cot)
		{
		case Interoperability101.COT_ACTIVATION:
			this.expectedAsdu = new AsduKey101(
				Interoperability101.ID_C_CD_NA_1,
				(short)NOTUSED,
				Interoperability101.COT_ACTIVATIONCONF,
				(short)NOTUSED,
				DEFAULT_CAA
			);
			break;
		case Interoperability101.COT_SPONTANEOUS:
		default:
			this.expectedAsdu = null;
			break;					
		}
	}	
}	// end MDelayAppSvc101.java
