/*
 * Class    br.org.scadabr.protocol.iec101.master101.MGeneralCounterInterrogationAppSvc101
 * File     MGeneralCounterInterrogationAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;
/**
 * <p>
 * This class defines a 'general interrogation' or 'station interrogation'
 * mechanism. In 101 implementation, such a command can be addressed to whole
 * station via 'global interrogation' mechanism or reduced to part of station
 * via 'group 1 to 16 interrogation'...
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class MGeneralCounterInterrogationAppSvc101 extends MAppService
{	
	/**
	 * Qualifier of counter command QCC.
	 */
	public byte rqt;
	public byte frz;
	/**
	 * <p>
	 * Allocates a new procedure for remote station general interrogation triggered
	 * by database notifier.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database notifier.
	 * @param rqt		Qualifier of counter command REQUEST. See QCC.
	 * @param frz		Qualifier of counter command FREEZE. See Qcc.
	 */
	public MGeneralCounterInterrogationAppSvc101(MAppManager manager, DbLocator notifier, byte rqt, byte frz)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_C_CI_NA_1,
					DEFAULT_VSQ,
					Interoperability101.COT_ACTIVATION,
					(short)NOTUSED,
					DEFAULT_CAA
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S5_GENERAL_COUNTER_INTERROGATION,
				MAppServiceIfc101.S5_PRIORITY					
			),
			// Database notifier
			notifier,
			// Execution context
			manager.context,
			// esxpected asdu
			new AsduKey101(
				Interoperability101.ID_C_CI_NA_1,
				(short)NOTUSED,
				(frz==0)?Interoperability101.COT_ACTIVATION_TERMINATION:Interoperability101.COT_ACTIVATIONCONF,
				(short)NOTUSED,
				NOTUSED
			)			
		);
		this.rqt	= rqt;
		this.frz	= frz;	
	}	
}	// end MGeneralCounterInterrogationAppSvc101.java
