/*
 * Class    br.org.scadabr.protocol.iec101.master101.MGeneralInterrogationAppSvc101
 * File     MGeneralInterrogationAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;
/**
 * <p>
 * This class defines a 'general interrogation' or 'station interrogation'
 * mechanism. In 101 implementation, such a command can be addressed to whole
 * station via 'global interrogation' mechanism or reduced to part of station
 * via 'group 1 to 16 interrogation'...
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class MGeneralInterrogationAppSvc101 extends MAppService
{	
	/**
	 * Qualifier of interrogation QOI.
	 */
	public short qoi;
	public int ioa = 0;	
	/**
	 * <p>
	 * Allocates a new procedure for remote station general interrogation triggered
	 * by database notifier.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database notifier.
	 * @param qoi		Qualifier of interrogation. Global:0, Group1:1, ..., Group16:16
	 */
	public MGeneralInterrogationAppSvc101(MAppManager manager, DbLocator notifier, int caa, short qoi)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_C_IC_NA_1,
					(short)1,
					Interoperability101.COT_ACTIVATION,
					(short)NOTUSED,
					caa
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S4_GENERAL_INTERROGATION,
				MAppServiceIfc101.S4_PRIORITY					
			),
			// Database notifier
			notifier,
			// Execution context
			manager.context,
			// esxpected asdu
			new AsduKey101(
				Interoperability101.ID_C_IC_NA_1,
				(short)NOTUSED,
				Interoperability101.COT_ACTIVATION_TERMINATION,
				(short)NOTUSED,
				NOTUSED
			)			
		);
		this.qoi	= (short)(qoi + 20); 	
	}	
}	// end MGeneralInterrogationAppSvc101.java
