/*
 * Class    br.org.scadabr.protocol.iec101.master101.MParamActAppSvc101
 * File     MParamActAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines single, double, regulating step, set point and bitstring
 * commands. Asdus with type identification from 0x2D to 0x33.
 * <p>
 * So user must pass as parameter kind of command, and activation or deactivation
 * status.
 * <p>
 * Notice that activation will be validated on reception of same asdu with
 * activation termination cause of transmission. Whereas deactivation will be
 * validated on reception of same asdu with deactivation confirmation  cause of
 * transmission.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.IV.03
 */
public class MParamActAppSvc101 extends MAppService
{
	/**
	 * Information object address. The addressed object to act/deact.
	 */
	public int ioa;
	/**
	 * <p>
	 * Qualifier of parameter activation.
	 * <p>
	 * <3> := act/deact of persistent cyclic or periodic transmission
	 * of the addressed object
	 */
	public short qpa = 3;
	/**
	 * Instanciates a new triggered single command asdu.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Luciol database notifier.
	 * @param caa		Common address of asdus.
	 * @param ioa		Information object address to read.
	 * @param act		Activation/Deactivation flag.
	 */		
	public MParamActAppSvc101(MAppManager manager, DbLocator notifier, int caa, int ioa, boolean act)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_P_AC_NA_1,
					(short)1,
					(act)?Interoperability101.COT_ACTIVATION:Interoperability101.COT_DEACTIVATION,
					(short)NOTUSED,
					caa
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S11_PARAM_ACT,
				MAppServiceIfc101.S11_PRIORITY					
			),
			// database locator as luciol notifier
			notifier,
			// execution context
			manager.context,
			// expected ok asdu
			new AsduKey101(
				Interoperability101.ID_P_AC_NA_1,
				(short)NOTUSED,
				(act)?Interoperability101.COT_ACTIVATIONCONF:Interoperability101.COT_DEACTIVATIONCONF,
				(short)NOTUSED,
				NOTUSED
			)
		);
		this.ioa	= ioa;	
	}	
}	// end MParamActAppSvc101.java
