/*
 * Class    br.org.scadabr.protocol.iec101.master101.MParameterAppSvc101
 * File     MParameterAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines single, double, regulating step, set point and bitstring
 * commands. Asdus with type identification from 0x2D to 0x33.
 * <p>
 * So user must pass as parameter kind of command, and activation or deactivation
 * status.
 * <p>
 * Notice that activation will be validated on reception of same asdu with
 * activation termination cause of transmission. Whereas deactivation will be
 * validated on reception of same asdu with deactivation confirmation  cause of
 * transmission.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.IV.03
 */
public class MParameterAppSvc101 extends MAppService
{
	/**
	 * Information object address.
	 */
	public int ioa;
	/**
	 * Kind of parameter. See QPM.
	 */
	public byte kpa;
	/**
	 * Local parameter change. See QPM.
	 */
	public boolean plc;
	/**
	 * Parameter in operation. See QPM.
	 */
	public boolean pop;
	/**
	 * Instanciates a new triggered single command asdu. Notifier is also
	 * container for a mapped value.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Luciol database notifier.
	 * @param id		Type identification. Single, double, regulating step, ... commands.
	 * @param caa		Common address of asdus.
	 * @param ioa		Information object address to read.
	 * @param act		Activation/Deactivation flag.
	 * @param kpa		QPM's kind of parameter. See QPM.
	 * @param lpc		QPM's local parameter change. See QPM.
	 * @param pop		QPM's parameter in operation. See QPM.
	 */		
	public MParameterAppSvc101(MAppManager manager, DbLocator notifier, byte id, int caa, int ioa, boolean act, byte kpa, boolean plc, boolean pop)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					id,
					(short)1,
					(act)?Interoperability101.COT_ACTIVATION:Interoperability101.COT_DEACTIVATION,
					(short)NOTUSED,
					caa
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S10_PARAMETER,
				MAppServiceIfc101.S10_PRIORITY					
			),
			// database locator as luciol notifier
			notifier,
			// execution context
			manager.context,
			// expected ok asdu
			new AsduKey101(
				id,
				(short)NOTUSED,
				(act)?Interoperability101.COT_ACTIVATION_TERMINATION:Interoperability101.COT_DEACTIVATIONCONF,
				(short)NOTUSED,
				NOTUSED
			)
		);
		this.ioa	= ioa;
		this.kpa	= kpa;
		this.plc	= plc;
		this.pop	= pop;
	}	
}	// end MParameterAppSvc101.java
