/*
 * Class    br.org.scadabr.protocol.iec101.master101.MReqStatusAppSvc101
 * File     MReqStatusAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MReqStatusLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a request for remote link status application
 * procedure. 
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppService, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */

public class MReqStatusAppSvc101 extends MAppService
{		
	/**
	 * <p>
	 * Allocates a new service for requesting remote link status
	 * for master 101 application.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database locator notifier.
	 */
	public MReqStatusAppSvc101(MAppManager manager, DbLocator notifier)
	{
		super(
			// no asdu to build
			null,
			// related link service
			new MReqStatusLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S13_REQ_STATUS,
				MAppServiceIfc101.S13_PRIORITY					
			),
			// database notifier
			notifier,
			// execution context
			manager.context,
			// end of init asdu expected.
			new AsduKey101(
				(short)Interoperability101.ID_M_EI_NA_1,
				(short)NOTUSED,
				(short)Interoperability101.COT_INITIALIZED,
				(short)NOTUSED,
				NOTUSED
			)
		);
	}	
}	// end MReqStatusAppSvc101.java
