/*
 * Class    br.org.scadabr.protocol.iec101.master101.MResetCuAppSvc101
 * File     MResetCuAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MResetCuLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'station initialization' mechanism for 101 master
 * application. Mode of initialization is Reset Communication Unit or Reset
 * Remote Link standard application procedure.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class is to
 * format the procedure to make it match with a standardized application
 * procedure.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class MResetCuAppSvc101 extends MAppService {
	/**
	 * <p>
	 * Allocates a new service for resetting remote communication unit in 101
	 * implementation.
	 * 
	 * @param manager
	 *            Application layer manager.
	 * @param notifier
	 *            Database locator notifier.
	 */
	public MResetCuAppSvc101(MAppManager manager, DbLocator notifier) {
		super(
		// no asdu to build
				null,
				// related link service
				new MResetCuLnkSvc((MLnkManager) manager.lnk),
				// procedure identifier
				new MSvcId(MAppServiceIfc101.S1_RESET_CU,
						MAppServiceIfc101.S1_PRIORITY),
				// database notifier
				notifier,
				// execution context
				manager.context,
				// expected asdu
				null);
	}
} // end MResetCuAppSvc101.java
