/*
 * Class    br.org.scadabr.protocol.iec101.master101.MSynchroAppSvc101
 * File     MSynchroAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'synchronization' application procedure for
 * master 101 application.
 * <p>
 * Here we set procedure identification, an asdu key to build a synchro
 * asdu, another asdu key as an expected answer from slave remote, the
 * related link service to implement,
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.IV.03
 */
public class MSynchroAppSvc101 extends MAppService
{
	public int ioa = 0;
	/**
	 * Instanciates a new time synchronization application procedure.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Luciol database notifier.
	 */		
	public MSynchroAppSvc101(MAppManager manager, DbLocator notifier, int caa)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_C_CS_NA_1,
					(short)1,
					Interoperability101.COT_ACTIVATION,
					(short)NOTUSED,
					caa
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MAppServiceIfc101.S3_TIME_SYNCHRONIZATION,
				MAppServiceIfc101.S3_PRIORITY					
			),
			// database locator as luciol notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			new AsduKey101(
				Interoperability101.ID_C_CS_NA_1,
				(short)NOTUSED,
				Interoperability101.COT_ACTIVATIONCONF,
				(short)NOTUSED,
				NOTUSED
			)			
		);
	}	
}	// end MSynchroAppSvc101.java
