/*
 * Class    br.org.scadabr.protocol.iec101.master101.Master101App
 * File     Master101App.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.luciol.AppSession;
import br.org.scadabr.luciol.Application;

/**
 * 
 * <p>
 * This class implements Application interface that is entry point to launch
 * applications from the luciol main. This constructs a Master 101 application.
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class Master101App implements Application {
	/**
	 * application name
	 */
	private String name;
	/**
	 * maximum nuber of sessions
	 */
	private static final int maxSessions = 3;
	/**
	 * count number of session
	 */
	private static int countSessions;
	/**
	 * Sessions array static reference
	 */
	private static Master101Session session[];
	/**
	 * This application static reference.
	 */
	private static Master101App application = new Master101App();

	/**
	 * Return a title for the configuration page
	 * 
	 * @return this application name
	 */
	public String applicationName() {
		return name;
	}

	public String getAuthor() {
		return "l.cauvy";
	}

	public String getVersion() {
		return "1.0";
	}

	public String getDate() {
		return "10.II.03";
	}

	/**
	 * Open a new session for this application
	 * 
	 * @return number of this session
	 */
	public int openNewSession() {
		if (countSessions < maxSessions) {
			session[countSessions++] = new Master101Session();
			return countSessions - 1;
		}
		return -1;
	}

	/**
	 * Close all sessions of this application.
	 */
	public void deleteAllSessions() {
		for (int i = 0; i < countSessions; i++) {
			session[i].kill();
		}
		countSessions = 0;

	}

	/**
	 * Return the current number of open sessions for this application
	 * 
	 * @return number of instanciated sessions
	 */
	public int numberOfCurrentSessions() {
		return countSessions;
	}

	/**
	 * Return the maximum number of sessions this application can run
	 * concurrently.
	 * 
	 * @return maximum number of session for this iec60870-5 application
	 */
	public int numberOfSessionsAllowed() {
		return maxSessions;
	}

	/**
	 * Return the session of this application identified by the parameter.
	 * 
	 * @param id
	 *            indice of session
	 * 
	 * @return AppSession an application session
	 */
	public AppSession getSession(int id) {
		return session[id];
	}

	/**
	 * get accessor on instance ( constructor is private )
	 * 
	 * @return this application
	 */
	public static Master101App getInstance() {
		return application;
	}

	/**
	 * constructs a master application process for iec 60870-5-101 protocol
	 */
	protected Master101App() {
		Master101App.countSessions = 0;
		this.name = "Master 101";
		session = new Master101Session[maxSessions];
	}
}