/*
 * Class    br.org.scadabr.protocol.iec101.master101.Master101Session
 * File     Master101Session.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master101;

import br.org.scadabr.luciol.AppSession;
import br.org.scadabr.luciol.Configurable;
import br.org.scadabr.protocol.lib.xml.KXMLElement;

/**
 * <p>
 * This class specializes session process for 101 master implementation
 * 
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class Master101Session implements AppSession {
	/**
	 * 'Inline' configuration.
	 */
	private MConfiguration101 ACTIVE;

	/**
	 * 'Offline' configuration.
	 */
	private MConfiguration101 PASSIVE;

	/**
	 * /** Commit the configuration.(passive configuration becomes active)
	 */
	public synchronized void commit() {
		// XML element from PASSIVE
		KXMLElement newActiveConf = PASSIVE.outputToXML();

		// stops this process session
		ACTIVE.kill();

		// update ACTIVE from XML element 'newActiveConf'
		ACTIVE.inputFromXML(newActiveConf);

		// starts process session
		ACTIVE.start();
	}

	/**
	 * Roll back the configuration.(active may reset passive configuration)
	 */
	public synchronized void rollBack() {
		// XML element from ACTIVE
		KXMLElement newPassiveConf = ACTIVE.outputToXML();

		// update PASSIVE from XML element 'newPassiveConf'
		PASSIVE.inputFromXML(newPassiveConf);
	}

	/**
	 * test only.
	 */
	public void setPassive(MConfiguration101 passive) {
		PASSIVE = passive;
	}

	public void kill() {
		ACTIVE.kill();
		PASSIVE.kill();
	}

	/**
	 * Constructs an 101 master session process.
	 */
	public Master101Session() {
		ACTIVE = new MConfiguration101();
		PASSIVE = new MConfiguration101();
	}

	@Override
	public Configurable activeConfiguration() {
		return ACTIVE;
	}

	@Override
	public Configurable passiveConfiguration() {
		return PASSIVE;
	}

};