/*
 * Class    br.org.scadabr.protocol.iec101.master103.MAsduWriter103
 * File     MAsduWriter103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduWriter;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.InformationElement;
import br.org.scadabr.protocol.iec101.common.InformationObject;
import br.org.scadabr.protocol.iec101.common.LnkService;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.common103.information.*;

/**
 * <p>
 * This class defines mechanisms to build a 103 asdu to write
 * to lower link layer.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class MAsduWriter103 implements AsduWriter
{
	/**
	 * Fills information objects in a 103 asdu.
	 *
	 * @param proc	Application procedure reference.
	 *
	 * @return the information object.
	 */
	private InformationObject getIo(AppService proc)
	{
		InformationObject io = null;
		
		switch (proc.asdu.key.id)
		{		
		case Interoperability103.ID_SYNCHRO:
			io = new InformationObject();
			io.setElement(new CP56Time2a());
			break;
			
			
		case Interoperability103.ID_INIT_GI:
			io = new InformationObject();
			io.setElement(new SCN(((MContext103)proc.context).scn++));
			break;
			
			
		case Interoperability103.ID_GENERAL_CD:
			MGeneralCommandAppSvc103 procedure = (MGeneralCommandAppSvc103)proc;
			io = new InformationObject();
			io.setElement(new DCO(procedure.dco));
			io.setElement(new RII(((MContext103)proc.context).rii++));
			break;		
			
		default:
			CodeProperties.log("FUNCTION NOT IMPLEMENTED.");
			break;			
		}
		
		return io;
	}	
	/**
	 * Main method of this asdu writer factory for master 103 station.
	 *
	 * @param proc	an application procedure to turn into link service.
	 *
	 * @return the built asdu hold by a link service object.
	 */
	public LnkService build(AppService proc)
	{
		int size = 0;
		LnkService lnk	= proc.svc;
		if (proc.asdu!=null)// user data
		{
			AsduKey103 key = (AsduKey103)proc.asdu.key;
			InformationObject io = getIo(proc);
			for(int i=0;i<io.ie.size();i++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(i);
				size += ie.octetset.length;
			}
			int i=0;
			lnk.data		= new short[size+Interoperability103.HEADER_ASDU_SIZE];
			lnk.data[i++]	= key.id;
			lnk.data[i++]	= key.vsq;
			lnk.data[i++]	= key.cot;
			lnk.data[i++]	= (short)key.caa;
			lnk.data[i++]	= key.ft;
			lnk.data[i++]	= key.in;
			for(int j=0;j<io.ie.size();j++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(j);
				for(int k=0;k<ie.octetset.length;k++)
					lnk.data[i++] = ie.octetset[k];
			}
		}		
		return lnk;	
	}	
}	// end MAsduWriter103.java