/*
 * Class    br.org.scadabr.protocol.iec101.master103.MGeneralCommandAppSvc103;
 * File     MGeneralCommandAppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a general command procedure for master 103
 * application.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class MGeneralCommandAppSvc103 extends MAppService
{
	/**
	 * <p>
	 * Double command.
	 * <p><i>
	 * <1> OFF<br>
	 * <2> ON
	 * </i>
	 */
	public byte dco;
		
	/**
	 * <p>
	 * Allocates a general command service.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database notifier.
	 * @param ft		Function type value.
	 * @param in		Information number value.
	 * @param dco		Double command value.(<1>OFF <2>ON)
	 */
	public MGeneralCommandAppSvc103(MAppManager manager, DbLocator notifier, int caa, short ft, short in, byte dco)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey103(
					Interoperability103.ID_GENERAL_CD,
					DEFAULT_VSQ,
					Interoperability103.COT_ACK_COM,
					caa,
					ft,
					in
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S6_GENERAL_COMMAND,
				MProcedure103.S6_PRIORITY					
			),
			// Database notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			new AsduKey103(
				Interoperability103.ID_TIME_MSG,
				(short)NOTUSED,
				Interoperability103.COT_ACK_COM,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)
		);
		// double command value	
		this.dco	= dco;	
	}	
}	// end MGeneralCommandAppSvc103.java