/*
 * Class    br.org.scadabr.protocol.iec101.master103.MGeneralInterrogationAppSvc103
 * File     MGeneralInterrogationAppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;
/**
 * <p>
 * This class defines a 'general interrogation' mechanism.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class MGeneralInterrogationAppSvc103 extends MAppService
{		
	/**
	 * <p>
	 * Allocates a new procedure for remote station general interrogation.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database notifier.
	 */
	public MGeneralInterrogationAppSvc103(MAppManager manager, DbLocator notifier,int caa)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey103(
					Interoperability103.ID_INIT_GI,
					DEFAULT_VSQ,
					Interoperability103.COT_GI,
					caa,
					Interoperability103.FT_GLB,
					Interoperability103.IN_GI
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S4_GENERAL_INTERROGATION,
				MProcedure103.S4_PRIORITY					
			),
			// Database notifier
			notifier,
			// Execution context
			manager.context,
			// esxpected asdu
			new AsduKey103(
				Interoperability103.ID_END_GI,
				(short)NOTUSED,
				Interoperability103.COT_END_GI,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)			
		);
	}
	
};	// end MGeneralInterrogationAppSvc103.java
