/*
 * Class    br.org.scadabr.protocol.iec101.master103.MMapping103
 * File     MMapping103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.AsduKey;
import br.org.scadabr.protocol.iec101.common103.Asdu103;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MMapping;

/**
 * <p>
 * This class specializes single station mapping for
 * 103 master & slave applications. It only implements
 * instanciateKey method to match with 103 asdu format.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class MMapping103 extends MMapping
{
	/**
	 * Generates a mapping key. Asdu from 103 protocol contains supplementary
	 * data such as Function Type & Information Number. These attributes are
	 * used to map 103 data to luciol database. So this only interest of this
	 * class is ability to create 103 mapping key.
	 *
	 * @param object	a 103 formatted asdu to turn into mapping key.
	 *
	 * @return	the requested mapping key.
	 */	
	protected AsduKey instanciateKey(Object object)
	{		
		if (object instanceof Asdu103)
		{
			Asdu103 asdu = (Asdu103)object;
			AsduKey103 key = new AsduKey103(
				asdu.id,
				asdu.vsq,
				AsduKey103.NOTUSED,
				asdu.caa,
				asdu.ft,
				asdu.in
			);
			return key;
		}		
		return null;
	}
	
	/**
	 * Allocates a mapping for 103 master application according to
	 * a single remote station identified in Network by its address.
	 */		
	public MMapping103(MAppManager manager)
	{
		super(manager);
	}

};	// end MMapping103.java