/*
 * Class    br.org.scadabr.protocol.iec101.master103.MPoll2AppSvc103
 * File     MPoll2AppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MPoll2LnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;
/**
 * <p>
 * This class defines a 'polling measures' mechanism.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class MPoll2AppSvc103 extends MAppService
{		
	/**
	 * <p>
	 * Constructs a new polling procedure triggered by database notification.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database locator notifier.
	 */		
	public MPoll2AppSvc103(MAppManager manager, DbLocator notifier)
	{
		super(
			// no asdu to build
			null,
			// related link service
			new MPoll2LnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S8_POLLING,
				MProcedure103.S8_PRIORITY					
			),
			// luciol database notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			new AsduKey103(
				(short)NOTUSED,
				(short)NOTUSED,
				Interoperability103.COT_CYCLIC,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)
		);
	}	
}	// end MPoll2AppSvc103.java
