/*
 * Class    br.org.scadabr.protocol.iec101.master103.MResetCuAppSvc103
 * File     MResetCuAppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MResetCuLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'station initialization' mechanism for 103 master
 * application. Mode of initialization is Reset Communication Unit or
 * Reset Remote Link standard application procedure.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */

public class MResetCuAppSvc103 extends MAppService
{
	/**
	 * <p>
	 * Allocates a new service for resetting remote communication
	 * unit in 103 implementation.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database locator notifier.
	 */
	public MResetCuAppSvc103(MAppManager manager, DbLocator notifier)
	{
		super(
			// no asdu to build
			null,
			// related link service
			new MResetCuLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S1_RESET_CU,
				MProcedure103.S1_PRIORITY					
			),
			// database notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			new AsduKey103(
				Interoperability103.ID_IDENTIFICATION,
				(short)NOTUSED,
				Interoperability103.COT_RESET_CU,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)
		);
	}	
};	// end MResetCuAppSvc103.java
