/*
 * Class    br.org.scadabr.protocol.iec101.master103.MResetFcbAppSvc103
 * File     MResetFcbAppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MResetFcbLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'station initialization' mechanism for 103 master
 * application. Mode of initialization is Reset Frame Count Bit standard
 * application procedure.
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends master procedure one, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */

public class MResetFcbAppSvc103 extends MAppService
{		
	/**
	 * <p>
	 * Allocates a new application procedure for resetting remote
	 * communication unit in master 103 implementation.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Database locator notifier.
	 */
	public MResetFcbAppSvc103(MAppManager manager, DbLocator notifier)
	{
		super(
			// no asdu to build
			null,
			// related link service
			new MResetFcbLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S2_RESET_FCB,
				MProcedure103.S2_PRIORITY					
			),
			// luciol database notifier
			notifier,
			// Execution context
			manager.context,
			// exepected asdu
			new AsduKey103(
				Interoperability103.ID_IDENTIFICATION,
				(short)NOTUSED,
				Interoperability103.COT_RESET_FCB,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)
		);
	}
};	// end MResetFcbAppSvc103.java
