/*
 * Class    br.org.scadabr.protocol.iec101.master103.MSynchroAppSvc103
 * File     MSynchroAppSvc103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MLnkManager;
import br.org.scadabr.protocol.iec101.master.MSendConfirmLnkSvc;
import br.org.scadabr.protocol.iec101.master.MSvcId;

/**
 * <p>
 * This class defines a 'synchronization' application procedure for
 * master 103 application.
 * <p>
 * Here we set procedure identification, an asdu key to build a synchro
 * asdu, another asdu key as an expected answer from slave remote, the
 * related link service to implement,
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class MSynchroAppSvc103 extends MAppService
{	
	/**
	 * Instanciates a new time synchronization application procedure.
	 *
	 * @param manager	Application layer manager.
	 * @param notifier	Luciol database notifier.
	 */		
	public MSynchroAppSvc103(MAppManager manager, DbLocator notifier, int caa)
	{
		super(
			// asdu to build
			new MapAsdu(
				new AsduKey103(
					Interoperability103.ID_SYNCHRO,
					DEFAULT_VSQ,
					Interoperability103.COT_SYNCHRO,
					caa,
					Interoperability103.FT_GLB,
					Interoperability103.IN_SYNCHRO
				)
			),
			// related link service
			new MSendConfirmLnkSvc((MLnkManager)manager.lnk),
			// procedure identifier
			new MSvcId(
				MProcedure103.S3_TIME_SYNCHRONIZATION,
				MProcedure103.S3_PRIORITY					
			),
			// database locator as luciol notifier
			notifier,
			// execution context
			manager.context,
			// expected asdu
			new AsduKey103(
				Interoperability103.ID_SYNCHRO,
				(short)NOTUSED,
				Interoperability103.COT_SYNCHRO,
				NOTUSED,
				(short)NOTUSED,
				(short)NOTUSED
			)			
		);
	}	
}	// end MSynchroAppSvc103.java
