/*
 * Class    br.org.scadabr.protocol.iec101.master103.Master103Session
 * File     Master103Session.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.master103;

import br.org.scadabr.luciol.AppSession;
import br.org.scadabr.luciol.Configurable;
import br.org.scadabr.protocol.lib.xml.KXMLElement;

/**
 * <p>
 * This class specializes session process for 103 master implementation
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class Master103Session implements AppSession
{
	/**
	 * 'Inline' configuration.
	 */
	private MConfiguration103 ACTIVE;

	/**
	 * 'Offline' configuration.
	 */
	private MConfiguration103 PASSIVE;

	/**
	 * Return the active configuration of this application session
	 *
	 * @return active configuration
	 */
	public Configurable activeConfiguration()
	{
		return ACTIVE;
	}

	/**
	 * Return the passive configuration of this application session
	 *
	 * @return passive configuration
	 */
	public Configurable passiveConfiguration()
	{
		return PASSIVE;
	}

	/**
	 * Commit the configuration.(passive configuration becomes active)
	 */
	public synchronized void commit()
	{
		// XML element from PASSIVE
		KXMLElement newActiveConf = PASSIVE.outputToXML();

		// stops this process session
		ACTIVE.kill();

		// update ACTIVE from XML element 'newActiveConf'
		ACTIVE.inputFromXML(newActiveConf);

		// starts process session
		ACTIVE.start();
	}


	/**
	 * Roll back the configuration.(active may reset passive configuration)
	 */
	public synchronized void rollBack()
	{
		// XML element from ACTIVE
		KXMLElement newPassiveConf = ACTIVE.outputToXML();

		// update PASSIVE from XML element 'newPassiveConf'
		PASSIVE.inputFromXML(newPassiveConf);
	}
	
	/**
	 * test only.
	 */
	public void setPassive(MConfiguration103 passive)
	{
		PASSIVE = passive;
	}
	
	public void kill()
	{
		ACTIVE.kill();
		PASSIVE.kill();
	}


	/**
	 * Constructs an 103 master session process.
	 */
	public Master103Session()
	{
		ACTIVE = new MConfiguration103();
		PASSIVE = new MConfiguration103();
	}
	
};	// end Master103Session.java

























