package br.org.scadabr.protocol.iec101.session;

import br.org.scadabr.protocol.iec101.common.LinkLayer;
import br.org.scadabr.protocol.iec101.common.PhyEthernetLayer;
import br.org.scadabr.protocol.iec101.common.PhySerialLayer;
import br.org.scadabr.protocol.iec101.common.PhysicalLayer;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master.MApplicationLayer;
import br.org.scadabr.protocol.iec101.master.MLinkLayer;
import br.org.scadabr.protocol.iec101.master101.MAsduReader101;
import br.org.scadabr.protocol.iec101.master101.MAsduWriter101;
import br.org.scadabr.protocol.iec101.master101.MContext101;
import br.org.scadabr.protocol.iec101.session.IECConfig.COMM;
import br.org.scadabr.protocol.iec101.session.commands.ClassPoll;
import br.org.scadabr.protocol.iec101.session.commands.DoubleCommand;
import br.org.scadabr.protocol.iec101.session.commands.GeneralInterrogation;
import br.org.scadabr.protocol.iec101.session.commands.InitializationCommand;
import br.org.scadabr.protocol.iec101.session.commands.SetPointCommand;
import br.org.scadabr.protocol.iec101.session.commands.SingleCommand;
import br.org.scadabr.protocol.iec101.session.database.Database;

public class IECUser {
	private IECConfig config;
	private MApplicationLayer app;
	private MAppManager manager;
	private Database database;

	public IECUser(IECConfig config) {
		this.config = config;
	}

	public int init() throws Exception {
		MLinkLayer lnk = null;
		PhysicalLayer phy = null;
		app = new MApplicationLayer(1);
		lnk = new MLinkLayer(1);

		if (config.getCommType() == COMM.SERIAL) {
			phy = new PhySerialLayer(config.getRetries(), config.getTimeout(),
					config.getSerialPort(), config.getBaudrate(), config
							.getDatabits(), config.getStopbits(), config
							.getParity());
		} else {
			phy = new PhyEthernetLayer(config.getHost(), config.getPort());
		}

		app.reader = new MAsduReader101(config.getCotSize(), config
				.getAsduAddressSize(), config.getObjectAddressSize(), config
				.getLinkAddressSize());
		app.writer = new MAsduWriter101(config.getCotSize(), config
				.getAsduAddressSize(), config.getObjectAddressSize(), config
				.getLinkAddressSize());
		app.lower = lnk;
		lnk.upper = app;
		lnk.lower = phy;
		phy.upper = lnk;

		MContext101 context = new MContext101(config.getLinkAddress(), "REMOTE");
		manager = new MAppManager(app, context, 0);

		manager.initial = new MAppService[0];
		manager.layer = app;
		manager.lnk.layer = (LinkLayer) app.lower;
		app.manager.put(config.getLinkAddress(), manager);
		((MLinkLayer) app.lower).manager.put(config.getLinkAddress(),
				manager.lnk);
		manager.setTimeout(config.getTimeout());
		setDatabase(manager.getDatabase());
		((MAsduReader101) app.reader).setManager(manager);
		app.start();
		return initializationCommand();
	}

	public synchronized int send(CompositeMessage message) throws Exception {
		int response = manager.send(message);
		if (response != CompositeMessage.SUCCESS)
			throw new Exception("[IEC101] Command Failed!");
		return response;
	}

	public synchronized int initializationCommand() throws Exception {
		CompositeMessage init = new InitializationCommand(manager);
		int response = manager.send(init);
		return response;
	}

	public synchronized int sendGeneralInterrogation() throws Exception {
		CompositeMessage gi = new GeneralInterrogation(manager);
		int response = manager.send(gi);
		return response;
	}

	public synchronized int class2Request() throws Exception {
		int response = manager.send(new ClassPoll(manager));
		return response;
	}

	public synchronized int singleCommand(int ioa, boolean select,
			byte qualifier, boolean value) throws Exception {

		CompositeMessage singleCommand = new SingleCommand(manager, ioa,
				select, qualifier, value);
		int response = manager.send(singleCommand);
		if (response != CompositeMessage.SUCCESS)
			throw new Exception("[IEC101] Class 2 Request Failed!");

		return 0;
	}

	public synchronized int doubleCommand(int ioa, boolean select,
			byte qualifier, boolean value) throws Exception {

		CompositeMessage DoubleCommand = new DoubleCommand(manager, ioa,
				select, qualifier, value);
		int response = manager.send(DoubleCommand);

		if (response != CompositeMessage.SUCCESS)
			throw new Exception("[IEC101] Class 2 Request Failed!");

		return response;
	}

	public synchronized int setPointCommand(int ioa, boolean select,
			byte qualifier, int value) throws Exception {

		CompositeMessage SetPointCommand = new SetPointCommand(manager, ioa,
				false, qualifier, value);
		int response = manager.send(SetPointCommand);

		if (response != CompositeMessage.SUCCESS)
			throw new Exception("[IEC101] Class 2 Request Failed!");

		return 0;
	}

	public void stop() throws Exception {
		if (app != null) {
			app.kill();
			try {
				app.join();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void setDatabase(Database database) {
		this.database = database;
	}

	public Database getDatabase() {
		return database;
	}
}
