package br.org.scadabr.protocol.iec101.session.commands;

import br.org.scadabr.protocol.iec101.common.DbLocator;
import br.org.scadabr.protocol.iec101.master.MAppManager;
import br.org.scadabr.protocol.iec101.master.MAppService;
import br.org.scadabr.protocol.iec101.master101.MCommandAppSvc101;
import br.org.scadabr.protocol.iec101.master101.MPoll2AppSvc101;
import br.org.scadabr.protocol.iec101.session.CompositeMessage;
import br.org.scadabr.protocol.iec101.session.Response;

public class SetPointCommand implements CompositeMessage {

	private MAppManager manager;
	private boolean select = false;

	public SetPointCommand(MAppManager manager, int ioa, boolean select,
			byte qualifier, int value) {
		this.manager = manager;

		if (!select) {
			MCommandAppSvc101 command = new MCommandAppSvc101(manager,
					(DbLocator) null, (byte) 48, 3, ioa, true, (byte) 0x00,
					qualifier, true);
			command.setPointValue = value;

			services[0] = command;
			services[1] = new MPoll2AppSvc101(manager, null);
		} else {
			this.select = true;
			services = new MAppService[3];

			MCommandAppSvc101 executeCommand = new MCommandAppSvc101(manager,
					(DbLocator) null, (byte) 48, 3, ioa, true, (byte) 0x00,
					qualifier, true);
			executeCommand.setPointValue = value;
			MCommandAppSvc101 selectCommand = new MCommandAppSvc101(manager,
					(DbLocator) null, (byte) 48, 3, ioa, true, (byte) 0x00,
					qualifier, false);
			selectCommand.setPointValue = value;

			services[0] = executeCommand;
			services[1] = new MPoll2AppSvc101(manager, null);
			services[2] = selectCommand;
			index = 2;
		}

	}

	private int index = 0;
	private MAppService services[] = new MAppService[2];

	@Override
	public int responseReceived(Response response) {

		if (select) {
			if (index == 2) {
				if (response.isAck()) {
					index = 1;
					return CONTINUE;
				}
			} else if (response.isActivationConfirmation()) {
				index = 0;
				select = false;
				return CONTINUE;
			} else {
				index = 1;
				return CONTINUE;
			}
		} else if (index == 0) {
			if (response.isAck()) {
				index = 1;
				return CONTINUE;
			}
		} else {
			if (response.isActivationConfirmation()) {
				index = 1;
				return CONTINUE;
			} else if (response.isActivationTermination()) {
				index = 1;
				return SUCCESS;
			} else
				return CONTINUE;
		}
		return FAILED;
	}

	@Override
	public MAppService nextMessage() {
		if (services.length - 1 < index)
			return null;
		else
			return services[index];
	}

}
