package br.org.scadabr.protocol.iec101.session.database;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import br.org.scadabr.protocol.iec101.common101.Interoperability101;

public class Database {
	
	public static final int SINGLE_POINT_INFORMATION = Interoperability101.ID_M_SP_NA_1;
	public static final int DOUBLE_POINT_INFORMATION = Interoperability101.ID_M_DP_NA_1;
	public static final int STEP_POSITION = Interoperability101.ID_M_ST_NA_1;
	public static final int NORMALIZED_MEASURE = Interoperability101.ID_M_ME_NA_1;
	
	private HashMap<Integer, DataBuffer> singlePoints;
	private HashMap<Integer, DataBuffer> doublePoints;
	private HashMap<Integer, DataBuffer> stepPositions;
	private HashMap<Integer, DataBuffer> normalizedMeasures;

	public Database() {
		singlePoints = new HashMap<Integer, DataBuffer>();
		doublePoints = new HashMap<Integer, DataBuffer>();
		stepPositions = new HashMap<Integer, DataBuffer>();
		normalizedMeasures = new HashMap<Integer, DataBuffer>();
	}

	public List<DataElement> readBinaryInputPoint(int address) {
		if (singlePoints.get(address) != null)
			return singlePoints.get(address).readAndPop();
		else
			return null;
	}


	public void writeRecord(DataElement element) {
		getDataBuffer(element.getAddress(), element.getTypeIdentification()).insert(element); 
	}

	public List<DataElement> read(int address, int typeIdentification) {
		switch (typeIdentification) {
		case Interoperability101.ID_M_SP_NA_1:
		case Interoperability101.ID_M_SP_TA_1:
		case Interoperability101.ID_M_SP_TB_1:
			if (singlePoints.get(address) == null)
				singlePoints.put(address, new DataBuffer());
			return singlePoints.get(address).readAndPop();
		case Interoperability101.ID_M_DP_NA_1:
		case Interoperability101.ID_M_DP_TA_1:
		case Interoperability101.ID_M_DP_TB_1:
			if (doublePoints.get(address) == null)
				doublePoints.put(address, new DataBuffer());
			return doublePoints.get(address).readAndPop();
		case Interoperability101.ID_M_ST_NA_1:
		case Interoperability101.ID_M_ST_TA_1:
		case Interoperability101.ID_M_ST_TB_1:
			if (stepPositions.get(address) == null)
				stepPositions.put(address, new DataBuffer());
			return stepPositions.get(address).readAndPop();
		case Interoperability101.ID_M_ME_NA_1:
			if (normalizedMeasures.get(address) == null)
				normalizedMeasures.put(address, new DataBuffer());
			return normalizedMeasures.get(address).readAndPop();
		default:
			return new ArrayList<DataElement>();
		}
	}

	private DataBuffer getDataBuffer(int address, int typeIdentification) {
		switch (typeIdentification) {
		case Interoperability101.ID_M_SP_NA_1:
		case Interoperability101.ID_M_SP_TA_1:
		case Interoperability101.ID_M_SP_TB_1:
			if (singlePoints.get(address) == null)
				singlePoints.put(address, new DataBuffer());
			return singlePoints.get(address);
		case Interoperability101.ID_M_DP_NA_1:
		case Interoperability101.ID_M_DP_TA_1:
		case Interoperability101.ID_M_DP_TB_1:
			if (doublePoints.get(address) == null)
				doublePoints.put(address, new DataBuffer());
			return doublePoints.get(address);
		case Interoperability101.ID_M_ST_NA_1:
		case Interoperability101.ID_M_ST_TA_1:
		case Interoperability101.ID_M_ST_TB_1:
			if (stepPositions.get(address) == null)
				stepPositions.put(address, new DataBuffer());
			return stepPositions.get(address);
		case Interoperability101.ID_M_ME_NA_1:
			if (normalizedMeasures.get(address) == null)
				normalizedMeasures.put(address, new DataBuffer());
			return normalizedMeasures.get(address);
		default:
			return new DataBuffer();
		}
	}

	public HashMap<Integer, DataBuffer> getSinglePoints() {
		return singlePoints;
	}

	public void setSinglePoints(HashMap<Integer, DataBuffer> singlePoints) {
		this.singlePoints = singlePoints;
	}

	public HashMap<Integer, DataBuffer> getDoublePoints() {
		return doublePoints;
	}

	public void setDoublePoints(HashMap<Integer, DataBuffer> doublePoints) {
		this.doublePoints = doublePoints;
	}

	public HashMap<Integer, DataBuffer> getStepPositions() {
		return stepPositions;
	}

	public void setStepPositions(HashMap<Integer, DataBuffer> stepPositions) {
		this.stepPositions = stepPositions;
	}

	public HashMap<Integer, DataBuffer> getNormalizedMeasures() {
		return normalizedMeasures;
	}

	public void setNormalizedMeasures(
			HashMap<Integer, DataBuffer> normalizedMeasures) {
		this.normalizedMeasures = normalizedMeasures;
	}
	
}
