/*
 * Class    br.org.scadabr.protocol.iec101.slave.SApplicationLayer
 * File     SApplicationLayer.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave;

import br.org.scadabr.protocol.iec101.common.ApplicationLayer;
import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.UserData;

/**
 * <p>
 * This class defines an implementation of IEC 608705 application layer from EPA model.<br>
 * Application layer is in charge of:<br>
 * - application service data units (iec 60870-5-3).<br>
 * - information elements (iec 60870-5-4).
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class SApplicationLayer extends ApplicationLayer
{
	/**
	 * Application manager reference.
	 */
	public SAppManager manager;
	/**
	 * Clock permission.
	 */
	public boolean clock = false;
	/**
	 * Common address of asdu.
	 */
	public int caa = 1;
	/**
	 * Start application.
	 */
	public void start()
	{
		launch();
		manager.start();
	}
	/**
	 * Event from lower layer. Raised when user data is received at link
	 * layer level from a remote master station.
	 *
	 * @param object	User data reference.
	 */
	public void lowerEvent(Object object) throws IEC608705Exception
	{
		UserData data = (UserData)object;
		Asdu asdu = reader.build(object,manager.prange);
		manager.evaluate(asdu);
		asdu = null;
	}	
	/**
	 * Allocates an application layer for slave applications.
	 *
	 * @param polling	Measure cycle duration. Expressed in seconds.
	 */
	public SApplicationLayer()
	{
	}	
}	// end SApplicationLayer.java