/*
 * Class    br.org.scadabr.protocol.iec101.slave.SConfirmLnkSvc
 * File     SConfirmLnkSvc.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave;

import br.org.scadabr.protocol.iec101.common.Interoperability;

/**
 * <p>
 * This class defines both positive and negative confirmation services.
 * <p>
 * Such a link service is used to confirm reception of user data from
 * a remote master.
 * <p>
 * Positive confirmation will imply that local application can conform
 * to the master order. Negative confirmation means that local application
 * don't understand or can't apply master command.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class SConfirmLnkSvc extends SLnkService
{
	/**
	 * Instanciates a new postive or negative acknoweldgement confirmation service.
	 * Positive or negative state is selected by the boolean 'ack' argument that
	 * is passed here.
	 *
	 * @param manager	Link layer manager.
	 * @param ack		Positive or negative indicator. 
	 */	
	public SConfirmLnkSvc(SLnkManager manager, boolean ack)
	{
		super(
			(ack)?Interoperability.SERVICE_S2_CONFIRM_ACK:Interoperability.SERVICE_S2_CONFIRM_NACK,
			manager.address,
			manager.acd,
			manager.dfc
		);
	}	
}	// end SConfirmLnkSvc.java
