/*
 * Class    br.org.scadabr.protocol.iec101.slave.SLinkLayer
 * File     SLinkLayer.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave;

import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.Frame;
import br.org.scadabr.protocol.iec101.common.FrameReader;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.LinkLayer;
import br.org.scadabr.protocol.iec101.common.LnkService;


/**
 * <p>
 * This class defines used implementation of IEC 608705 link layer from EPA model.<br>
 * Link layer from iec60870 has to:<br>
 * - check transmission frame format (IEC 60870-5-1).<br>
 * - provide transmission procedures (IEC 60870-5-2).<br>
 * <p>
 * Notice that FT1.2 format compliance verification has been moved to lower physical layel
 * for convenience and reasons of performance.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class SLinkLayer extends LinkLayer
{
	/**
	 * Final static identifier to check prm direction
	 * on received frames. PRM is a bit hold by control
	 * field of a frame.
	 */
	private static final boolean MODE = true;	
	/**
	 * Link manager reference.
	 */
	public SLnkManager manager;
	
	public void lowerEvent(Object object) throws IEC608705Exception
	{
		// build the frame related to AppService object.
		Frame frame = (Frame)(FrameReader.build(object));
		// test on frame.		
		if (frame==null)
		{
			CodeProperties.lnklog("Receive a null frame from lower physical layer.");
			return;
		}
		// test on primary direction.
		if (frame.control.prm!=MODE)
		{
			CodeProperties.lnklog("Receive a reverse direction frame: PRM="+frame.control.prm);
			frame = null;
			return;
		}
		
				
		if (!frame.user.data.isEmpty())
		{
			send(frame.user);
		}
		
		// Get the link service resulting from manager evaluation
		LnkService svc = manager.evaluate(frame);
		// If the service exists, we have to write smthg to physical layer.
		if (svc!=null)
			toLower(svc);
		frame = null;		
	}
	public void toLower(LnkService svc) throws IEC608705Exception
	{
			svc.control.bool1 = manager.computeAcd();
			svc.control.bool2 = manager.computeDfc();	
			// write to lower layer.
			buildAndWrite(svc);
	}
	public void upperEvent(Object object)
	{
		try
		{
			manager.request((LnkService)object);
		}
		catch(IEC608705Exception e)
		{
		}
	}
	/**
	 * <p>
	 * Allocates a new link layer for slave application.
	 */
	public SLinkLayer()
	{
		mode = true;
	}

}	// end SLinkLayer.java



