/*
 * Class    br.org.scadabr.protocol.iec101.slave.SLnkManager
 * File     SLnkManager.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave;

import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.Frame;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.Interoperability;
import br.org.scadabr.protocol.iec101.common.LnkManager;
import br.org.scadabr.protocol.iec101.common.LnkService;

import com.ajile.util.FifoQueue;

/**
 * <p>
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date
 */
public class SLnkManager extends LnkManager
{
	public static final byte ACK	= 0x01;
	public static final byte NACK	= 0x02;
	/**
	 * Primary message direction.
	 */
	public static final boolean PRM		= false;
	/**
	 * Fifo for user data class 1. Events from application layer.
	 */
	protected FifoQueue class1;
	/**
	 * Fifo for user data class 2. Measures from application layer.
	 */
	protected FifoQueue class2;
	
	protected SLnkService endOfInit;
	
	/**
	 * Last answer to an order with fcv significant. We've got to
	 * backup it to avoid loose of an answer in case of transmission
	 * error.
	 */
	protected SLnkService lastMessage;
	/**
	 * Internal method to compute acd value. This bit stands for
	 * Access Demand, and must be raised when there's some user data
	 * class 1 that are 'Events' in this standard.
	 *
	 * @return updated acd.
	 */
	protected boolean computeAcd()
	{
		if (!class1.isEmpty())
			acd = true;
		else
			acd = false;
			
		return acd;	
	}
	/**
	 * Internal method to compute dfc value. This bit stands for
	 * Data Flow Control and must be raised when there's no more
	 * empty areas in fifo keeping user data class 1.
	 *
	 * @return updated dfc.
	 */
	protected boolean computeDfc()
	{
		if(class1.isFull())
			dfc = true;
		else
			dfc = false;
				
		return dfc;
	}

//	/**
//	 * Internal method to backup last message. In fact, just a
//	 * copy of current running service if fcv is set to 1.
//	 */	
//	protected void backupMessage()
//	{
//		if(fcv)
//			lastMessage = current.copy();
//	}


	public void appEvent(byte type) throws IEC608705Exception
	{
		switch (type)
		{
		case ACK:
			((SLinkLayer)layer).toLower(cack);
			break;
		case NACK:
			break;
		}
		// passe data par la meme voie que master.?
		// oui, par exemple sur reset, on vide les piles
		// et comme il n'y a pas d'asdu, on doit alerter remote
		// app. Par contre la notification ne vient pas de remote
		// app. ce dernier doit lancer la fabrication des asdus
		// dans le thread app layer qui descendent par la voie 
		// normale. puis appel du request au niveau link sur remote
		// qui place le LinkService dans la pile et notifie!
		// Remote app se demmerde pour construire les asdus voulues
		// soit on bloque jusqu' la dernire. 

		// Ok. link fait une interro boolean statusOfService(service) sur app
		// Si oui, acd = 1, 
		
		// Ok 2 types de service liens
		// 1) pas de user data en reception et ack
		//		byte requestStatusOnEmptyService(service)
		/*		retour 0x00 ok pas de donnes
					   0x01 ok donnes
					   0x02 ko pas de donnes
					   0x03 ko donnes
					   0x04 wait!
				on peut rpondre avec ACD correct
				
				rcu,rfcb,rls
			
			2) respond
				
				r1,r2 --> ne remontent pas au niveau app
				juste du dequeue. le boulot de remote app de construire
				les asdus
				
			3) user data et pas de rponse
			
				rien  faire.
				
			4) user data et ack
			
				wait();
		*/
	}
	
	protected boolean needAck = false;

	/**
	 * Request for a service to handle.
	 *
	 * @param service	the requested link service.
	 */
	public void request(LnkService service) throws IEC608705Exception
	{
		SLnkService svc = (SLnkService)service;
		
		svc.address = address;
		
		switch (svc.type)
		{
		case SLnkService.END_OF_INIT:
			endOfInit	= svc;
			break;
			
		case SLnkService.DATA_CLASS_1:
			class1.enqueue(svc);
//			if (needAck)
//			{
//				((SLinkLayer)layer).toLower(cack);
//				needAck = false;
//			}
			break;
		case SLnkService.DATA_CLASS_2:
			CodeProperties.lnklog("Data class 2");
			if (class2.isFull())
			{
				class2.dequeue();
			}
			class2.enqueue(svc);
			break;
		}
	}	
	/**
	 * Evaluation of a remote master order at link layer level.
	 *
	 * @param frame	frame sent by master remote.
	 *
	 * @return a link service corresponding to the received order.
	 */
	public LnkService evaluate(Frame frame) throws IEC608705Exception
	{
		SLnkService svc = null;	
		fcv = frame.control.bool2;
		
		if(fcv)
			if (frame.control.bool1 == fcb)
			{
				
				return lastMessage;
			}
			else
				fcb = !fcb;
				
		switch (frame.control.function)
		{
		
		case Interoperability.SERVICE_S2_SC_RESET_REMOTE_LINK:
			CodeProperties.lnklog("Reset link layer");
			if (fcv)
			{
				fcb = !fcb;
				return null;	
			}
			// reset communication unit.
			while(!class1.isEmpty())
				class1.dequeue();
			while(!class2.isEmpty())
				class2.dequeue();
			fcb = false;	
			if (endOfInit!=null)
			{
				class1.enqueue(endOfInit);
				lastMessage = cack;
			}
			else
				lastMessage = cnack;
			return lastMessage;
			
		case Interoperability.SERVICE_S1_SN_USER_DATA:
			if (fcv)
			{
				fcb = !fcb;
				return null;
			}
			// nothing to do.
			break;
			
		case Interoperability.SERVICE_S2_SC_USER_DATA:
			if (!fcv)
				lastMessage = cnack;
//			else
//			{
//				needAck=true;
//				return null;
//			}
			lastMessage = cack;
			return lastMessage;	
							
		case Interoperability.SERVICE_S3_RR_USER_DATA_CLASS_1:
			if (!fcv)
				return null;
			svc = (SLnkService)class1.dequeue();
			if (svc==null)
				lastMessage = rnack;
			else
				lastMessage = svc;
			return lastMessage;	

		case Interoperability.SERVICE_S3_RR_USER_DATA_CLASS_2:
			if (!fcv)
				return null;
			svc = (SLnkService)class2.dequeue();
			if(svc==null)
				lastMessage = rnack;
			else
			{
				svc.data[2] = (short)1; // oh le vilain tricheur!
				lastMessage = svc;
			}
			return lastMessage;	

		case Interoperability.SERVICE_S3_RR_LINK_STATUS:
			CodeProperties.lnklog("Request Link Status");
			if (fcv)
				return null;
			lastMessage = rlnk;
			return lastMessage;
			
		default:
			// not implemented.
			return null;
		}
		return null;
	}

	protected final SLnkService cack	= new SConfirmLnkSvc(this,true);

	protected final SLnkService cnack	= new SConfirmLnkSvc(this,false);

	protected final SLnkService rnack	= new SRespondNackLnkSvc(this);
	
	protected final SLnkService rlnk	= new SRespondLinkStatusLnkSvc(this);

	
	/**
	 * <p>
	 */
	public void msgFromAppManager()
	{
	}
	public void setNbMeasure(int nb)
	{
		class2 = new FifoQueue(nb+1);
	}
	/**
	 * <p>
	 * Instanciates a new manager for slave link layer.
	 *
	 * @param layer		Link layer reference.
	 * @param address	Local station address.
	 */
	public SLnkManager(SLinkLayer layer, SAppManager app, int address)
	{
		super(address);
		this.layer		= layer;
		this.app		= app;
		class1 			= new FifoQueue(100);
		layer.manager = this;
	}
}	// end SLnkManager.java