/*
 * Class    br.org.scadabr.protocol.iec101.slave101.SAppManager101
 * File     SAppManager101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave101;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.PeriodicService;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.slave.SAppManager;
import br.org.scadabr.protocol.iec101.slave.SApplicationLayer;

import java.util.Enumeration;
/**
 * <p>
 * General contract is to define manager object for application
 * layer. An application manager interfaces Mapping and therefore
 * database with application layer.
 * <p>
 * One one hand, an aplication manager is in charge of handling
 * application procedures and services. It's the command role of
 * such an object. Notice that these mechanisms, the manager must
 * drive, come from database notification via application mapping.
 * Or it can also be internal procedures the manager launches by its
 * own.
 * <p>
 * One the other hand, SAppManager101 must be able to write some protocol
 * data to the database.
 * <p>
 * Notice that for a slave application, there wiil be only one unique
 * instance of this manager. Whereas for a master application, there
 * will be as many SAppManager101 instances as there will be remote stations.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class SAppManager101 extends SAppManager
{
	/**
	 *
	 */
	public SEndOfInitializationAppSvc101 endOfInit;
	public SSynchroAppSvc101 synchroConf;
	public SGeneralInterrogationAppSvc101 giConf;
	public SGeneralInterrogationAppSvc101 giTerm;
	public SGeneralCounterInterrogationAppSvc101 ciConf;
	public SGeneralCounterInterrogationAppSvc101 ciTerm;
	public SDelayAppSvc101 delayConf;


	/**
	 * Start the slave application manager.
	 */
	public void start()
	{
    CodeProperties.applog("Starting cyclic application services");
		layer.receive(endOfInit);
		
		for(int i=0;i<periodic.size();i++)
		{
			((PeriodicService)periodic.elementAt(i)).resume();
		}
	}

	/**
	 * Method to build all ci's asdus.
	 */
	public void cis()
	{
		for(Enumeration e = ((SMapping101)map).enumerateCounters();e.hasMoreElements();)
		{
			AppService proc = (AppService)e.nextElement();
			((SMapping101)map).resolveMapping(proc);
			layer.receive(proc);
		}
	}
	
	public void evaluate(Asdu asdu) throws IEC608705Exception
	{
		switch (asdu.id)
		{
				
		case Interoperability101.ID_C_CD_NA_1:
			if (asdu.cot==Interoperability101.COT_ACTIVATION)
			{
				CodeProperties.applog("Delay acquisition");
				layer.receive(delayConf);
			}
			break;
			
		case Interoperability101.ID_C_CS_NA_1:
			CodeProperties.applog("Clock Synchronization");
			if (((SContext101)context).clock)
			{
				// change system clock
				// event synchro...
				layer.receive(synchroConf);
			}
			else
			{
				// change not allowed
			}
			break;
			
		case Interoperability101.ID_C_IC_NA_1:
			SContext101 c101 = (SContext101)context;
			CodeProperties.applog("General Interrogation");
			c101.gi	= true;
			c101.qoi = ((Short)asdu.value(0,0,0)).shortValue();
			giConf.context = context;
			layer.receive(giConf);
			gis();
			giTerm.context = context;
			layer.receive(giTerm);
			break;
			
		case Interoperability101.ID_C_CI_NA_1:
			// context would be rather be filled by SAsduReader101....
			((SContext101)context).qcc = ((Short)asdu.value(0,0,0)).shortValue();
			ciConf.context = context;
			layer.receive(ciConf);
			cis();
			ciTerm.context = context;
			layer.receive(ciTerm);
			break;
		}
	}	
	/**
	 * Allocates an application layer manager according to local station
	 * context for this slave application.
	 *
	 * @param layer		Application layer reference.
	 * @param context	Execution context.
	 * @param polling	Measure periodicity in seconds.
	 */
	public SAppManager101(SApplicationLayer layer, SContext101 context, int polling)
	{
		super(layer,context,polling);
		endOfInit = new SEndOfInitializationAppSvc101(this,context.caa);
		synchroConf = new SSynchroAppSvc101(this,context.caa);
		giConf = new SGeneralInterrogationAppSvc101(this,context.caa,Interoperability101.COT_ACTIVATIONCONF);
		giTerm = new SGeneralInterrogationAppSvc101(this,context.caa,Interoperability101.COT_ACTIVATION_TERMINATION);
		ciConf = new SGeneralCounterInterrogationAppSvc101(this,context.caa,Interoperability101.COT_ACTIVATIONCONF);
		ciTerm = new SGeneralCounterInterrogationAppSvc101(this,context.caa,Interoperability101.COT_ACTIVATION_TERMINATION);
		delayConf = new SDelayAppSvc101(this,context.caa,Interoperability101.COT_ACTIVATIONCONF);
//		delaySpont = new SDelayAppSvc101(this,context.caa,Interoperability101.COT_SPONTANEOUS);
	}	
}	// end SAppManager101.java
