/*
 * Class    br.org.scadabr.protocol.iec101.slave101.SAsduWriter101
 * File     SAsduWriter101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave101;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduLocator;
import br.org.scadabr.protocol.iec101.common.AsduWriter;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.InformationElement;
import br.org.scadabr.protocol.iec101.common.InformationObject;
import br.org.scadabr.protocol.iec101.common.LnkService;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common.MapSingleValue;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.common101.information.*;

/**
 * This class defines mechanisms to build a 101 asdu to write
 * to lower link layer.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 18.II.03
 */

public class SAsduWriter101 implements AsduWriter
{		
	/**
	 * Cause of transmission field length.
	 */
	private byte szCot = 1;
	/**
	 * Common Address of Asdus field length.
	 */
	private byte szCaa = 1;
	/**
	 * Information Object Address field length.
	 */
	private byte szIoa = 1;
	/**
	 * Link address field length.
	 */
	private byte szLnk = 1;	
	/**
	 * Fills information objects in 103 asdu.
	 *
	 * @param proc	Application procedure reference
	 */
	private InformationObject getIo(AppService proc)
	{
		InformationObject io = null;
		MapAsdu asdu = proc.asdu;
		MapSingleValue[] value = asdu.value;
		AsduLocator locator = null;
		InformationElement element = null;
		SContext101 context = (SContext101)proc.context;
		
		switch (asdu.key.id)
		{
		case Interoperability101.ID_M_SP_NA_1:
//			CodeProperties.applog("[SAsduReader101] type identification _ single point");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SIQ();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			// if general interrogation , cause of transmission = qoi.
			if(context.gi)
				asdu.key.cot = context.qoi;
			break;
							
		case Interoperability101.ID_M_SP_TA_1:
//			CodeProperties.applog("[SAsduReader101] type identification _ single point");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SIQ();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new CP24Time2a());
			// if general interrogation , cause of transmission = qoi.
			if(context.gi)
				asdu.key.cot = context.qoi;
			break;
							
		case Interoperability101.ID_M_SP_TB_1:
//			CodeProperties.applog("[SAsduReader101] type identification _ single point");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SIQ();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new CP56Time2a());
			// if general interrogation , cause of transmission = qoi.
			if(context.gi)
				asdu.key.cot = context.qoi;
			break;
					
		case Interoperability101.ID_M_DP_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new DIQ();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			System.out.println("context.gi="+context.gi+" context.qoi="+context.qoi);
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
			
		case Interoperability101.ID_M_ST_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new VTI();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
		case Interoperability101.ID_M_BO_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new BSI();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
					
		case Interoperability101.ID_M_ME_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new NVA();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;	
					
		case Interoperability101.ID_M_ME_NB_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SVA();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
					
		case Interoperability101.ID_M_ME_NC_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new R32_23();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
					
		case Interoperability101.ID_M_IT_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new BCR();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
					
		case Interoperability101.ID_M_PS_NA_1:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SCD();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QDS((short)128));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
			
		case Interoperability101.ID_C_SC_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ single command");
			io.setElement(new SCO());
			break;
		case Interoperability101.ID_C_DC_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ double command");
			io.setElement(new DCO());
			break;
		case Interoperability101.ID_C_RC_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ regulating step command");
			io.setElement(new RCO());
			break;
		case Interoperability101.ID_C_SE_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ set point command, normalized value");
			io.setElement(new NVA());
			io.setElement(new QOS());
			break;
		case Interoperability101.ID_C_SE_NB_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ set point command, scaled value");
			io.setElement(new SVA());
			io.setElement(new QOS());
			break;
		case Interoperability101.ID_C_SE_NC_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ set point command, short floating point number");
			io.setElement(new R32_23());
			io.setElement(new QOS());
			break;
		case Interoperability101.ID_C_BO_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ bitsrting 32 bits");
			io.setElement(new BSI());//map
			break;
		case Interoperability101.ID_C_IC_NA_1:
			CodeProperties.applog("[SAsduWriter101] type identification _ interrogation command");
			io = new InformationObject();
			io.setElement(new QOI(context.qoi));
			break;
		case Interoperability101.ID_C_CI_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ counter interrogation command");
			io = new InformationObject();
			io.setElement(new QCC(context.qcc));
			break;
		case Interoperability101.ID_C_RD_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ read command");
			// nada
			break;
		case Interoperability101.ID_C_CS_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ clock synchronization command");
			io = new InformationObject();
			io.setElement(new CP56Time2a());
			break;
		case Interoperability101.ID_C_TS_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ test command");
			io = new InformationObject();
			io.setElement(new FBP());
			break;
		case Interoperability101.ID_C_RP_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ reset process command");
			io.setElement(new QRP());
			break;			
		case Interoperability101.ID_C_CD_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ delay acquisition command");
			io = new InformationObject();
			io.setElement(new CP16Time2a());
			break;
		case Interoperability101.ID_M_EI_NA_1:
			SEndOfInitializationAppSvc101 svc = (SEndOfInitializationAppSvc101)proc;
			io = new InformationObject();
			io.setElement(new COI(svc.coi));
			break;
		case Interoperability101.ID_P_ME_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ parameter of measured value, normalized value");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new NVA();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QPM((short)127));
			System.out.println("context.gi="+context.gi+" context.qoi="+context.qoi);
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
		case Interoperability101.ID_P_ME_NB_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ parameter of measured value, scaled value");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new SVA();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QPM((short)127));
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
		case Interoperability101.ID_P_ME_NC_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ parameter of measured value, short floating point number");
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new R32_23();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			io.setElement(new QPM((short)127)); //proc
			if (context.gi)
				asdu.key.cot = context.qoi;
			break;
		case Interoperability101.ID_P_AC_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ parameter activation");
			io.setElement(new QPA());
			break;
		case Interoperability101.ID_F_FR_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ file ready");
			io.setElement(new NOF());
			io.setElement(new LOF());
			io.setElement(new FRQ());
			break;
		case Interoperability101.ID_F_SR_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ section ready");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new LOF());
			io.setElement(new SRQ());
			break;
		case Interoperability101.ID_F_SC_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ call directory, select file, call file, call section");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new SCQ());
			break;
		case Interoperability101.ID_F_LS_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ last section, last segment");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new LSQ());
			io.setElement(new CHS());
			break;
		case Interoperability101.ID_F_AF_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ last section, last segmentack file, ack section");
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(new AFQ());
			break;
		case Interoperability101.ID_F_SG_NA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ segment");
			LOS los;
			io.setElement(new NOF());
			io.setElement(new NOS());
			io.setElement(los = new LOS());
			for(short j=0;j<los.octetset[0];j++)
			{
				io.setElement(new OCTET());
			}
			break;
		case Interoperability101.ID_F_DR_TA_1:
//			CodeProperties.applog("[SAsduWriter101] type identification _ directory");
			// VSQ gives number of elements !!!!
			io.setElement(new NOF());
			io.setElement(new LOF());
			io.setElement(new SOF());
			io.setElement(new CP56Time2a());
			break;		
		default:
			CodeProperties.log("FUNCTION NOT IMPLEMENTED."+proc.asdu.key.id);
			break;			
		}
		
		return io;
	}
	
	/**
	 * Main method of this asdu writer factory for master 101 station.
	 *
	 * @param proc	An application procedure to turn into link service.
	 *
	 * @return the built asdu hold by a link service object.
	 */
	public LnkService build(AppService proc)
	{
		int size = 0;
		LnkService lnk	= proc.svc;
		if (proc.asdu!=null)// user data
		{
			AsduKey101 key = (AsduKey101)proc.asdu.key;
			InformationObject io = getIo(proc);
			for(int i=0;i<io.ie.size();i++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(i);
				size += ie.octetset.length;
			}
			int i=0;
			lnk.data		= new short[2+szCot+szCaa+szIoa+size];
			lnk.data[i++]	= key.id;
			lnk.data[i++]	= key.vsq;
			lnk.data[i++]	= key.cot;
			if(szCot==2)
				lnk.data[i++]	= key.oa;
			switch (szCaa)	
			{
			case 1:
				lnk.data[i++] = (short)key.caa;
				break;
			case 2:
				lnk.data[i++]	= (short)((key.caa & 0x00FF));
				lnk.data[i++]	= (short)((key.caa & 0xFF00) >> 8);
				break;			
			}
			for(byte j=0;j<szIoa;j++)
				lnk.data[i++] = 0;
			for(int j=0;j<io.ie.size();j++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(j);
				for(int k=0;k<ie.octetset.length;k++)
					lnk.data[i++] = ie.octetset[k];
			}
		}
		
		return lnk;	
	}
	
	/**
	 * Constructs a writer factory specialized for 101 master station.
	 *
	 * @param szCot	Cause of transmission length (1-2).
	 * @param szCaa	Common address of asdus length (1-2).
	 * @param szIoa	Information object address length (1-2-3).
	 * @param szLnk	Link address field length (1-2).
	 */
	public SAsduWriter101(byte szCot, byte szCaa, byte szIoa, byte szLnk)
	{
		this.szCot = szCot;
		this.szCaa = szCaa;
		this.szIoa = szIoa;
	}
	
};	// end SAsduWriter101.java