/*
 * Class    br.org.scadabr.protocol.iec101.slave101.SGeneralInterrogationAppSvc101
 * File     SGeneralInterrogationAppSvc101.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave101;

import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common101.AsduKey101;
import br.org.scadabr.protocol.iec101.common101.Interoperability101;
import br.org.scadabr.protocol.iec101.slave.SDataClass1AppSvc;
/**
 * <p>
 * This class defines a 'general interrogation' or 'station interrogation'
 * mechanism. In 101 implementation, such a command can be addressed to whole
 * station via 'global interrogation' mechanism or reduced to part of station
 * via 'group 1 to 16 interrogation'...
 * <p>
 * ///// IEC DESCRIPTION
 * <p>
 * This class extends MAppSvc, and general contract for such a class
 * is to format the procedure to make it match with a standardized
 * application procedure.
 *
 * @author	lionnel cauvy
 * @version	1.0
 * @date 01.II.03
 */
public class SGeneralInterrogationAppSvc101 extends SDataClass1AppSvc
{
	public int ioa = 0;	
	/**
	 * <p>
	 * Allocates a new procedure for remote station general interrogation triggered
	 * by database notifier.
	 *
	 * @param manager	Application layer manager.
	 * @param caa		Common address of asdus.
	 * @param cot		Activation confirmation or activation termination.
	 */
	public SGeneralInterrogationAppSvc101(SAppManager101 manager, int caa, byte cot)
	{
		super(
			manager,
			// asdu to build
			new MapAsdu(
				new AsduKey101(
					Interoperability101.ID_C_IC_NA_1,
					(short)1,
					cot,
					(short)NOTUSED,
					caa
				)
			)	
		);	
	}	
}	// end SGeneralInterrogationAppSvc101.java
