/*
 * Class    br.org.scadabr.protocol.iec101.slave103.SAppManager103
 * File     SAppManager103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave103;

import br.org.scadabr.protocol.iec101.common.Asdu;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.IEC608705Exception;
import br.org.scadabr.protocol.iec101.common.PeriodicService;
import br.org.scadabr.protocol.iec101.common103.Asdu103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.slave.SAppManager;
import br.org.scadabr.protocol.iec101.slave.SApplicationLayer;

/**
 * <p>
 * General contract is to define manager object for application
 * layer. An application manager interfaces Mapping and therefore
 * database with application layer.
 * <p>
 * One one hand, an aplication manager is in charge of handling
 * application procedures and services. It's the command role of
 * such an object. Notice that these mechanisms, the manager must
 * drive, come from database notification via application mapping.
 * Or it can also be internal procedures the manager launches by its
 * own.
 * <p>
 * One the other hand, SAppManager103 must be able to write some protocol
 * data to the database.
 * <p>
 * Notice that for a slave application, there wiil be only one unique
 * instance of this manager. Whereas for a master application, there
 * will be as many SAppManager103 instances as there will be remote stations.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class SAppManager103 extends SAppManager
{
	/**
	 *
	 */
	public SIdentificationAppSvc103 endOfInit;
	public SSynchroAppSvc103 endOfSynchro;
	public SEndGiAppSvc103 endOfGi;
	public SEndGeneralCdAppSvc103 endOfCd;
	
	/**
	 * Start the slave application manager.
	 */
	public void start()
	{
    	CodeProperties.applog("Starting cyclic application services");
		layer.receive(endOfInit);
		
		for(int i=0;i<periodic.size();i++)
			((PeriodicService)periodic.elementAt(i)).resume();
	}
	/**
	 * Trigger event from periodic task.
	 * 
	 * @param id	The periodic task identifier.
	 */
	
	public void evaluate(Asdu asdu) throws IEC608705Exception
	{
		switch (asdu.id)
		{
		case Interoperability103.ID_SYNCHRO:
			if (((SApplicationLayer)layer).clock)
			{
				// change system clock
				// event synchro...
				layer.receive(endOfSynchro);
			}
			else
			{
				// change not allowed
			}
			break;
		case Interoperability103.ID_INIT_GI:
			gis();
			// context would be rather be filled by SAsduReader103....
			((SContext103)context).sin = ((Short)asdu.value(0,0,0)).shortValue();
			layer.receive(endOfGi);
			break;
		case Interoperability103.ID_GENERAL_CD:
			// write some data to database?
			boolean res = map.write(asdu);
			// ACK OR NACK COM ACCORDING TO RES (TRUE IF EFFECTIVE WRITING)
			Asdu103 data = (Asdu103)asdu;
			SAckComAppSvc103 ackCd = new SAckComAppSvc103(this,res,asdu.caa,data.ft,data.in,((Byte)asdu.value(0,0,0)).byteValue());
			ackCd.context = context;
			layer.receive(ackCd);
			break;
		}
	}
	
	/**
	 * Allocates an application layer manager according to local station
	 * context for a slave application and in regards to a single remote
	 * slave station for a master application.
	 *
	 * @param layer		Application layer reference.
	 * @param context	Execution context.
	 */
	public SAppManager103(SApplicationLayer layer, SContext103 context, int polling)
	{
		super(layer,context,polling);
		endOfInit = new SIdentificationAppSvc103(this,Interoperability103.COT_RESET_CU,context.caa);
		endOfSynchro = new SSynchroAppSvc103(this,context.caa);
		endOfGi = new SEndGiAppSvc103(this,context.caa);
	}	
}	// end SAppManager103.java
