/*
 * Class    br.org.scadabr.protocol.iec101.slave103.SAsduWriter103
 * File     SAsduWriter103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave103;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduLocator;
import br.org.scadabr.protocol.iec101.common.AsduWriter;
import br.org.scadabr.protocol.iec101.common.CodeProperties;
import br.org.scadabr.protocol.iec101.common.InformationElement;
import br.org.scadabr.protocol.iec101.common.InformationObject;
import br.org.scadabr.protocol.iec101.common.LnkService;
import br.org.scadabr.protocol.iec101.common.MapAsdu;
import br.org.scadabr.protocol.iec101.common.MapSingleValue;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.common103.Interoperability103;
import br.org.scadabr.protocol.iec101.common103.PrivateAsdu103;
import br.org.scadabr.protocol.iec101.common103.PrivateRange103;
import br.org.scadabr.protocol.iec101.common103.information.*;

/**
 * This class defines mechanisms to build a 103 asdu to write
 * to lower link layer.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */

public class SAsduWriter103 implements AsduWriter
{
	/**
	 * Private range reference. May be used by getIo function if asdu type
	 * identification is not recognized as a standard one.
	 */
	protected PrivateRange103 prange;
	/**
	 * Fills information objects in a 103 asdu.
	 *
	 * @param proc	Application procedure reference.
	 *
	 * @return the formed information object.
	 */
	private InformationObject getIo(AppService proc)
	{
		SContext103 context = (SContext103)proc.context;
		MapAsdu asdu = proc.asdu;
		InformationObject io = null;
		MapSingleValue[] value = asdu.value;
		AsduLocator locator = null;
		InformationElement element = null;
		switch (asdu.key.id)
		{
		case Interoperability103.ID_TIME_MSG:
			io = new InformationObject();
			// special SAckComSvc103...
			if (proc instanceof SAckComAppSvc103)
			{
				SAckComAppSvc103 service = (SAckComAppSvc103)proc;
				io.setElement(new DPI(service.dpi));
			}
			else
			{
				// we can check mapping compatibility...
				if(value.length!=1)
					return null;	
				locator = value[0].locator;
				if(locator.pio!=0 || locator.pie!=0 || locator.pval!=0)
					return null;
				// mapping is ok, just go on...	
				// Notice dpi comes from mapping, time can be computed by system,
				// and last sin comes from context object.
				element = new DPI();
				element.setData(value[0].mapped);
				io.setElement(element);
			}
			io.setElement(new CP32Time2a());
			io.setElement(new SIN(context.sin));
			break;
		case Interoperability103.ID_RELATIVE_TIME_MSG:
			io = new InformationObject();
			// we must check mapping compatibility...
			if(value.length!=1)
				return null;	
			locator = value[0].locator;
			if(locator.pio!=0 || locator.pie!=0 || locator.pval!=0)
				return null;
			// mapping is ok, just go on...	
			// Notice dpi comes from mapping, time can be computed by system,
			// and last ret, fan & sin come from context object.
			element = new DPI();
			element.setData((short)0,value[0].mapped);
			io.setElement(element);
			element = new RET();
			element.setData(new Integer(context.ret));
			io.setElement(element);
			element = new FAN();
			element.setData(new Integer(context.fan));
			io.setElement(element);
			io.setElement(new CP32Time2a());
			io.setElement(new SIN(context.sin));
			break;
		case Interoperability103.ID_MEASURANDSI:
			io = new InformationObject();
			// we can sort out used from not used information elements.
			for(int i=0;i<4;i++)
				io.setElement(new MEA());
			for(int i=0;i<value.length;i++)
				((InformationElement)io.ie.elementAt(value[i].locator.pie)).setData(value[i].mapped);	
			break;		
		case Interoperability103.ID_MEASURANDSII:
			io = new InformationObject();
			// we can sort out used from not used information elements.
			for(int i=0;i<9;i++)
				io.setElement(new MEA());
			for(int i=0;i<value.length;i++)
				((InformationElement)io.ie.elementAt(value[i].locator.pie)).setData(value[i].mapped);	
			break;
		case Interoperability103.ID_IDENTIFICATION:
			io = new InformationObject();
			io.setElement(new COL(SContext103.COL));
			for(int i=0;i<context.asc.length;i++)
				io.setElement(new ASC(context.asc[i]));
			break;
		case Interoperability103.ID_END_GI:
			io = new InformationObject();
			io.setElement(new SCN(context.sin));//same value for sin or scn.
			break;
		case Interoperability103.ID_SYNCHRO:
			io = new InformationObject();
			io.setElement(new CP56Time2a());
			break;			
		default:
			// is it private ?
			// is this a private asdu?
			PrivateAsdu103 pasdu = (PrivateAsdu103)prange.getAsdu(asdu.key.id);
			if (pasdu!=null)
			{				
				try
				{				
					io = new InformationObject();					
					for(byte w=0;w<pasdu.nio;w++)
					{
						for(int z=0;z<pasdu.ie.length;z++)
						{						
							Class clazz = (Class)prange.CLASSES.get(pasdu.ie[z]);
							InformationElement elt = (InformationElement)clazz.newInstance();								
							for(short y=0;y<elt.octetset.length;y++)
							{
								elt.setOctet(y,(short)0);
							}
							io.setElement(elt);	
						}
						for(int z=0;z<value.length;z++)
							((InformationElement)io.ie.elementAt(value[z].locator.pie)).setData(value[z].mapped);	
					}
				}
				catch(InstantiationException e2)
				{
					CodeProperties.applog("[MAsduWriter103] not instanciable "+e2);
				}
				catch(IllegalAccessException e3)
				{
					CodeProperties.applog("[SAsduWriter103] bad access "+e3);
				}
			}
			// no sorry.....
			else
			{
				CodeProperties.applog("[SAsduWriter103] type identification _ unknow type! "+asdu.key.id);
			}
			break;			
		}		
		return io;
	}	
	/**
	 * Implementer of AsduWriter interface must define this method to build
	 * asdus, in the form of a link service that contains an asdu, from application
	 * procedure data.
	 *
	 * @param proc	an application procedure to turn into link service.
	 *
	 * @return the built asdu in the form of a link service object.
	 */
	public LnkService build(AppService proc)
	{
		int size = 0;
		LnkService lnk	= proc.svc;
		if (proc.asdu!=null)// test on user data presence
		{
			AsduKey103 key = (AsduKey103)proc.asdu.key;
			InformationObject io = getIo(proc);
			for(int i=0;i<io.ie.size();i++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(i);
				size += ie.octetset.length;
			}
			int i=0;
			lnk.data		= new short[size+Interoperability103.HEADER_ASDU_SIZE];
			lnk.data[i++]	= key.id;
			lnk.data[i++]	= key.vsq;
			lnk.data[i++]	= key.cot;
			lnk.data[i++]	= (short)key.caa;
			lnk.data[i++]	= key.ft;
			lnk.data[i++]	= key.in;
			for(int j=0;j<io.ie.size();j++)
			{
				InformationElement ie = (InformationElement)io.ie.elementAt(j);
				for(int k=0;k<ie.octetset.length;k++)
					lnk.data[i++] = ie.octetset[k];
			}
		}		
		return lnk;	
	}	
}	// end SAsduWriter103.java