/*
 * Class    br.org.scadabr.protocol.iec101.slave103.SMapping103
 * File     SMapping103.java
 * Author   lionnel cauvy
 */
package br.org.scadabr.protocol.iec101.slave103;

import br.org.scadabr.protocol.iec101.common.AppService;
import br.org.scadabr.protocol.iec101.common.AsduKey;
import br.org.scadabr.protocol.iec101.common103.Asdu103;
import br.org.scadabr.protocol.iec101.common103.AsduKey103;
import br.org.scadabr.protocol.iec101.slave.SMapping;

import java.util.Vector;
import java.util.Enumeration;

/**
 * <p>
 * This class defines mapping for slave 103 application.
 *
 * @author lionnel cauvy
 * @version 1.0
 * @date 01.II.03
 */
public class SMapping103 extends SMapping
{
	/**
	 * Identification message. CAUSE OF TRANSMISSION LET EMPTY.
	 */
	private Vector id;
	/**
	 * Add an 'identification' asdu to map.
	 *
	 * @param proc	The procedure to record.
	 */	
	public void addIdentification(AppService proc)
	{
		id.addElement(proc);
	}
	/**
	 * Get accessor on all identification asdus recorded
	 * in this mapping.
	 *
	 * @return an enumeration of asdus.
	 */
	public Enumeration enumerateIds()
	{
		return id.elements();
	}
	/**
	 * <p>
	 * Generates a mapping key. Asdu from 103 protocol contains supplementary
	 * data such as Function Type & Information Number. These attributes are
	 * used to map 103 data to luciol database. So this only interest of this
	 * class is ability to create 103 mapping key.
	 *
	 * @param object	a 103 formatted asdu to turn into mapping key.
	 *
	 * @return	the requested mapping key.
	 */	
	protected AsduKey instanciateKey(Object object)
	{		
		if (object instanceof Asdu103)
		{
			Asdu103 asdu = (Asdu103)object;			
			AsduKey103 key = new AsduKey103(
				asdu.id,
				asdu.vsq,
				AsduKey103.NOTUSED,
				asdu.caa,
				asdu.ft,
				asdu.in
			);
			return key;
		}		
		return null;
	}	
	/**
	 * <p>
	 * Allocates a mapping for slave 103 application.
	 *
	 * @param manager	Application layer manager reference.
	 */		
	public SMapping103(SAppManager103 manager)
	{
		super(manager);
		id = new Vector();
	}
}	// end SMapping103.java