//
// Null output stream
// 
// Class     br.org.scadabr.io.NullOutputStream.java
// File      NullOutputStream.java
// Author    Alexis BIETTI <alexis.bietti@sysaware.com> 
// (c)       SysAware <http://www.sysaware.com/>
//
package br.org.scadabr.protocol.lib.xml;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

/**
 * This output stream discard all the data that is written to it.
 * 
 * @author <a href="mailto:alexis.bietti@sysaware.com">Alexis BIETTI</a>
 * @version 1.0 - 14 avr. 03
 */
public class NullOutputStream extends OutputStream {
  
  private static NullOutputStream nullOutputStream = null;
  private static PrintStream nullPrintStream = null;
  
  /**
   * Return an instance of NullOutputStream that can be shared.
   * 
   * @return OutputStream
   */
  public static NullOutputStream getNullOutputStream() {
    if (nullOutputStream == null) {
      nullOutputStream = new NullOutputStream();
    }
    return nullOutputStream;
  }
  
  /**
   * Return an instance of {@link java.io.PrintStream} that writes to a
   * NullOutputStream
   * 
   * @return PrintStream
   */
  public static PrintStream getNullPrintStream() {
    if (nullPrintStream == null) {
      nullPrintStream = new PrintStream(getNullOutputStream());
    }
    return nullPrintStream;
  }
  
  public void write(int b) throws IOException {
  }
  
  public void close() throws IOException {
  }

  public void flush() throws IOException {
  }

  public void write(byte[] b, int off, int len) throws IOException {
  }

  public void write(byte[] b) throws IOException {
  }
}

// End NullOutputStream.java
