package com.i2msolucoes.alpha24j;

import com.i2msolucoes.alpha24j.util.ByteUtils;
import com.serotonin.util.queue.ByteQueue;

public class DeviceLocator {

	private byte deviceCode;
	private DeviceCodes code;
	private int deviceNumber;

	public enum DeviceCodes {
		SYSTEM_BIT(1, (byte) 0x40, true, false), INPUT_TERMINAL(2, (byte) 0x41,
				true, true), EXTERNAL_INPUT(3, (byte) 0x41, true, true), OUTPUT_TERMINAL(
				4, (byte) 0x42, true, true), EXTERNAL_OUTPUT(5, (byte) 0x42,
				true, true), KEY_INPUT(6, (byte) 0x44, true, true), LINK_INPUT(
				7, (byte) 0x45, true, true), LINK_OUTPUT(8, (byte) 0x46, true,
				true), CONTROL_DEVICE(9, (byte) 0x47, true, true), ANALOG_IN(
				10, (byte) 0x61, true, false), COMM_BIT_DEVICE(11, (byte) 0x48,
				true, true), COMM_WORD_DEVICE(12, (byte) 0x69, true, true);

		private final byte code;
		private final int id;
		private final boolean readAccess;
		private final boolean writeAccess;

		private DeviceCodes(int id, byte code, boolean readAccess,
				boolean writeAccess) {
			this.id = id;
			this.code = code;
			this.readAccess = readAccess;
			this.writeAccess = writeAccess;
		}

		public boolean isReadAccess() {
			return readAccess;
		}

		public boolean isWriteAccess() {
			return writeAccess;
		}

		public int getId() {
			return this.id;
		}

		public byte getCode() {
			return this.code;
		}

		public static DeviceCodes toDeviceCode(int id) {
			DeviceCodes[] codes = DeviceCodes.values();
			for (DeviceCodes code : codes) {
				if (code.getId() == id)
					return code;
			}
			return null;
		}
	}

	public DeviceLocator(DeviceCodes deviceCode, int deviceNumber) {
		this.code = (deviceCode);
		this.deviceCode = deviceCode.getCode();
		this.deviceNumber = deviceNumber;
	}

	public byte[] getBytes() {
		ByteQueue queue = new ByteQueue();
		queue.push(deviceCode);
		queue.push(ByteUtils.intTo2Bytes(deviceNumber));
		return queue.popAll();
	}

	public boolean isWordDevice() {
		return (this.deviceCode == DeviceCodes.COMM_WORD_DEVICE.getCode())
				|| this.deviceCode == DeviceCodes.ANALOG_IN.getCode();
	}

	public DeviceCodes getCode() {
		return code;
	}
}
