package com.i2msolucoes.alpha24j.element;

public class Elements {
	public static final byte COMPLETE_FLAG = 0x21;

	public enum CommandCodes {
		READ((byte) 0x00), WRITE((byte) 0x01), RUN_STOP((byte) 0x10);

		private final byte code;

		private CommandCodes(byte code) {
			this.code = code;
		}

		public byte getCode() {
			return this.code;
		}
	}

	public enum ControlCodes {
		STX((byte) 0x02), ETX((byte) 0x03), ENQ((byte) 0x05), ACK((byte) 0x06), NACK(
				(byte) 0x15), COMPLETION((byte) 0x21);

		private final byte code;

		private ControlCodes(byte code) {
			this.code = code;
		}

		public byte getCode() {
			return this.code;
		}
	}

	public enum ErrorCodes {
		SUM_CHECK_ERROR((byte) 0x01,
				"There is an error in the Sum Check value used in Protocol Format \"B\""), PROTOCOL_ERROR(
				(byte) 0x02,
				"There is a discrepancy in the number of communication bytes in the message OR command for READ, WRITE or RUN/STOP operation is incorrect."), DEVICE_ERROR(
				(byte) 0x03,
				"The data stored in the device specification is outside the device range OR the Device Status setting value in out of the intended device's range OR the write protection for the memory cassette is enabled."), REMOTE_ERROR(
				(byte) 0x04,
				"There is a problem in the remote Start/Stop operation. Usually the cause of this error is an error in the controller program."), DATE_TIME_ERROR(
				(byte) 0x05, "The Date or Time is invalid.");

		private final byte code;
		private final String description;

		private ErrorCodes(byte code, String description) {
			this.code = code;
			this.description = description;
		}

		public byte getCode() {
			return this.code;
		}

		public String getDescription() {
			return this.description;
		}

		public static boolean isValidErrorCode(byte value) {
			ErrorCodes[] codes = ErrorCodes.values();
			for (ErrorCodes errorCode : codes) {
				if (errorCode.getCode() == value)
					return true;
			}
			return false;
		}

		public static ErrorCodes toErrorCode(byte value) {
			ErrorCodes[] codes = ErrorCodes.values();
			for (ErrorCodes errorCode : codes) {
				if (errorCode.getCode() == value)
					return errorCode;
			}
			return null;
		}
	}

	public enum FrameFormat {
		A((byte) 0x40), B((byte) 0x41);

		private final byte code;

		private FrameFormat(byte code) {
			this.code = code;
		}

		public byte getCode() {
			return this.code;
		}

		public static boolean isValidFormatNumber(byte value) {
			FrameFormat[] codes = FrameFormat.values();
			for (FrameFormat format : codes) {
				if (format.getCode() == value)
					return true;
			}
			return false;
		}

	}
}
