package com.i2msolucoes.alpha24j.frame.request;

import com.i2msolucoes.alpha24j.element.Elements.ControlCodes;
import com.i2msolucoes.alpha24j.element.Elements.FrameFormat;
import com.i2msolucoes.alpha24j.util.ByteUtils;
import com.serotonin.util.queue.ByteQueue;

abstract public class Alpha2RequestFrame {

	public abstract byte getFormatNumber();

	protected abstract byte getNoCommBytes();

	protected abstract byte[] getDataBytes();

	private ByteQueue frame = new ByteQueue();
	private byte station;

	public Alpha2RequestFrame(byte station) {
		this.station = station;
	}

	public byte[] getFrame() {
		frame.clear();

		frame.push(ControlCodes.STX.getCode());
		frame.push(getNoCommBytes()); // no off comm bytes
		byte frameFormat = getFormatNumber();
		frame.push(frameFormat);
		frame.push(station);

		frame.push(getDataBytes());

		if (isCRCNeeded(frameFormat))
			frame.push(doCRC());

		return frame.popAll();
	}

	private boolean isCRCNeeded(byte frameFormat) {
		if (frameFormat == FrameFormat.A.getCode()) {
			return false;
		}
		return true;
	}

	private byte[] doCRC() {
		return ByteUtils.calculateCRC(frame);
	}

	public boolean isReadCommandMessage() {
		return false;
	}

	public boolean isWriteCommandMessage() {
		return false;
	}

	public boolean isLineCheckMessageMessage() {
		return false;
	}

	public byte getStationNumber() {
		return this.station;
	}
}
