package com.i2msolucoes.alpha24j.frame.response;

import com.i2msolucoes.alpha24j.element.Elements.ControlCodes;
import com.i2msolucoes.alpha24j.element.Elements.FrameFormat;
import com.i2msolucoes.alpha24j.exception.ErrorMessageException;
import com.i2msolucoes.alpha24j.frame.request.Alpha2RequestFrame;
import com.serotonin.util.queue.ByteQueue;

abstract public class Alpha2ResponseFrame {
	protected Alpha2RequestFrame sent;
	protected ByteQueue received;

	abstract protected void checkAndBuildResponse() throws Exception;

	public Alpha2ResponseFrame(Alpha2RequestFrame sent, ByteQueue received)
			throws Exception {
		this.sent = sent;
		this.received = received;
	}

	public static Alpha2ResponseFrame buildResponseFrame(
			Alpha2RequestFrame sent, ByteQueue received) throws Exception {

		Alpha2ResponseFrame response = null;
		try {
			checkCommonData(sent, received);
			if (isErrorMessage(sent, received)) {
				response = new ErrorResponseFrame(sent, received);
			} else if (sent.isReadCommandMessage()) {
				response = new ReadResponseFrame(sent, received);
			} else if (sent.isLineCheckMessageMessage()) {
				response = new LineCheckResponseFrame(sent, received);
			} else if (sent.isWriteCommandMessage()) {
				response = new WriteResponseFrame(sent, received);
			} else {
				response = new InvalidFrame(sent, received);
			}
			response.checkAndBuildResponse();
		} catch (ErrorMessageException e) {
			response = e.getErrorFrame();
		} catch (Exception e) {
			response = new InvalidFrame(sent, received);
		}
		return response;
	}

	private static boolean isErrorMessage(Alpha2RequestFrame sent,
			ByteQueue received) throws Exception {
		try {
			if (!sent.isLineCheckMessageMessage()) {
				ensureTrue(received.peek(4) != ControlCodes.NACK.getCode());
				return false;
			}
		} catch (Exception e) {
			return true;
		}
		return false;
	}

	private static void checkCommonData(Alpha2RequestFrame sent,
			ByteQueue received) throws Exception {
		ensureTrue(received.size() > 4);
		ensureTrue(received.peek(0) == ControlCodes.STX.getCode());
		ensureTrue(received.peek(1) > 0);
		ensureTrue(FrameFormat.isValidFormatNumber(received.peek(2)));
		ensureTrue(received.peek(3) == sent.getStationNumber());

	}

	protected static void ensureTrue(boolean bool) throws Exception {
		if (!bool)
			throw new Exception("ensureTrue: INVALID!");
	}

	public boolean isInvalidFrame() {
		return false;
	}

	public boolean isErrorMessage() {
		return false;
	}

	public boolean isReadResponseMessage() {
		return false;
	}

	public boolean isWriteResponseMessage() {
		return false;
	}

	public boolean isLineCheckResponseMessage() {
		return false;
	}

	public ByteQueue getReceived() {
		return received;
	}

}
