package com.i2msolucoes.alpha24j.frame.response;

import com.i2msolucoes.alpha24j.element.Elements.ControlCodes;
import com.i2msolucoes.alpha24j.element.Elements.ErrorCodes;
import com.i2msolucoes.alpha24j.frame.request.Alpha2RequestFrame;
import com.serotonin.util.queue.ByteQueue;

public class LineCheckResponseFrame extends Alpha2ResponseFrame {

	private boolean lineStatus = false;
	private ErrorCodes errorCode;

	public LineCheckResponseFrame(Alpha2RequestFrame sent, ByteQueue received)
			throws Exception {
		super(sent, received);
	}

	@Override
	protected void checkAndBuildResponse() throws Exception {
		ensureTrue(received.size() > 4);
		byte ackOrNack = received.peek(4);
		if (ackOrNack == ControlCodes.ACK.getCode()) {
			ensureTrue(received.size() == 5);
			lineStatus = true;
		} else if (ackOrNack == ControlCodes.NACK.getCode()) {
			ensureTrue(received.size() == 6);
			ensureTrue(ErrorCodes.isValidErrorCode(received.peek(5)));
			lineStatus = false;
			errorCode = ErrorCodes.toErrorCode(received.peek(5));
		} else
			throw new Exception();

	}

	public boolean isLineOK() {
		return lineStatus;
	}

	public ErrorCodes getErrorCode() {
		return errorCode;
	}

	@Override
	public boolean isLineCheckResponseMessage() {
		return true;
	}

}
