package com.i2msolucoes.alpha24j.frame.response;

import java.util.ArrayList;
import java.util.List;

import com.i2msolucoes.alpha24j.DeviceLocator;
import com.i2msolucoes.alpha24j.DeviceValue;
import com.i2msolucoes.alpha24j.element.Elements;
import com.i2msolucoes.alpha24j.element.Elements.CommandCodes;
import com.i2msolucoes.alpha24j.element.Elements.ControlCodes;
import com.i2msolucoes.alpha24j.element.Elements.ErrorCodes;
import com.i2msolucoes.alpha24j.element.Elements.FrameFormat;
import com.i2msolucoes.alpha24j.exception.ErrorMessageException;
import com.i2msolucoes.alpha24j.frame.request.Alpha2RequestFrame;
import com.i2msolucoes.alpha24j.frame.request.ReadCommand;
import com.i2msolucoes.alpha24j.util.ByteUtils;
import com.serotonin.util.queue.ByteQueue;

public class ReadResponseFrame extends Alpha2ResponseFrame {

	private List<DeviceValue> values;

	public ReadResponseFrame(Alpha2RequestFrame sent, ByteQueue received)
			throws Exception {
		super(sent, received);
	}

	@Override
	protected void checkAndBuildResponse() throws Exception {
		checkErrorResponseFrame();

		ensureTrue(received.size() > 10);
		ensureTrue(received.peek(4) == Elements.COMPLETE_FLAG);
		ensureTrue(received.peek(5) == ControlCodes.STX.getCode());
		ensureTrue(received.peek(6) > 0);

		ensureTrue(received.peek(7) == FrameFormat.B.getCode());
		ensureTrue(received.peek(8) == sent.getStationNumber());
		ensureTrue(received.peek(9) == CommandCodes.READ.getCode());

		int numberOfDevices = received.peek(10);
		ensureTrue(numberOfDevices > 0);

		List<DeviceLocator> locators = ((ReadCommand) sent).getDevices();
		List<DeviceValue> values = new ArrayList<DeviceValue>();

		int i = 11;
		for (DeviceLocator deviceLocator : locators) {
			DeviceValue device = new DeviceValue();
			device.setLocator(deviceLocator);
			int v = 0;
			if (deviceLocator.isWordDevice()) {
				v = ByteUtils.bytesToInt(received.peek(i, 2));
				i++;
			} else {
				v = received.peek(i);
			}
			i++;
			device.setValue("" + v);
			values.add(device);
		}
		ensureTrue(received.peek(i) == ControlCodes.ETX.getCode());

		this.values = values;
	}

	private void checkErrorResponseFrame() throws Exception {
		boolean isErrorMessage = false;

		try {
			ensureTrue(received.peek(7) == FrameFormat.A.getCode());
			ensureTrue(received.peek(8) == sent.getStationNumber());
			ensureTrue(received.peek(9) == ControlCodes.NACK.getCode());
			ensureTrue(ErrorCodes.isValidErrorCode(received.peek(10)));
			isErrorMessage = true;
		} catch (Exception e) {

		}

		if (isErrorMessage)
			throw new ErrorMessageException(new ErrorResponseFrame(sent,
					received, ErrorCodes.toErrorCode(received.peek(10))));
	}

	@Override
	public boolean isReadResponseMessage() {
		return true;
	}

	public List<DeviceValue> getValues() {
		return values;
	}

}
