package com.i2msolucoes.alpha24j.frame.response;

import com.i2msolucoes.alpha24j.element.Elements;
import com.i2msolucoes.alpha24j.element.Elements.ControlCodes;
import com.i2msolucoes.alpha24j.element.Elements.ErrorCodes;
import com.i2msolucoes.alpha24j.element.Elements.FrameFormat;
import com.i2msolucoes.alpha24j.exception.ErrorMessageException;
import com.i2msolucoes.alpha24j.frame.request.Alpha2RequestFrame;
import com.serotonin.util.queue.ByteQueue;

public class WriteResponseFrame extends Alpha2ResponseFrame {

	public WriteResponseFrame(Alpha2RequestFrame sent, ByteQueue received)
			throws Exception {
		super(sent, received);
	}

	@Override
	protected void checkAndBuildResponse() throws Exception {
		checkErrorResponseFrame();
		ensureTrue(received.size() > 5);
		ensureTrue(received.peek(4) == Elements.COMPLETE_FLAG);
		ensureTrue(received.peek(5) == ControlCodes.STX.getCode());
		ensureTrue(received.peek(6) > 0);

		ensureTrue(received.peek(7) == FrameFormat.A.getCode());
		ensureTrue(received.peek(8) == sent.getStationNumber());
		ensureTrue(received.peek(9) == ControlCodes.ACK.getCode());
	}

	@Override
	public boolean isWriteResponseMessage() {
		return true;
	}

	private void checkErrorResponseFrame() throws Exception {
		boolean isErrorMessage = false;

		try {
			ensureTrue(received.peek(7) == FrameFormat.A.getCode());
			ensureTrue(received.peek(8) == sent.getStationNumber());
			ensureTrue(received.peek(9) == ControlCodes.NACK.getCode());
			ensureTrue(ErrorCodes.isValidErrorCode(received.peek(10)));
			isErrorMessage = true;
		} catch (Exception e) {

		}

		if (isErrorMessage)
			throw new ErrorMessageException(new ErrorResponseFrame(sent,
					received, ErrorCodes.toErrorCode(received.peek(10))));
	}

}
