package com.i2msolucoes.alpha24j.layer.application;

import java.io.IOException;

import com.i2msolucoes.alpha24j.exception.COMMException;
import com.i2msolucoes.alpha24j.frame.request.Alpha2RequestFrame;
import com.i2msolucoes.alpha24j.frame.response.Alpha2ResponseFrame;
import com.i2msolucoes.alpha24j.frame.response.InvalidFrame;
import com.i2msolucoes.alpha24j.layer.physical.PhysicalLayer;
import com.serotonin.util.queue.ByteQueue;

public class ApplicationLayer {

	private PhysicalLayer phyLayer;

	public ApplicationLayer(PhysicalLayer phyLayer) {
		this.phyLayer = phyLayer;
	}

	public void init() throws COMMException {
		this.phyLayer.init();
	}

	public void terminate() throws COMMException {
		this.phyLayer.close();
	}

	public Alpha2ResponseFrame sendFrame(Alpha2RequestFrame frame)
			throws COMMException {
		Alpha2ResponseFrame response = null;
		try {
			byte[] received = phyLayer.sendAndReceive(frame.getFrame());
			ByteQueue queue = new ByteQueue(received);
			response = Alpha2ResponseFrame.buildResponseFrame(frame, queue);
			return response;
		} catch (IOException e) {
			throw new COMMException(e.getMessage());
		} catch (Exception e) {
			return emptyInvalidFrame(frame);
		}
	}

	private Alpha2ResponseFrame emptyInvalidFrame(Alpha2RequestFrame frame) {
		try {
			return new InvalidFrame(frame, new ByteQueue());
		} catch (Exception e) {
			return null;
		}
	}
}
