package com.i2msolucoes.alpha24j.test;

import gnu.io.SerialPort;

import java.util.ArrayList;
import java.util.List;

import com.i2msolucoes.alpha24j.DeviceLocator;
import com.i2msolucoes.alpha24j.DeviceValue;
import com.i2msolucoes.alpha24j.DeviceWrite;
import com.i2msolucoes.alpha24j.DeviceLocator.DeviceCodes;
import com.i2msolucoes.alpha24j.exception.COMMException;
import com.i2msolucoes.alpha24j.exception.ErrorMessageReceivedException;
import com.i2msolucoes.alpha24j.exception.InvalidFrameReceivedException;
import com.i2msolucoes.alpha24j.layer.user.Alpha2Master;
import com.i2msolucoes.alpha24j.util.PrintUtils;
import com.serotonin.io.serial.SerialParameters;

public class CompleteTest {
	private static Alpha2Master master;

	public static void main(String[] args) throws InterruptedException {
		SerialParameters parameters = new SerialParameters();
		parameters.setBaudRate(9600);
		parameters.setCommPortId("COM19");
		parameters.setParity(SerialPort.PARITY_NONE);
		parameters.setStopBits(SerialPort.STOPBITS_1);
		parameters.setDataBits(SerialPort.DATABITS_8);

		master = new Alpha2Master((byte) 0x00, parameters, 1000, 2);

		init();
		lineCheck();

		Thread.sleep(2000);
		// runController();
		//

		// write(new DeviceWrite[] { new DeviceWrite(new DeviceLocator(
		// DeviceCodes.COMM_WORD_DEVICE, 2), 1) });

		// List<DeviceValue> values = read(new DeviceLocator[] { new
		// DeviceLocator(
		// DeviceCodes.OUTPUT_TERMINAL, 1), });
		//
		// PrintUtils.printDeviceValues(values);

		for (int i = 0; i < 10; i++) {
			List<DeviceValue> values = read(new DeviceLocator[] { new DeviceLocator(
					DeviceCodes.ANALOG_IN, 5) });
			PrintUtils.printDeviceValues(values);
			System.out.println();
			Thread.sleep(1000);
		}

		// stopController();
		// write(new DeviceWrite[] { new DeviceWrite(new DeviceLocator(
		// DeviceCodes.EXTERNAL_OUTPUT, 1), 12) });

		terminate();
		Thread.sleep(3000);
		System.exit(0);

	}

	private static void init() {
		try {
			master.init();
		} catch (COMMException e1) {
			e1.printStackTrace();
			return;
		}
	}

	private static void lineCheck() {
		try {
			master.lineCheck();
		} catch (COMMException e) {
			e.printStackTrace();
		} catch (ErrorMessageReceivedException e) {
			e.printStackTrace();
		} catch (InvalidFrameReceivedException e) {
			e.printStackTrace();
		}
	}

	private static void write(DeviceWrite[] devices) {
		List<DeviceWrite> array = new ArrayList<DeviceWrite>();
		for (DeviceWrite deviceWrite : devices) {
			array.add(deviceWrite);
		}

		try {
			master.write(array);
		} catch (COMMException e) {
			e.printStackTrace();
		} catch (ErrorMessageReceivedException e) {
			e.printStackTrace();
		} catch (InvalidFrameReceivedException e) {
			e.printStackTrace();
		}
	}

	private static List<DeviceValue> read(DeviceLocator[] devices) {
		List<DeviceLocator> array = new ArrayList<DeviceLocator>();
		for (DeviceLocator deviceLocator : devices) {
			array.add(deviceLocator);
		}

		try {
			return master.read(array);
		} catch (COMMException e) {
			e.printStackTrace();
		} catch (ErrorMessageReceivedException e) {
			e.printStackTrace();
		} catch (InvalidFrameReceivedException e) {
			e.printStackTrace();
		}
		return new ArrayList<DeviceValue>();
	}

	private static void terminate() {
		try {
			master.terminate();
		} catch (COMMException e) {
			e.printStackTrace();
		}
	}

	private static void runController() {
		try {
			master.runController();
		} catch (COMMException e) {
			e.printStackTrace();
		} catch (ErrorMessageReceivedException e) {
			e.printStackTrace();
		} catch (InvalidFrameReceivedException e) {
			e.printStackTrace();
		}
	}

	private static void stopController() {
		try {
			master.stopController();
		} catch (COMMException e) {
			e.printStackTrace();
		} catch (ErrorMessageReceivedException e) {
			e.printStackTrace();
		} catch (InvalidFrameReceivedException e) {
			e.printStackTrace();
		}
	}

}
